/* $Id: cll.h,v 1.2 2000/03/02 16:17:26 lfm Exp $ */
/*
** --------------------------------------------------------------------
** cll.h - Public definitions and prototypes for module of primitive 
**         collection.
**
** Created:    12-Apr-94    Luiz F. Martha
**
** Modified:   12-Apr-94    Joao Luiz E. Campos (Acai)
**     Added CllPoint and CllCircle collectors type.
**
** Modified:   15-Jun-94    Joao Luiz E. Campos (Acai)
**     Added CllArc collectors type.
**
** Modified:   30-May-95    LFM
**     Version to use with CD graphics package.
**     Changed float to double.
**     Got rid of GraPnt, replacing it by CllPnt.  CllPoint was
**     redefined as CllPnt: structure containing two fields: x, y.
**     Changed prototype of snap and echo functions so that point 
**     structure is replaced by x and y coordinates.
**     Moved primitive type definitions to specific primitive 
**     header files.
**
** Modified:   08-Mar-96    LFM
**     When CD changed, creating cdCreateCanvas, which returns
**     a CD handle for the canvas.
**     Changed CllCreate adding a parameter which is the CD
**     canvas handle.
**     Created CLL_PLGON for polygon primitive.
**
** Modified:   28-Jan-97    LFM
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection function CllKbdCreate.
**
** Modified:   11-Apr-97    LFM
**     Added function CllShiftSnap.
**
** Modified:   17-Apr-97    LFM
**     Created macro definitions for end action return status.
**     Added return status parameter for end action function.
**
** Modified:   02-Mar-2000    LFM
**     Created collection of dimension lines.
**
** Modified:   18-Dec-2002    Marcos C. de Arruda
**     Created CLL_SPLINE for spline primitive. 
**
** Modified:   21-Apr-2004    William Lira
**     Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
** Modified:   11-Mar-2005    Marcos C. de Arruda
**     Created CLL_ELPARC for elipse arc primitive. 
**
** Modified:   18-Mar-2005    Marcos C. de Arruda
**     Created CLL_ELIPSE for elipse primitive. 
*/

#ifndef _CLL_H
#define _CLL_H

#include "iup.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public definitions:
*/

#define   CLL_NORMAL      1
#define   CLL_QUIT        0
#define   CLL_INTERUPTED -1

typedef enum _clltype {      /* Collector types:              */
  CLL_PNT,                      /* collector of point         */
  CLL_LINE,                     /* collector of straight line */
  CLL_PLINE,                    /* collector of polyline      */
  CLL_SPLINE,                   /* collector of spline        */    
  CLL_PLGON,                    /* collector of polygon       */
  CLL_CIRCLE,                   /* collector of circle        */
  CLL_ARC,                      /* collector of arc           */
  CLL_ELPARC,                   /* collector of elipse arc    */
  CLL_ELIPSE,                   /* collector of elipse        */
  CLL_BEZIER,                   /* collector of bezier        */
  CLL_DIMLINE,                  /* collector of dimension line*/
  NumCllTypes
 } CllType;

typedef struct _cllpnt {     /* Collected point              */
  double     x, y;              /* point coordinates         */
 } CllPnt;

/* Public functions:
*/
void CllCreate( CllType type,
                Ihandle *iup_cv, void *cd_cv,
                void (*end_action)( Ihandle *iup_cv, int status ),
                void (*put)( Ihandle *iup_cv, void *prmtv, double tol ),
                void (*prompt)( Ihandle *iup_cv, char *msg ),
                void (*snap)( Ihandle *iup_cv, double *x, double *y, double tol,
                              int shift, int control ),
                void (*echo)( Ihandle *iup_cv, double x, double y ) );
int  CllKbdCreate( CllType type, Ihandle *iup_cv, 
                   void (*put)( Ihandle *iup_cv, void *prmtv, double tol ),
                   void (*prompt)( Ihandle *iup_cv, char *msg ) );
void CllOpen( void *cllprmtv, void (*display)( void *cllprmtv ) );
void CllClose( void );
void CllDisplay( void );
void CllShiftSnap( double x0, double y0, double *x1, double *y1 );

#ifdef __cplusplus
}
#endif

#include "cllline.h"
#include "cllpline.h"
#include "cllspline.h"
#include "cllcircl.h"
#include "cllarc.h"
#include "cllelparc.h"
#include "cllelipse.h"
#include "cllbezie.h"

#endif
