/* $Id: cllbezie.h,v 1.1 1997/04/23 19:28:18 lfm Exp $ */
/*
** --------------------------------------------------------------------
** cllbezie.h - Prototypes for module of cubic bezier curve primitive 
**              collection.
**
** Created:    16-Jun-94    Joao Luiz E. Campos
**
** Modified:   15-Oct-96    LFM
**     Version to use with CD graphics package.
**
** Modified:   28-Jan-97    LFM
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection function CllKbdBezierCreate.
**
** Modified:   17-Apr-97    LFM
**     Added return status parameter for end action function.
**
** Modified:   21-Apr-2004    William Lira
**     Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

#ifndef _CLLBEZIE_H
#define _CLLBEZIE_H

#include "cll.h"   /* CllPnt */

#ifdef __cplusplus
extern "C" {
#endif

/* Public definitions:
*/
typedef struct _cllbezier {    /* Collected bezier:     */
  CllPnt     p0;                /* first point          */
  CllPnt     p1;                /* last point           */
  CllPnt     pc0;               /* first control point  */
  CllPnt     pc1;               /* second control point */
 } CllBezier;

/* Public functions:
*/
void CllBezierCreate( Ihandle *iup_cv, void *cd_cv,
                void (*end_action)( Ihandle *iup_cv, int status ),
                void (*put)( Ihandle *iup_cv, void *bezier, double tol ),
                void (*prompt)( Ihandle *iup_cv, char *msg ),
                void (*snap)( Ihandle *iup_cv, double *x, double *y, double tol,
                              int shift, int control ),
                void (*echo)( Ihandle *iup_cv, double x, double y ) );
int  CllKbdBezierCreate( Ihandle *iup_cv, 
                         void (*put)( Ihandle *iup_cv, void *bezier, double tol ),
                         void (*prompt)( Ihandle *iup_cv, char *msg ) );

#ifdef __cplusplus
}
#endif

#endif
