/* $Id: cllpline.h,v 1.1 1997/04/23 19:29:48 lfm Exp $ */
/*
** --------------------------------------------------------------------
** cllpline.h - Prototypes for module of polyline primitive 
**              collection.
**
** Created:    12-Apr-94    Luiz F. Martha
**
** Modified:   30-May-95    LFM
**     Version to use with CD graphics package.
**     Got rid of GraPnt, replacing it by CllPnt.  CllPoint was
**     redefined as CllPnt: structure containing two fields: x, y.
**     Replaced double by float.
**     Changed prototype of snap and echo functions so that point
**     structure is replaced by x and y coordinates.
**
** Modified:   07-Mar-96    LFM
**     When CD changed, creating cdCreateCanvas, which returns
**     a CD handle for the canvas.
**     Changed CllPLineCreate adding a parameter which is the CD
**     canvas handle.
**     Created function CllPlgonCreate.
**
** Modified:   28-Jan-97    LFM
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection functions CllKbdPLineCreate and 
**     CllKbdPlgonCreate.
**
** Modified:   17-Apr-97    LFM
**     Added return status parameter for end action function.
**
** Modified:   21-Apr-2004    William Lira
**     Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

#ifndef _CLLPLINE_H
#define _CLLPLINE_H

#include "cll.h"   /* CllPnt */

#ifdef __cplusplus
extern "C" {
#endif

/* Public definitions:
*/
typedef struct _cllpline {   /* Collected polyline:          */
  int        npts;              /* number of points in line  */
  CllPnt     *pts;              /* array of line points      */
 } CllPLine;

/* Public functions:
*/
void CllPLineCreate( Ihandle *iup_cv, void *cd_cv,
                void (*end_action)( Ihandle *iup_cv, int status ),
                void (*put)( Ihandle *iup_cv, void *pline, double tol ),
                void (*prompt)( Ihandle *iup_cv, char *msg ),
                void (*snap)( Ihandle *iup_cv, double *x, double *y, double tol,
                              int shift, int control ),
                void (*echo)( Ihandle *iup_cv, double x, double y ) );
int  CllKbdPLineCreate( Ihandle *iup_cv, 
                        void (*put)( Ihandle *iup_cv, void *pline, double tol ),
                        void (*prompt)( Ihandle *iup_cv, char *msg ) );

void CllPlgonCreate( Ihandle *iup_cv, void *cd_cv,
                void (*end_action)( Ihandle *iup_cv, int status ),
                void (*put)( Ihandle *iup_cv, void *pline, double tol ),
                void (*prompt)( Ihandle *iup_cv, char *msg ),
                void (*snap)( Ihandle *iup_cv, double *x, double *y, double tol,
                              int shift, int control ),
                void (*echo)( Ihandle *iup_cv, double x, double y ) );
int  CllKbdPlgonCreate( Ihandle *iup_cv, 
                        void (*put)( Ihandle *iup_cv, void *pline, double tol ),
                        void (*prompt)( Ihandle *iup_cv, char *msg ) );

#ifdef __cplusplus
}
#endif

#endif
