/* $Id: pck.h,v 1.3 1998/10/16 18:17:52 lfm Exp $ */
/*
** ---------------------------------------------------------------
** pck.h  -  Definitions and function prototypes of 
**           Pck: module to manage mouse events in a canvas
**           window.
**
** remarks:
** ---------------------------------------------------------------
** See description and functionality in header comments of file
** pck.c.
**
** ---------------------------------------------------------------
**
** Created:    29-May-95   Luiz Fernando Martha
**      Stolen from previous version that used the Gra package.
**      Created public function PckDefFunc.
**      Changed parameters of client functions echofunc and
**      snapfunc.  The parameters now are the pick point desc.,
**      and the x and y coordinates of the cursor point.
**      Replaced macro RetangleCursor by RectangleCursor.
**
** Modified:   23-Aug-95   LFM
**      Added field "context" in PckDesc struct.
**      Created VertLineCursor and HorizLineCursor cursor types.
**
** Modified:   29-Feb-96    LFM
**      When CD and G3d changed, creating cdCreateCanvas and 
**      G3dCreateVisSurf, which return a CD or G3d handle for the 
**      canvas.  Now to activate a canvas with CD or G3d one needs 
**      only its CD or G3d handle.  Before it was needed the IUP 
**      canvas handle and the CD or G3d driver context. Therefore, 
**      the driver context field in PckDesc structure was replaced
**      by the CD or G3d canvas handle.
**
** Modified:   07-Sep-96    LFM
**      Added fields p0_raw and p1_raw to PckDesc structure.
**
** Modified:   17-Apr-97    LFM
**      Created macro definitions for pick return status, adding
**      the interupted status.
**
** Modified:   27-Feb-98    LFM
**      Added x0dc, y0dc, x1dc, and y1dc fields to PckDesc structure.
**      Created function PckTolDir.
**      Created BallCursor type.
**
** Modified:   15-Oct-98    LFM
**      Created function PckDefCursor.
**
** Modified:   21-Apr-2004    William Lira
**     Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

#ifndef _PCK_H
#define _PCK_H

#include "iup.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public definitions:
 */

#define   PCK_VALID       1
#define   PCK_QUIT        0
#define   PCK_INTERUPTED -1

typedef struct _pckpnt {
 double     x, y;
 } PckPnt;

typedef struct _pckdesc {         /* Pick point descriptor:     */
 void     *iup_cv;                /* IUP handle of canvas  (I)  */
 void     *gra_cv;                /* CD/G3d hdle of canvas (I)  */
 PckPnt   p0;                     /* Initial locator point (I/O)*/
 PckPnt   p1;                     /* Final locator point   (O)  */
 PckPnt   p0_raw;                 /* p0 not snapped        (O)  */
 PckPnt   p1_raw;                 /* p1 not snapped        (O)  */
 int      x0dc, y0dc;             /* p0 in raster coords.  (O)  */
 int      x1dc, y1dc;             /* p1 in raster coords.  (O)  */
 int      mode;                   /* Call mode (bt up/dwn) (I)  */
 int      cursorecho;             /* Cursor echo type      (I)  */
 int      button;                 /* Button selected       (I/O)*/
 char     mousecfg[6];            /* Mouse configuration   (O)  */
 double   toler;                  /* Current pick tolerance(O)  */
                                  /* Bt-down callback func.(I)  */
 void     (*btdwn_callback)(struct _pckdesc *);
                                  /* Bt-up callback func.  (I)  */
 void     (*btup_callback)(struct _pckdesc *);
                                  /* Client echo function  (I)  */
 void     (*echofunc)(struct _pckdesc *, double x, double y);
                                  /* Client snap function  (I)  */
 void     (*snapfunc)(struct _pckdesc *, double *x, double *y,
                                         double tol);
 void     *data;                  /* Client data           (I)  */
 int      status;                 /* Pick status           (O)  */
 } PckDesc;

/* Types of pick callback mode:
 */
#define ButtonUp          0
#define ButtonDown        1

/* Definitions for cursor echo type:
 */
#define NoEchoCursor       0
#define CrossCursor        1
#define CrossHairCursor    2
#define BoxCursor          4
#define RubberBandCursor   8
#define RectangleCursor   16
#define VertLineCursor    32
#define HorizLineCursor   64
#define BallCursor       128

/* Public function prototypes:
 */
void PckDefFunc  ( void (*mousebt)( Ihandle *iup_cv, int b, int mode,
                                    int x, int y, char *mousecfg ),
                   void (*mousemv)( Ihandle *iup_cv, int x, int y,
                                    char *mousecfg ) );
void PckDefCursor( char *cursor );
void PckTolDir   ( char d );
void PckSetup    ( PckDesc *pk );
void PckMouseBt  ( Ihandle *iup_cv, int b, int mode, int x, int y,
                   char *mousecfg );
void PckMouseMv  ( Ihandle *iup_cv, int x, int y, char *mousecfg );
void PckClose    ( void );
void PckEchoOff  ( void );

#ifdef __cplusplus
}
#endif

#endif
