/* $Id: vis.h,v 1.2 1997/05/19 11:31:45 lfm Exp $ */
/*
** ---------------------------------------------------------------
** vis.h - Header file of 2D View Specification Package.  Generic 
**         visualization functions for use with CD graphics package 
**         and Iup graphics interface.
**
** ---------------------------------------------------------------
**
** Version:    2.0
**
** Created:    13-Jan-94    Luiz F. Martha
**
** Modified:   11-Apr-94    Joao Luiz E. Campos (Acai)
**   Created function VisPrevZoom.
**
** Modified:   01-Sep-94    Joao Luiz E. Campos (Acai)
**   Created function VisAdjustWindow.
**
** Modified:   29-May-95    LFM
**   Version 2.0:  Version to use with CD graphics package.
**   Added width and height parameters in function VisResize.
**   Changed float to double.
**   Eliminated function VisFitWindow.
**   Renamed function VisAdjustWindow to VisFitLimits.
**
** Modified:   24-Jan-96    LFM
**   Created functions VisZoonIn and VisZoomOut.
**
** Modified:   07-Mar-96    LFM
**   When CD changed, creating cdCreateCanvas, which returns
**   a CD handle for the canvas.
**   Changed VisOpen adding a second parameter which is the CD
**   canvas handle.
**
** Modified:   10-Apr-96    LFM
**   Added functions VisScaleFac, VisSetLimits, VisGetLimits,
**   VisSetSizes, VisSetWidth, VisSetHeight, and VisGetSizes.
**
** Modified:   17-Aug-96    LFM
**   Added four window parameters to client redisplay callback function.
**   Got rid of public function VisLimits.
**   Renamed VisSetLimits to VisSetWindow.
**   Renamed VisGetLimits to VisGetWindow.
**   Replaced public function VisDistort by VisDistortFac.
**   Created function VisGetDistortFac.
**
** Modified:   04-Apr-97    LFM
**   Created function VisDefault.
**
** Modified:   17-Apr-97    LFM
**     Created macro definitions for zoom and scale end action 
**     return status.  Added return status parameter for end action 
**     function.
**
** Modified:   16-May-97    LFM
**     Eliminated functions VisReset and VisFitLimits.
**     Clients should call VisFit instead of VisReset and
**     VisDefault instead of VisFitLimits.
**     Created functions VisPan, VisPanX, and VisPanY.
**
** Modified:   21-Apr-2004    William Lira
**     Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

#ifndef _VIS_H
#define _VIS_H

#include "iup.h"

#ifdef __cplusplus
extern "C" {
#endif

#define   VIS_NORMAL      1
#define   VIS_QUIT        0
#define   VIS_INTERUPTED -1

int  VisOpen         ( Ihandle *iup_cv, void *cd_cv );
void VisClose        ( Ihandle *iup_cv );
void VisRegFunc      ( Ihandle *iup_cv, 
                       void (*rdsp)(Ihandle *iup_cv, double xmn, double xmx,
                                                  double ymn, double ymx),
                       int  (*fmnmx)(Ihandle *iup_cv, double *xmn,double *xmx,
                                                   double *ymn,double *ymx) );
void VisDefault      ( Ihandle *iup_cv );
void VisFit          ( Ihandle *iup_cv );
void VisResize       ( Ihandle *iup_cv, int width, int height );
void VisRedraw       ( Ihandle *iup_cv, double posx, double posy );
void VisRedisplay    ( Ihandle *iup_cv );
void VisZoom         ( Ihandle *iup_cv, 
                       void (*end_action)( Ihandle *iup_cv, int status ) );
void VisZoomIn       ( Ihandle *iup_cv );
void VisZoomOut      ( Ihandle *iup_cv );
void VisPrevZoom     ( Ihandle *iup_cv );
void VisPan          ( Ihandle *iup_cv, 
                       void (*end_action)( Ihandle *iup_cv, int status ) );
void VisPanX         ( Ihandle *iup_cv, double fac );
void VisPanY         ( Ihandle *iup_cv, double fac );
void VisScale        ( Ihandle *iup_cv, 
                       void (*end_action)( Ihandle *iup_cv, int status ) );
void VisScaleFac     ( Ihandle *iup_cv, double fac );
void VisDistortFac   ( Ihandle *iup_cv, double vscalefac );
void VisGetDistortFac( Ihandle *iup_cv, double *vscalefac );
void VisSetWindow    ( Ihandle *iup_cv, double xmin, double xmax,
                                     double ymin, double ymax );
void VisGetWindow    ( Ihandle *iup_cv, double *xmin, double *xmax,
                                     double *ymin, double *ymax );
void VisSetSizes     ( Ihandle *iup_cv, double width, double height );
void VisSetWidth     ( Ihandle *iup_cv, double width, double *height );
void VisSetHeight    ( Ihandle *iup_cv, double *width, double height );
void VisGetSizes     ( Ihandle *iup_cv, double *width, double *height );

#ifdef __cplusplus
}
#endif

#endif
