/* $Id: wim.h,v 1.4 1999/03/29 15:08:08 lfm Exp $ */
/*
** wim.h - Definitions and prototypes of world (coords) image package.
**         Functions to manage an image for applications 
**         that work with WD/CD world coordinates.
**
** ---------------------------------------------------------------
** Remarks:
** *  The module uses the IM package to treat different image 
**    file formats.
** *  It works with only one image at a time.
** *  It considers two image modes: free from world coordinate (WC)
**    space and glued to a WC space.
** *  When the image is not glued to a WC space, it is always 
**    displayed with it original sizes and centered in the canvas.
** *  When the image is glued to a WC space, it is displayed 
**    according to the window that was defined at the moment 
**    that it was glued.  This means that the image may be 
**    reduced or expanded.
** *  The redifinition of a WC scale (zoom in/out) might take 
**    some time becase the scaled image has to be recreated in
**    memory.
** *  Once a WC scale is defined to a glued image, the image can
**    be translated (pan) very efficiently because the scaled 
**    image is already created in memory.  
** *  It assumes that the viewport defined in the WD package
**    always takes the entire canvas.
**
** ---------------------------------------------------------------
*/

#ifndef _WIM_H
#define _WIM_H

/* Error Codes */
enum    {
        WIM_ERR_NONE,    /* No error */
        WIM_ERR_OPEN,    /* Error while opening the file */
        WIM_ERR_READ,    /* Error while reading the file */
        WIM_ERR_WRITE,   /* Error while writing the file */
        WIM_ERR_FORMAT,  /* Invalid format */
        WIM_ERR_TYPE,    /* Invalid image type */
        WIM_ERR_COMP,    /* Compression not supported */
        WIM_ERR_NOIMAGE, /* No image defined */
        WIM_ERR_NOTGLUED,/* Image is not glued */
        WIM_ERR_NOMEMORY /* Not enough memory */
       };

int  wimRead( char *filename );
int  wimRebuild( void );
int  wimGlueToCurrent( void );
int  wimGlueToFence( int vxmin, int vxmax, int vymin, int vymax, 
                     double wxmin, double wxmax, double wymin, double wymax );
int  wimGlueToBoundBox( double wxmin, double wxmax, 
                        double wymin, double wymax );
int  wimUnglue( void );
int  wimGetRarterSizes( int *iw, int *ih );
int  wimGetBoundBox( double *wxmin, double *wxmax, 
                     double *wymin, double *wymax );
int  wimDisplay( void );
int  wimVisibility( int on_off );
int  wimFree( void );
int  wimCheck( void );

#endif
