char *RCScllcircl="$Id: cllcircl.c,v 1.8 2001/09/13 14:34:48 william Exp $";
/*
** cllcircl.c - Functions to collect circles.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllCircleCreate( Ihandle *iup_cv, void *cd_cv,
**               void (*end_action)( Ihandle *iup_cv, int status ),
**               void (*put)( Ihandle *iup_cv, void *circl, double tol ), 
**               void (*prompt)( Ihandle *iup_cv, char *msg ), 
**               void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                             double tol, int shift, int control ),
**               void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - funct. to be called after finishing circle creat. (IN)
** put        - ptr. to function to put circle after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a circle
** on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something
**     else (not by the mouse interaction).
**
** --------------------------------------------------------------------
**
** int  CllKbdCircleCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *circl, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a circle.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    25-Mar-94    Luiz F. Martha and Joao Luiz E.C. (Acai)
**
** Modified:   01-Abr-94    Acai
**   When created Pck package: change old Gra mouse input (pick)
**   calls to Pck calls.
**
** Modified:   11-Apr-94    LFM
**     Eliminated "reinit" parameter in CllCircleCreate and added
**     "end_action" parameter.
**
** Modified:   13-Oct-96    LFM
**     Due to change in CD package, creating cdCreateCanvas, which 
**     returns a CD handle for the canvas.  Changed CllCircleCreate 
**     adding a parameter which is the CD canvas handle.
**     Modified local snap function such that it calls client
**     snap function only if control key is pressed.  Before
**     this was done in Pck module.
**     Changed such that nothing is done when end_action function
**     in null.  Before the collector reinitialized itself.
**
** Modified:   28-Jan-97    LFM 
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection function CllKbdCircleCreate.
**
** Modified:   08-Mar-97    LFM
**     Changed interaction such that circle collection is performed
**     in two (button down) picks.
**     Added radius echo during collection.
**
** Modified:   09-Apr-97    LFM
**     Added call to CllClose in function CllCircleCreate to close
**     any previous data structure in Cll super-class.
**
** Modified:   11-Apr-97    LFM
**     Added call to CllShiftSnap.
**
** Modified:   17-Apr-97    LFM
**      Added return status parameter for end action function.
**
** Modified:   23-Apr-97    LFM
**      Considered environment variable IUP_INGLES for prompt and
**      dialog messages.
**
** Modified:   18-Feb-98    LFM
**      Modification to avoid displaying echo messages on the canvas.
**      Instead, echo messages are send to prompt message UI object.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   10-Sep-2001  William
**      Substitution of the calls the IupScanf( ) for dialogs defined in the 
**      dlgcircl module.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "cll.h"
#include "cllcircl.h"
#include "dlgcircl.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Circle data structure:
 */
typedef struct _cllcirclep {  /* Collected circle private spec.descrip.: */
        CllCircle  c;               /* collected circle              */
        int        ctr;             /* if true, center already given */
        void       *iup_cv;         /* IUP handle to target canvas   */
        void       *cd_cv;          /* cd handle to corresp.canvas   */
        void       (*end_action)    /* end action callback           */
                       ( Ihandle *iup_cv, int status );
        void       (*put)           /* function to put collect.circle*/
                       ( Ihandle *iup_cv, void *circle, double tol );
        void       (*prompt)        /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void       (*snap)          /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void       (*echo)          /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y );
        } CllCircleP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;

static char *prompt_msg1p = { "Entre o centro com botao esquerdo do mouse; "
                              "Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com um ponto no circulo; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "Enter center with mouse left button; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter a point on the circle; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* ==========================  CllCircleDel  ======================== */

static void CllCircleDel( CllCircleP *circle )
{
 if( circle == 0L ) return;

/*  Release memory used for circle
 */    
 free( circle );
}


/* ========================  CllCircleSnap  ========================= */

static void CllCircleSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 CllCircleP  *circle = (CllCircleP *)pk->data;

/* If shift key is pressed and first line point has been given,
 * snap line to the horizontal axis, to the vertical axis, or
 * the bi-sector lines.
 */
 if( (circle->ctr) && isshift(pk->mousecfg) )
 {
  CllShiftSnap( circle->c.center.x, circle->c.center.y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( circle->snap != 0L )
  (*circle->snap )( circle->iup_cv, x, y, tol,
                              isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}


/* ========================  CllCircleEcho  ========================= */
 
static void CllCircleEcho( PckDesc *pk, double x, double y )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;
 PckPnt p1;
 double r, d;
 char   echotext[40];

/* Restore registered collection spec. from the data field.
 */
 CllCircleP  *circle = (CllCircleP *)pk->data;
 
/* Check to see if center has been given.
 * If that is the case, draw a circle in Xor mode.
 */
 if( circle->ctr )
 {
  p1.x = x;
  p1.y = y;
  r = LEN( pk->p0, p1 );
  d = 2.0 * r;

#ifdef _UNIX_
  write_mode = cdWriteMode( CD_XOR );
#else
  write_mode = cdWriteMode( CD_NOT_XOR );
#endif
  fore_color = cdForeground( CD_RED );
  mark_type = cdMarkType( CD_BOX );
  mark_size = cdMarkSize( 4 );
  line_style = cdLineStyle( CD_CONTINUOUS );
  line_width = cdLineWidth( 1 );

  wdMark( pk->p0.x, pk->p0.y );
  wdArc( pk->p0.x, pk->p0.y, d, d, 0.0, 360.0 );

  if( iup_english )
   sprintf( echotext, "Radius = %10.3f", r );
  else
   sprintf( echotext, "Raio = %10.3f", r );
  if( circle->prompt != 0L )
   (*circle->prompt)( circle->iup_cv, echotext );

  cdLineWidth( line_width );
  cdLineStyle( line_style );
  cdMarkSize( mark_size );
  cdMarkType( mark_type );
  cdForeground( fore_color );
  cdWriteMode( write_mode );
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( circle->echo != 0L )
  (*circle->echo )( circle->iup_cv, x, y ) ;
}
 

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ========================  CllCircleCreate  ======================= */

static void CllCircleCenter( PckDesc * );
static void CllCirclePoint( PckDesc * );

void CllCircleCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *circle, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllCircleP  *circle;
 PckDesc     pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
 }

 CllClose( );

/* Allocate a circle collection descriptor
 */
 circle = (CllCircleP *)malloc( sizeof( CllCircleP ) );

/* Initialize collection
 */
 circle->ctr = 0;
 circle->iup_cv = iup_cv;
 circle->cd_cv = cd_cv;
 circle->end_action = end_action; 
 circle->put = put; 
 circle->prompt = prompt; 
 circle->snap = snap; 
 circle->echo = echo; 
 circle->c.center.x = circle->c.center.y = 0.0;
 circle->c.p0.x = circle->c.p0.y = 0.0;
 circle->c.radius = 0.0;

/* Setup pick descriptor.  Register the appropriate circle collector
 * function as the callback for pick interaction.
 */
 if( circle->prompt != 0L )
  (*circle->prompt )( circle->iup_cv, prompt_msg1 );
 pk.iup_cv = circle->iup_cv;
 pk.gra_cv = circle->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = CllCircleCenter;
 pk.btup_callback = 0L;
 pk.echofunc = CllCircleEcho;
 pk.snapfunc = CllCircleSnap;
 pk.data = circle;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* --------------------------  CllCircleCenter  ----------------------- */

static void CllCircleCenter( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllCircleP  *circle = (CllCircleP *)pk->data;
 void      *iup_cv = circle->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = circle->end_action;

/* Check to see if pick is valid.  If not, delete collection data
 * call end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_QUIT;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllCircleDel( circle );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as center point.
 */
 circle->c.center.x = pk->p0.x;
 circle->c.center.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * circle using the keyboard. In this case, prompt the user for
 * center coordinates and return.
 */
 if( pk->button == 2 )
 {
  float xc, yc;
  xc = circle->c.center.x;
  yc = circle->c.center.y;
  if( DlgCaptureFCircle( &xc, &yc ) )
  {
   circle->c.center.x = xc;
   circle->c.center.y = yc;
  }
  else
  {
   CllCircleDel( circle );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that center has been selected.
 * This is used in the collection function to draw the circle.
 */
 circle->ctr = 1;

/* Get pick from user for circle point.
 */
 if( circle->prompt != NULL )
  (*circle->prompt )( iup_cv, prompt_msg2 ) ;
 pk->button = 0;       /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllCirclePoint;
 PckSetup( pk );
}

/* --------------------------  CllCirclePoint  ------------------------ */

static void CllCirclePoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllCircleP  *circle = (CllCircleP *)pk->data;
 void      *iup_cv = circle->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = circle->end_action;

/* Get button down point as circle point and compute circle radius.
 */
 circle->c.p0.x = pk->p0.x;
 circle->c.p0.y = pk->p0.y;
 circle->c.radius = LEN(circle->c.center,circle->c.p0);

/* If middle button was pressed, the user wants to enter the
 * circle using the keyboard. In this case, prompt the user for
 * center coordinates and radius, and return.
 */
 if( (pk->status == PCK_VALID) && (pk->button == 2) )
 {
  float xc, yc, r;
  xc = circle->c.center.x;
  yc = circle->c.center.y;
  r = circle->c.radius;
  if( DlgCaptureSCircle( &xc, &yc, &r ) )
  {
   circle->c.center.x = xc;
   circle->c.center.y = yc;
   circle->c.radius = r;
   circle->c.p0.x = circle->c.center.x + circle->c.radius;
   circle->c.p0.y = circle->c.center.y;
   if( circle->c.radius > 0.0 )
   {
    if( circle->put != 0L )
     (*circle->put )( circle->iup_cv, &circle->c, pk->toler );
   }
   CllCircleDel( circle );
   if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
  }
  else
  {
   CllCircleDel( circle );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
  }
  return;
 }

/* Check to see if collected circle is valid (greater then tolerance).
 * If not valid, just quit the collection, calling end action function.
 */
 if( (pk->status != PCK_VALID) || (circle->c.radius < pk->toler) )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_VALID:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllCircleDel( circle );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Call the client function to put the collected circle, 
 * and finish collection calling end action function.
 */
 if( circle->put != 0L )
  (*circle->put )( iup_cv, &circle->c, pk->toler );
 CllCircleDel( circle );
 if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
}


/* =======================  CllKbdCircleCreate  ===================== */

static CllCircle  kbdcircle;
static double     kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados do circulo. " };
static char *kbdprompt_msge = { "Use the keyboard to enter circle data. " };

int  CllKbdCircleCreate( Ihandle *iup_cv,
              void (*put)( Ihandle *iup_cv, void *circl, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllCircle   circle;
 float       xc, yc, r;
 float       toler;
 int         status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 circle = kbdcircle;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 xc = circle.center.x;
 yc = circle.center.y;
 r = circle.radius;
 toler = kbdtol;
 if( DlgCaptureCircle( &xc, &yc, &r, &toler ) )
 {
  if( r > 0.0 )
  {
   circle.center.x = xc;
   circle.center.y = yc;
   circle.radius = r;
   circle.p0.x = circle.center.x + circle.radius;
   circle.p0.y = circle.center.y;
   kbdcircle = circle;
   kbdtol = toler;
   if( put != 0L )
    (*put)( iup_cv, &circle, kbdtol );
   status = CLL_NORMAL;
  }
 }

 return( status );
}


