char *RCSclldimln="$Id: clldimln.c,v 1.4 2001/05/17 12:07:57 lfm Exp $";
/*
** clldimln.c - Functions to collect dimension lines.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllDimLnCreate( Ihandle *iup_cv, void *cd_cv,
**               void (*end_action)( Ihandle *iup_cv, int status ),
**               void (*put)( Ihandle *iup_cv, void *dimln, double tol ), 
**               void (*prompt)( Ihandle *iup_cv, char *msg ), 
**               void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                             double tol, int shift, int control ),
**               void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - func.to be called after finishing dim.ln. creation(IN)
** put        - ptr. to function to put dim.line after collection (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a dimension 
** line on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something 
**     else (not by the mouse interaction). 
**
** --------------------------------------------------------------------
**
** int  CllKbdDimLnCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *dimln, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
** 
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a dimension 
** line.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    02-Mar-2000  Luiz F. Martha
**      Stolen from cllarc.c and modified.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Fixed keyboard entry mistake in function CllDimLnRefPoint:
**      wrong assignment of first point coordinate after collecting
**      from user.
**      Modified IupScanf floating point format.
**
** Modified:   08-Aug-2002    Marcos C. de Arruda
**      Replaced call to IupScanf by call to corresponding dialog
**      (Dlg) function.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "cll.h"
#include "clldimln.h"
#include "dlgdimln.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Dimension line collection data structure:
 */
typedef struct _clldimlnp  {   /* Collected dim.line private spec.descrip.: */
        CllDimLn l;                 /* collected dimension line      */
        int      fst;               /* if true, first line pt. given */
        int      snd;               /* if true, second line pt. given*/
        void     *iup_cv;           /* IUP handle to target canvas   */
        void     *cd_cv;            /* cd handle to corresp.canvas   */
        void     (*end_action)      /* end action callback           */
                       ( Ihandle *iup_cv, int status );
        void     (*put)             /* function to put collect prtv. */
                       ( Ihandle *iup_cv, void *dimln, double tol );
        void     (*prompt)          /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void     (*snap)            /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void     (*echo)            /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y );
        } CllDimLnP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;
static char *prompt_msg3;

static char *prompt_msg1p = { "Com botao esquerdo do mouse entre "
                              "com o primeiro ponto; "
                              "Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com o segundo ponto da linha; "
                              "Botao do meio => Teclado." };
static char *prompt_msg3p = { "Posicione a linha com um ponto de ref.; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "With mouse left button enter first line pt.; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter line second point; "
                              "Mid button => Keyboard." };
static char *prompt_msg3e = { "Enter line reference point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };


/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  CllDimLnOffsetPts  ===================== */

static int CllDimLnOffsetPts( double x0, double y0, double x1, double y1, 
                              double xref, double yref, double *xoff0, 
                              double *yoff0, double *xoff1, double *yoff1 )
{
 double   len, vx, vy, t;
 double   xclst, yclst;

 len = sqrt( (x1-x0) * (x1-x0) + (y1-y0) * (y1-y0) );
 if( len == 0.0 )
 {
  *xoff0 = x0;
  *yoff0 = y0;
  *xoff1 = x1;
  *yoff1 = y1;
  return( 0 );
 }

 vx = (x1-x0) / len;
 vy = (y1-y0) / len;

 t = (xref-x0)*vx + (yref-y0)*vy;

 xclst = x0 + (t * vx);
 yclst = y0 + (t * vy);

 len = sqrt( (xref-xclst) * (xref-xclst) + (yref-yclst) * (yref-yclst) );
 if( len == 0.0 )
 {
  *xoff0 = x0;
  *yoff0 = y0;
  *xoff1 = x1;
  *yoff1 = y1;
  return( 0 );
 }

 vx = (xref-xclst) / len;
 vy = (yref-yclst) / len;

 *xoff0 = x0 + (len * vx);
 *yoff0 = y0 + (len * vy);
 *xoff1 = x1 + (len * vx);
 *yoff1 = y1 + (len * vy);

 return( 1 );
}

/* ========================  CllDimLnRotPts  ======================= */

static int CllDimLnRotPts( double dx, double dy, int npts, CllPnt *pts )
{
 double len;
 double c, s;
 double t;
 int    i;

 if( !dx && !dy )
  return( 0 );

 len = sqrt( dx*dx + dy*dy );
 c = dx / len;
 s = dy / len;
   
 for( i = 0; i < npts; i++ )
 {
  t        = (pts[i].x)*( c) + (pts[i].y)*(-s);
  pts[i].y = (pts[i].x)*( s) + (pts[i].y)*( c);
  pts[i].x = t;
 }

 return( 1 );
}

/* ========================  CllDimLnDisplay  ====================== */

static void CllDimLnDisplay( CllDimLn *l, double sizeshift, double sizearrow )
{
 CllPnt shift;
 CllPnt arrowleft[2];
 CllPnt arrowright[2];

 shift.x = sizeshift;
 shift.y = 0.0;
 arrowleft[0].x  =  sizearrow;
 arrowleft[0].y  =  sizearrow * 0.25;
 arrowleft[1].x  =  sizearrow;
 arrowleft[1].y  = -sizearrow * 0.25;
 arrowright[0].x = -sizearrow;
 arrowright[0].y =  sizearrow * 0.25;
 arrowright[1].x = -sizearrow;
 arrowright[1].y = -sizearrow * 0.25;

 if( ! CllDimLnRotPts( l->p0off.x - l->p0.x,  l->p0off.y - l->p0.y,
                       1, &shift ) )
  return;

 if( ! CllDimLnRotPts( l->p1.x - l->p0.x,  l->p1.y - l->p0.y,
                       2, arrowleft ) )
  return;

 if( ! CllDimLnRotPts( l->p1.x - l->p0.x,  l->p1.y - l->p0.y,
                       2, arrowright ) )
  return;

 wdLine( l->p0.x    + shift.x, l->p0.y    + shift.y, 
         l->p0off.x + shift.x, l->p0off.y + shift.y );
 wdLine( l->p1.x    + shift.x, l->p1.y    + shift.y, 
         l->p1off.x + shift.x, l->p1off.y + shift.y );
 wdLine( l->p0off.x, l->p0off.y, l->p1off.x, l->p1off.y );
 wdLine( l->p0off.x + arrowleft[0].x,  l->p0off.y + arrowleft[0].y, 
         l->p0off.x,                   l->p0off.y );
 wdLine( l->p0off.x,                   l->p0off.y, 
         l->p0off.x + arrowleft[1].x,  l->p0off.y + arrowleft[1].y );
 wdLine( l->p1off.x + arrowright[0].x, l->p1off.y + arrowright[0].y, 
         l->p1off.x,                   l->p1off.y );
 wdLine( l->p1off.x,                   l->p1off.y, 
         l->p1off.x + arrowright[1].x, l->p1off.y + arrowright[1].y );
}

/* ==========================  CllDimLnDel  ======================== */

static void CllDimLnDel( CllDimLnP *dimln )
{
 if( dimln == 0L ) return;

/*  Release memory used for dimension line.
 */    
 free( dimln );
}

/* ========================  CllDimLnSnap  ========================= */

static void CllDimLnSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 CllDimLnP  *dimln = (CllDimLnP *)pk->data;

/* If shift key is pressed and first line point has been given,
 * snap line to the horizontal axis, to the vertical axis, or
 * to the bi-sector lines.
 */
 if( (dimln->fst) && isshift(pk->mousecfg) )
 {
  CllShiftSnap( dimln->l.p0.x, dimln->l.p0.y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( dimln->snap != 0L )
  (*dimln->snap )( dimln->iup_cv, x, y, tol, 
                              isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}

/* ========================  CllDimLnEcho  ========================= */
 
static void CllDimLnEcho( PckDesc *pk, double x, double y )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;
 double len;
 double d;
 CllPnt pt;
 char   echotext[60];

/* Restore registered collection spec. from the data field.
 */
 CllDimLnP  *dimln = (CllDimLnP *)pk->data;
 
/* Check to see if first line point has been given.
 * If that is the case, draw a mark at this point in Xor mode
 * and echo line length.
 */
 if( dimln->fst )
 {
#ifdef _UNIX_
  write_mode = cdWriteMode( CD_XOR );
#else
  write_mode = cdWriteMode( CD_NOT_XOR );
#endif
  fore_color = cdForeground( CD_RED );
  mark_type = cdMarkType( CD_BOX );
  mark_size = cdMarkSize( 4 );

  wdMark( dimln->l.p0.x, dimln->l.p0.y );

  if( !dimln->snd )
  {
   pt.x = x;  pt.y = y;
   len = LEN( dimln->l.p0, pt );
   if( iup_english )
    sprintf( echotext, "Length = %10.3f", len );
   else
    sprintf( echotext, "Comprimento = %10.3f", len );
   if( dimln->prompt != 0L )
    (*dimln->prompt)( dimln->iup_cv, echotext );
  }

  cdMarkSize( mark_size );
  cdMarkType( mark_type );
  cdForeground( fore_color );
  cdWriteMode( write_mode );
 }

/* Check to see if second point has been given.
 * If that is the case, get and draw offset dimension line,
 * and draw extension lines, all in Xor mode.
 */
 if( dimln->snd )
 {
  len = LEN( dimln->l.p0, dimln->l.p1 );
  d = LEN( dimln->l.p0, dimln->l.p0off );

  dimln->l.ref.x = x;
  dimln->l.ref.y = y;
  CllDimLnOffsetPts( dimln->l.p0.x, dimln->l.p0.y, dimln->l.p1.x, 
                     dimln->l.p1.y, dimln->l.ref.x, dimln->l.ref.y, 
                     &dimln->l.p0off.x, &dimln->l.p0off.y, 
                     &dimln->l.p1off.x, &dimln->l.p1off.y );
#ifdef _UNIX_
  write_mode = cdWriteMode( CD_XOR );
#else
  write_mode = cdWriteMode( CD_NOT_XOR );
#endif
  fore_color = cdForeground( CD_RED );
  mark_type = cdMarkType( CD_BOX );
  mark_size = cdMarkSize( 4 );
  line_style = cdLineStyle( CD_CONTINUOUS );
  line_width = cdLineWidth( 1 );

  wdMark( dimln->l.p1.x, dimln->l.p1.y );
  CllDimLnDisplay( &dimln->l, pk->toler, 2.0 * pk->toler );
  if( iup_english )
   sprintf( echotext, "Length = %10.3f;  Extension = %10.3f", len, d );
  else
   sprintf( echotext, "Comprimento = %10.3f;  Extenso = %10.3f", len, d );
  if( dimln->prompt != 0L )
   (*dimln->prompt)( dimln->iup_cv, echotext );

  cdLineWidth( line_width );
  cdLineStyle( line_style );
  cdMarkSize( mark_size );
  cdMarkType( mark_type );
  cdForeground( fore_color );
  cdWriteMode( write_mode );
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( dimln->echo != 0L )
  (*dimln->echo )( dimln->iup_cv, x, y ) ;
}
 

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ========================  CllDimLnCreate  ======================= */

static void CllDimLn1stPoint( PckDesc * );

void CllDimLnCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *dimln, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllDimLnP  *dimln;
 PckDesc    pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
  prompt_msg3 = prompt_msg3e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
  prompt_msg3 = prompt_msg3p;
 }

 CllClose( );

/* Allocate a dimension line collection descriptor
 */
 dimln = (CllDimLnP *)malloc( sizeof( CllDimLnP ) );

/* Initialize collection
 */
 dimln->fst = 0;
 dimln->snd = 0;
 dimln->iup_cv = iup_cv;
 dimln->cd_cv = cd_cv;
 dimln->end_action = end_action; 
 dimln->put = put; 
 dimln->prompt = prompt; 
 dimln->snap = snap; 
 dimln->echo = echo; 
 dimln->l.p0.x = dimln->l.p0.y = 0.0;
 dimln->l.p1.x = dimln->l.p1.y = 0.0;
 dimln->l.p0off.x = dimln->l.p0off.y = 0.0;
 dimln->l.p1off.x = dimln->l.p1off.y = 0.0;
 dimln->l.ref.x = dimln->l.ref.y = 0.0;

/* Setup pick descriptor for getting first point at button down.  
 */
 if( dimln->prompt != 0L )
  (*dimln->prompt )( dimln->iup_cv, prompt_msg1 );
 pk.iup_cv = dimln->iup_cv;
 pk.gra_cv = dimln->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback  = CllDimLn1stPoint;
 pk.btup_callback = 0L;
 pk.echofunc = CllDimLnEcho;
 pk.snapfunc = CllDimLnSnap;
 pk.data = dimln;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* ------------------------  CllDimLn1stPoint  ----------------------- */

static void CllDimLn2ndPoint( PckDesc * );

static void CllDimLn1stPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllDimLnP  *dimln = (CllDimLnP *)pk->data;
 void       *iup_cv = dimln->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = dimln->end_action;

/* Check to see if collected data is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_QUIT;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllDimLnDel( dimln );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Set button down point as first point.
 */
 dimln->l.p0.x = pk->p0.x;
 dimln->l.p0.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter 
 * first point using the keyboard. In this case, prompt the user for
 * center coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  float xc, yc;
  xc = dimln->l.p0.x;
  yc = dimln->l.p0.y;
  if( DlgCaptureFDimLn( &xc, &yc ) == 1 )
  {
   dimln->l.p0.x = xc;
   dimln->l.p0.y = yc;
  }
  else
  {
   CllDimLnDel( dimln );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that the first point has been selected.
 */
 dimln->fst = 1;

/* Setup pick descriptor to collect second point.  Register the appropriate 
 * dim. line collector function as the callback for pick interaction.
 */
 if( dimln->prompt != 0L )
  (*dimln->prompt )( dimln->iup_cv, prompt_msg2 );
 pk->p0.x = dimln->l.p0.x;
 pk->p0.y = dimln->l.p0.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor | RubberBandCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllDimLn2ndPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllDimLn2ndPoint  ---------------------- */

static void CllDimLnRefPoint( PckDesc * );

static void CllDimLn2ndPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllDimLnP  *dimln = (CllDimLnP *)pk->data;
 void       *iup_cv = dimln->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = dimln->end_action;

/* Check to see if collected data is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllDimLnDel( dimln );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Set button down point as second line point.
 */
 dimln->l.p1.x = pk->p0.x;
 dimln->l.p1.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter second
 * point using the keyboard. In this case, prompt the user for
 * point coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  float x0, y0;
  x0 = dimln->l.p1.x;
  y0 = dimln->l.p1.y;
  if( DlgCaptureSDimLn( &x0, &y0 ) == 1 )
  {
   dimln->l.p1.x = x0;
   dimln->l.p1.y = y0;
  }
  else
  {
   CllDimLnDel( dimln );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that the second point has been selected.
 */
 dimln->snd = 1;

/* Setup pick descriptor to collect reference point (where offset
 * line is goint to pass).  Register the appropriate dimension 
 * line collector function as the callback for pick interaction.
 */
 if( dimln->prompt != 0L )
  (*dimln->prompt )( dimln->iup_cv, prompt_msg3 );
 pk->p0.x = dimln->l.p0.x;
 pk->p0.y = dimln->l.p0.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllDimLnRefPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllDimLnRefPoint  ---------------------- */

static void CllDimLnRefPoint( PckDesc *pk )
{
 int status;

/* Restore registered collection spec. from the data field.
 */
 CllDimLnP  *dimln = (CllDimLnP *)pk->data;
 void       *iup_cv = dimln->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = dimln->end_action;

/* Check to see if collected data is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllDimLnDel( dimln );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Set button down point reference point.
 */
 dimln->l.ref.x = pk->p0.x;
 dimln->l.ref.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter dimension
 * line points using the keyboard. In this case, prompt the user for
 * the point coordinates and finish collection.
 */
 if( pk->button == 2 )
 {
  float xc, yc, x0, y0, x1, y1;
  xc = dimln->l.p0.x;
  yc = dimln->l.p0.y;
  x0 = dimln->l.p1.x;
  y0 = dimln->l.p1.y;
  x1 = dimln->l.ref.x;
  y1 = dimln->l.ref.y;
  if( DlgCaptureTDimLn( &xc, &yc, &x0, &y0, &x1, &y1 ) == 1 )
  {
   dimln->l.p0.x = xc;
   dimln->l.p0.y = yc;
   dimln->l.p1.x = x0;
   dimln->l.p1.y = y0;
   dimln->l.ref.x = x1;
   dimln->l.ref.y = y1;

   /* Compute coordinates of offset point (dimension line).
    */
   status = CllDimLnOffsetPts( dimln->l.p0.x, dimln->l.p0.y, dimln->l.p1.x, 
                     dimln->l.p1.y, dimln->l.ref.x, dimln->l.ref.y, 
                     &dimln->l.p0off.x, &dimln->l.p0off.y, 
                     &dimln->l.p1off.x, &dimln->l.p1off.y );
   if( status )
   {
    if( (LEN( dimln->l.p0, dimln->l.p1    ) >= pk->toler) &&
        (LEN( dimln->l.p0, dimln->l.p0off ) >= pk->toler) )
    {
     if( dimln->put != 0L )
      (*dimln->put )( dimln->iup_cv, &dimln->l, pk->toler );
    }
   }
   CllDimLnDel( dimln );
   if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
  }
  else
  {
   CllDimLnDel( dimln );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
  }
  return;
 }

/* Compute coordinates of offset point (dimension line).
 */
 status = CllDimLnOffsetPts( dimln->l.p0.x, dimln->l.p0.y, dimln->l.p1.x, 
                     dimln->l.p1.y, dimln->l.ref.x, dimln->l.ref.y, 
                     &dimln->l.p0off.x, &dimln->l.p0off.y, 
                     &dimln->l.p1off.x, &dimln->l.p1off.y );

/* Call the client function to put the collected dimension line, 
 * and finish collection calling end action function.
 */
 if( status )
 {
  if( (LEN( dimln->l.p0, dimln->l.p1    ) >= pk->toler) &&
      (LEN( dimln->l.p0, dimln->l.p0off ) >= pk->toler) )
  {
   if( dimln->put != 0L )
    (*dimln->put )( dimln->iup_cv, &dimln->l, pk->toler );
  }
 }
 CllDimLnDel( dimln );
 if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
}


/* ======================  CllKbdDimLnCreate  ====================== */

static CllDimLn  kbddimln;
static double    kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados da linha de dimenso. " };
static char *kbdprompt_msge = { "Use the keyboard to enter dimension line. " };

int CllKbdDimLnCreate( Ihandle *iup_cv, 
              void (*put)( Ihandle *iup_cv, void *dimln, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllDimLn   dimln;
 float      x0, y0, x1, y1;
 float      xref, yref;
 float      toler;
 int        status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 dimln = kbddimln;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 x0 = dimln.p0.x;
 y0 = dimln.p0.y;
 x1 = dimln.p1.x;
 y1 = dimln.p1.y;
 xref = dimln.ref.x;
 yref = dimln.ref.y;
 toler = kbdtol;
 if( DlgCaptureDimLn( &x0, &y0, &x1, &y1, &xref, &yref, &toler ) == 1 )
 {
  dimln.p0.x = x0;
  dimln.p0.y = y0;
  dimln.p1.x = x1;
  dimln.p1.y = y1;
  dimln.ref.x = xref;
  dimln.ref.y = yref;

/* Compute coordinates of offset point (dimension line).
 */
  status = CllDimLnOffsetPts( dimln.p0.x, dimln.p0.y, dimln.p1.x, 
                              dimln.p1.y, dimln.ref.x, dimln.ref.y, 
                              &dimln.p0off.x, &dimln.p0off.y, 
                              &dimln.p1off.x, &dimln.p1off.y );
  if( status )
  {
   if( (LEN( dimln.p0, dimln.p1    ) > 0.0) &&
       (LEN( dimln.p0, dimln.p0off ) > 0.0) )
   {
    kbddimln = dimln;
    kbdtol = toler;
    if( put != 0L )
     (*put)( iup_cv, &dimln, kbdtol );
    status = CLL_NORMAL;
   }
   else
   {
    status = CLL_QUIT;
   }
  }
  else
  {
   status = CLL_QUIT;
  }
 }

 return( status );
}


