char *RCScllpline="$Id: cllpline.c,v 1.9 2001/09/13 14:34:48 william Exp $";
/*
** cllpline.c - Functions to collect polylines.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllPLineCreate( Ihandle *iup_cv, void *cd_cv, 
**                void (*end_action)( Ihandle *iup_cv, int status ),
**                void (*put)( Ihandle *iup_cv, void *line, double tol ), 
**                void (*prompt)( Ihandle *iup_cv, char *msg ),
**                void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                              double tol, int shift, int control ),
**                void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - function to be called after finishing line creat. (IN)
** put        - ptr. to function to put line after collection     (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a polyline
** on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something
**     else (not by the mouse interaction).
**
** --------------------------------------------------------------------
** 
** int  CllKbdPLineCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *line, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a polyline.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** void CllPlgonCreate( Ihandle *iup_cv, void *cd_cv, 
**                void (*end_action)( Ihandle *iup_cv, int status ),
**                void (*put)( Ihandle *iup_cv, void *line, double tol ), 
**                void (*prompt)( Ihandle *iup_cv, char *msg ),
**                void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                              double tol, int shift, int control ),
**                void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - function to be called after finishing line creat. (IN)
** put        - ptr. to function to put line after collection     (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a polyline
** on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something
**     else (not by the mouse interaction).
** The difference between this function and CllPLineCreate is that 
** during collection a rubberband line is drawn from the current
** point being collected and the first collect polyline point.  This
** will feedback to the user that the collected polyline is closed
** (i.e., it is a polygon).
**
** --------------------------------------------------------------------
** 
** int  CllKbdPlgonCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *line, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a polyline.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
** This is a dummy function.  It calls immediatly function 
** CllKbdPLineCreate above.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    25-Mar-94    Luiz F. Martha
**
** Modified:   31-Mar-94    LFM
**     When created Pck package: change old Gra mouse input (pick)
**     calls to Pck calls.
**
** Modified:   11-Apr-94    LFM
**     Eliminated "reinit" parameter in CllPLineCreate and added
**     "end_action" parameter.
**
** Modified:   29-May-95    LFM
**     Version 1: Version to use with CD graphics package.
**     Got rid of GraPnt, replacing it by CllPnt.  CllPoint was
**     redefined as CllPnt: structure containing two fields: x, y.
**     Replaced double by float.
**     Changed prototype of snap and echo functions so that point
**     structure is replaced by x and y coordinates.
**     Created local function CllPLineSnap due to change in Pck snap
**     function prototype.
**
** Modified:   16-Aug-95    LFM
**     When created "wd.h".
**
** Modified:   25-Ago-95    LFM
**     Created prompt messages in Portuguese.
**
** Modified:   14-Feb-96    LFM
**     Modified local snap function such that it calls client 
**     snap function only if control key is pressed.  Before
**     this was done in Pck module.
**     Changed such that nothing is done when end_action function
**     in null.  Before the collector reinitialized itself.
**
** Modified:   08-Mar-96    LFM
**     When CD changed, creating cdCreateCanvas, which returns
**     a CD handle for the canvas.
**     Changed CllPLineCreate adding a parameter which is the CD
**     canvas handle.
**     Created function CllPlgonCreate.
**     Added field "closed" to local line data structure to handle
**     creation of closed polyline (polygon).
**     Changed local function CllPLineEcho such that, if the 
**     closed flag is on, it draws the line closing off the polygon
**     in xor mode.
**
** Modified:   28-Jan-97    LFM 
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection functions CllKbdPLineCreate and 
**     CllKbdPlgonCreate.
**
** Modified:   11-Apr-97    LFM
**     Added call to CllShiftSnap.
**
** Modified:   17-Apr-97    LFM
**      Added return status parameter for end action function.
**
** Modified:   23-Apr-97    LFM
**      Considered environment variable IUP_INGLES for prompt and
**      dialog messages.
**
** Modified:   19-Aug-97    LFM
**      Fixed problem of accessing collected object after deleting
**      it for calling end action function.
**
** Modified:   18-Feb-98    LFM
**      Modification to avoid displaying echo messages on the canvas.
**      Instead, echo messages are send to prompt message UI object.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   11-Sep-2001  William
**      Substitution of the calls the IupScanf( ) for dialogs defined in the 
**      dlgpline module.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "cll.h"
#include "cllpline.h"
#include "dlgpline.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Polyline data structure:
 */
#define  ALLOC_SIZE   10

typedef struct _cllplinep { /* Collected line private spec.descrip.: */
        CllPLine pl;                /* collected polyline            */
        int      allocsize;         /* current allocated size        */
        int      closed;            /* flag for closed line (polygon)*/
        void     *iup_cv;           /* IUP handle to target canvas   */
        void     *cd_cv;            /* cd handle to corresp.canvas   */
        void     (*end_action)      /* end action callback           */
                       ( Ihandle *iup_cv, int status );
        void     (*put)             /* function to put collect.line  */
                       ( Ihandle *iup_cv, void *line, double tol );
        void     (*prompt)          /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void     (*snap)            /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void     (*echo)            /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y );
        } CllPLineP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

static int  iup_english;
static char *prompt_msg;

static char *prompt_msgp = { "Entre ponto (botao esquerdo do mouse); "
                             "Botao do meio => Teclado; "
                             "Botao da direita => Termina." };
static char *prompt_msge = { "Enter point with mouse left button; "
                             "Mid button => Keyboard; "
                             "Right button => Finish." };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  CllPLineRealloc  ======================= */

static void CllPLineRealloc( CllPLineP *line )
{
  if( line->pl.npts == line->allocsize )
  {
   line->allocsize += ALLOC_SIZE;
   line->pl.pts = (CllPnt *)realloc( line->pl.pts, 
                                     line->allocsize * sizeof( CllPnt ) );
  }
}


/* ==========================  CllPLineDel  ======================== */

static void CllPLineDel( CllPLineP *line )
{
 if( line == NULL ) return;

/*  Release memory used for line
 */    
 if( line->pl.npts > 0 )
 {
  free( line->pl.pts ) ;
 }
 free( line );
}


/* =========================  CllPLineDsp  ========================= */

static void CllPLineDsp( void *l )
{
 CllPLineP *line = (CllPLineP *)l;
 long int  fore_color;
 int       i;

 if( line->pl.npts < 2 ) return;

 fore_color = cdForeground( CD_RED );
 cdBegin( CD_OPEN_LINES );
 for( i=0; i<line->pl.npts; i++ )
  wdVertex( line->pl.pts[i].x, line->pl.pts[i].y );
 cdEnd( );
 cdForeground( fore_color );
}


/* =========================  CllPLineClr  ========================= */

static void CllPLineClr( CllPLineP *line )
{
 long int  fore_color;
 long int  back_color;
 int       i;

 if( line->pl.npts < 2 ) return;

 back_color = cdBackground( CD_BLACK );
 fore_color = cdForeground( back_color );
 cdBegin( CD_OPEN_LINES );
 for( i=0; i<line->pl.npts; i++ )
  wdVertex( line->pl.pts[i].x, line->pl.pts[i].y );
 cdEnd( );
 cdForeground( fore_color );
 cdBackground( back_color );
}


/* ========================  CllPLineSnap  ========================= */
 
static void CllPLineSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 CllPLineP  *line = (CllPLineP *)pk->data;
 
/* If shift key is pressed and first polyline point has been given,
 * snap current line segment to the horizontal axis, to the vertical 
 * axis, or the bi-sector lines.
 */
 if( (line->pl.npts > 0) && isshift(pk->mousecfg) )
 {
  CllShiftSnap( line->pl.pts[line->pl.npts-1].x, 
                line->pl.pts[line->pl.npts-1].y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( line->snap != 0L )
  (*line->snap )( line->iup_cv, x, y, tol,
                              isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}
 

/* ========================  CllPLineEcho  ========================= */
 
static void CllPLineEcho( PckDesc *pk, double x, double y )
{
 double len;
 int    write_mode;
 int    fore_color;
 int    line_style;
 int    line_width;
 char   echotext[60];

/* Restore registered collection spec. from the data field.
 */
 CllPLineP  *line = (CllPLineP *)pk->data;
 
/* If first point was already given, echo last line segment length.
 * Check to see if this is a polygon collection.
 * If that is the case, draws the line closing off the 
 * current polyline in xor mode.
 */
 if( line->pl.npts > 0 )
 {
  len = sqrt( (line->pl.pts[line->pl.npts-1].x-x)*
              (line->pl.pts[line->pl.npts-1].x-x) +
              (line->pl.pts[line->pl.npts-1].y-y) *
              (line->pl.pts[line->pl.npts-1].y-y) );

#ifdef _UNIX_
  write_mode = cdWriteMode( CD_XOR );
#else
  write_mode = cdWriteMode( CD_NOT_XOR );
#endif
  fore_color = cdForeground( CD_RED );

  if( line->closed && (line->pl.npts > 1) )
  {
   line_style = cdLineStyle( CD_CONTINUOUS );
   line_width = cdLineWidth( 1 );
   wdLine( x, y, line->pl.pts[0].x, line->pl.pts[0].y );
   cdLineWidth( line_width );
   cdLineStyle( line_style );
  }

  if( iup_english )
   sprintf( echotext, "Segment length = %10.3f", len );
  else
   sprintf( echotext, "Comprimento do segmento = %10.3f", len );
  if( line->prompt != NULL )
   (*line->prompt)( line->iup_cv, echotext ) ;

  cdWriteMode( write_mode );
  cdForeground( fore_color );
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( line->echo != NULL )
  (*line->echo )( line->iup_cv, x, y ) ;
}
 

/* ===========================  CllPLinePt  ======================== */

static void CllPLinePt( PckDesc *pk )
{
 CllPLineP *line = (CllPLineP *)pk->data;
 void      *iup_cv = line->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = line->end_action;

/* If user gave up adding points to line, check to see
 * if collected line is valid.  If it is, call the client 
 * function to put the collected line.
 * If collection was interupted, just quit collection.
 * In either case call end action function and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;

  CllClose( );
  if( line->pl.npts > 1 )
   CllPLineClr( line );

  switch( pk->status )
  {
   default:
   case PCK_QUIT:
    if( line->pl.npts > 1 )
    {
     if( line->put != NULL )
      (*line->put )( line->iup_cv, &line->pl, pk->toler );
    }
    if( line->pl.npts < 1 )
     status = CLL_QUIT;
    else
     status = CLL_NORMAL;
    break;

   case PCK_INTERUPTED:
    status = CLL_INTERUPTED;
    break;
  }
  CllPLineDel( line );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Display line collected so far
 */
 if( line->pl.npts > 1 )
 {
   CllPLineDsp( line );
 }

/* Reallocate space for line if necessary.
 */
 CllPLineRealloc( line );

 if( pk->button == 2 )
 {
/* If middle button was pressed, the user wants to enter the
 * line point using the keyboard. In this case, prompt the user for
 * point coordinates.
 */
  float x, y;
  x = pk->p1.x;  y = pk->p1.y;
  if( DlgCaptureXPLine( &x, &y ) )
  {
   pk->p1.x = x;
   pk->p1.y = y;
  }
  else
  {
   CllClose( );
   if( line->pl.npts > 1 )
    CllPLineClr( line );
   CllPLineDel( line );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* If last point is valid is valid (either first point or last segment 
 * greater than tolerance), add collected point to line, and increment 
 * number of collected points.
 */
 if( ( line->pl.npts == 0 ) || 
     ( LEN(pk->p1,line->pl.pts[line->pl.npts-1]) >= pk->toler ) )
 {
  line->pl.pts[line->pl.npts].x = pk->p1.x;
  line->pl.pts[line->pl.npts].y = pk->p1.y;
  line->pl.npts++;
 }

/* Display collected line segment.
 */
 if( line->pl.npts > 1 )
 {
  long int  fore_color = cdForeground( CD_RED );
  wdLine( line->pl.pts[line->pl.npts-2].x, line->pl.pts[line->pl.npts-2].y,
          line->pl.pts[line->pl.npts-1].x, line->pl.pts[line->pl.npts-1].y );
  cdForeground( fore_color );
 }

/* Keep getting picks from the user.
 */
 if( line->prompt != NULL )
  (*line->prompt )( line->iup_cv, prompt_msg ) ;
 pk->button = 0;       /* accept both mouse buttons 1 or 2 */
 if( line->pl.npts > 0 )
 {
  pk->p0.x = line->pl.pts[line->pl.npts-1].x;
  pk->p0.y = line->pl.pts[line->pl.npts-1].y;
  pk->cursorecho = RubberBandCursor | BoxCursor | CrossCursor;
 }
 pk->p1.x = pk->p1.y = 0.0;
 PckSetup( pk );
}


/* =======================  CllPLineCreateGen  ===================== */

static void CllPLineCreateGen( int closed, 
              Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *line, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ), 
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllPLineP  *line;
 PckDesc    pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg = prompt_msge;
 }
 else
 {
  prompt_msg = prompt_msgp;
 }

 CllClose( );

/* Allocate a new polyline descriptor
 */
 line = (CllPLineP *)malloc( sizeof( CllPLineP ) );

/* Initialize collection
 */
 line->pl.npts = 0; 
 line->allocsize = ALLOC_SIZE;
 line->pl.pts = (CllPnt *)calloc( line->allocsize, sizeof( CllPnt ) );
 line->closed = closed;
 line->iup_cv = iup_cv;
 line->cd_cv = cd_cv;
 line->end_action = end_action;
 line->put = put;
 line->prompt = prompt;
 line->snap = snap;
 line->echo = echo;

/* Register polyline display function with collection super class.
 */
 CllOpen( line, CllPLineDsp );

/* Setup pick descriptor.  Register the appropriate line collector
 * function as the callback for pick interaction.
 */
 if( line->prompt != NULL )
  (*line->prompt )( line->iup_cv, prompt_msg ) ;
 pk.iup_cv = line->iup_cv;
 pk.gra_cv = line->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = CllPLinePt;
 pk.btup_callback = NULL;
 pk.echofunc = CllPLineEcho;
 pk.snapfunc = CllPLineSnap;
 pk.data = line;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}


/* ======================  CllKbdPLineCreateGen  =================== */

static int     kbdnpts;
static CllPnt  kbdpnt;
static double  kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados da linha poligonal. " };
static char *kbdprompt_msge = { "Use the keyboard to enter polyline data. " };

static int  CllKbdPLineCreateGen( Ihandle *iup_cv,
              void (*put)( Ihandle *iup_cv, void *line, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllPLine line;
 int      npts;
 float    x, y;
 float    toler;
 int      i;
 int      status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 npts = kbdnpts;
 toler = kbdtol;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 if( DlgCaptureNPLine( &npts, &toler ) )
 {
  kbdtol = toler;
  if( (npts > 1) && (toler >= 0.0) )
  { 
   line.npts = 0;
   line.pts = (CllPnt *)calloc( npts, sizeof( CllPnt ) );
   for( i=0; i<npts; i++ )
   {
    x = kbdpnt.x;
    y = kbdpnt.y;
    if( DlgCaptureXPLine( &x, &y ) )
    {
     kbdpnt.x = x;
     kbdpnt.y = y;
     line.pts[line.npts++] = kbdpnt;
    }
    else
     break;
   }
   if( line.npts > 1 )
   {
    kbdnpts = line.npts;
    if( put != 0L )
     (*put)( iup_cv, &line, kbdtol );
    status = CLL_NORMAL;
   }
   free( line.pts );
  }
 }

 return( status );
}


/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ========================  CllPLineCreate  ======================= */

void CllPLineCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *line, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ), 
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllPLineCreateGen( 0, iup_cv, cd_cv, end_action, put, prompt, snap, echo );
}


/* ======================  CllKbdPLineCreate  ====================== */

int  CllKbdPLineCreate( Ihandle *iup_cv, 
              void (*put)( Ihandle *iup_cv, void *line, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 return( CllKbdPLineCreateGen( iup_cv, put, prompt ) );
}


/* ========================  CllPlgonCreate  ======================= */

void CllPlgonCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *line, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ), 
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllPLineCreateGen( 1, iup_cv, cd_cv, end_action, put, prompt, snap, echo );
}


/* ======================  CllKbdPlgonCreate  ====================== */

int  CllKbdPlgonCreate( Ihandle *iup_cv, 
              void (*put)( Ihandle *iup_cv, void *line, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 return( CllKbdPLineCreateGen( iup_cv, put, prompt ) );
}


