char *RCScllpnt="$Id: cllpnt.c,v 1.5 2001/09/13 14:34:48 william Exp $";
/*
** cllpnt.c - Functions to collect point.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllPntCreate( Ihandle *iup_cv, void *cd_cv,
**                void (*end_action)( Ihandle *iup_cv, int status );
**                void (*put)( Ihandle *iup_cv, void *pt, double tol ), 
**                void (*prompt)( Ihandle *iup_cv, char *msg ),
**                void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                              double tol, int shift, int control ),
**                void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - function to be called after finishing point creat.(IN)
** put        - ptr. to function to put point after collection    (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap point during collection  (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a point
** on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit point collection with mouse 
**     right button.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something
**     else (not by the mouse interaction).
**
** --------------------------------------------------------------------
**
** int  CllKbdPntCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *pt, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a point.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      1-000
**
** Created:    26-Mar-94    Luiz F. Martha and Joao Luiz E. Campos (Acai)
**
** Modified:   01-Abr-94    Acai
**   When created Pck package: change old Gra mouse input (pick)
**   calls to Pck calls.
**
** Modified:   11-Apr-94    LFM
**     Eliminated "reinit" parameter in CllPntCreate and added
**     "end_action" parameter.
**
** Modified:   29-May-95    LFM
**     Version 1: Version to use with CD graphics package.
**     This file was renamed from cllpoint.c to cllpnt.c.
**     Got rid of GraPnt, replacing it by CllPnt.  CllPoint was
**     redefined as CllPnt: structure containing two fields: x, y.
**     Replaced double by float.
**     Changed prototype of snap and echo functions so that point
**     structure is replaced by x and y coordinates.
**     Created local function CllPntSnap due to change in Pck snap
**     function prototype.
**
** Modified:   16-Aug-95    LFM
**     When created "wd.h".
**
** Modified:   25-Ago-95    LFM
**     Created prompt messages in Portuguese.
**
** Modified:   14-Feb-96    LFM
**     Modified local snap function such that it calls client 
**     snap function only if control key is pressed.  Before
**     this was done in Pck module.
**     Changed such that nothing is done when end_action function
**     in null.  Before the collector reinitialized itself.
**
** Modified:   08-Mar-96    LFM
**     When CD changed, creating cdCreateCanvas, which returns
**     a CD handle for the canvas.
**     Changed CllPntCreate adding a parameter which is the CD
**     canvas handle.
**
** Modified:   28-Jan-97    LFM 
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection function CllKbdPntCreate.
**
** Modified:   09-Apr-97    LFM
**     Added call to CllClose in function CllPntCreate to close
**     any previous data structure in Cll super-class.
**
** Modified:   17-Apr-97    LFM
**      Added return status parameter for end action function.
**
** Modified:   23-Apr-97    LFM
**      Considered environment variable IUP_INGLES for prompt and 
**      dialog messages.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   10-Sep-2001  William
**      Substitution of the calls the IupScanf( ) for dialogs defined in the 
**      dlgpnt module.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "cd.h"
#include "pck.h"
#include "cll.h"
#include "cllpnt.h"
#include "dlgpnt.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Point data structure:
 */

typedef struct _cllpntp { /* Collected point private spec.descrip.:  */
        CllPnt   p;                 /* collected point               */
        void     *iup_cv;           /* IUP handle to target canvas   */
        void     *cd_cv;            /* cd handle to corresp.canvas   */
        void     (*end_action)      /* end action callback           */
                       ( Ihandle *iup_cv, int status );
        void     (*put)             /* function to put collect.point */
                       ( Ihandle *iup_cv, void *pt, double tol );
        void     (*prompt)          /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void     (*snap)            /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void     (*echo)            /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y );
        } CllPntP;

static int  iup_english;
static char *prompt_msg;

static char *prompt_msgp = { "Com botao esquerdo do mouse entre com um ponto; "
                             "Botao do meio => Teclado." };
static char *prompt_msge = { "With mouse left button enter a point; "
                             "Mid button => Keyboard." };


/*
** --------------------------------------------------------------------
** Local functions:
*/

/* ===========================  CllPntDel  ========================= */

static void CllPntDel( CllPntP *pt )
{
 if( pt == 0L ) return;

/*  Release memory used for point.
 */    
 free( pt );
}


/* ==========================  CllPntSnap  ========================= */
 
static void CllPntSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 CllPntP  *pt = (CllPntP *)pk->data;
 
/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( pt->snap != 0L )
  (*pt->snap )( pt->iup_cv, x, y, tol, 
                              isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}


/* ==========================  CllPntEcho  ========================= */
 
static void CllPntEcho( PckDesc *pk, double x, double y )
{
/* Restore registered collection spec. from the data field.
 */
 CllPntP  *pt = (CllPntP *)pk->data;
 
/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( pt->echo != 0L )
  (*pt->echo )( pt->iup_cv, x, y ) ;
}


/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  CllPntCreate  ======================== */

static void CllPntPt( PckDesc *pk );

void CllPntCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *pt, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllPntP    *pt;
 PckDesc    pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg = prompt_msge;
 }
 else
 {
  prompt_msg = prompt_msgp;
 }

 CllClose( );

/* Allocate a point collection descriptor
 */
 pt = (CllPntP *)malloc( sizeof( CllPntP ) );

/* Initialize collection
 */
 pt->iup_cv = iup_cv;
 pt->cd_cv = cd_cv;
 pt->end_action = end_action;
 pt->put = put;
 pt->prompt = prompt;
 pt->snap = snap;
 pt->echo = echo;
 pt->p.x = pt->p.y = 0.0;

/* Setup pick descriptor.  Register the appropriate point collector
 * function as the callback for pick interaction.
 */
 if( pt->prompt != 0L )
  (*pt->prompt )( pt->iup_cv, prompt_msg ) ;
 pk.iup_cv = pt->iup_cv;
 pk.gra_cv = pt->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = CllPntPt;
 pk.btup_callback = 0L;
 pk.echofunc = CllPntEcho;
 pk.snapfunc = CllPntSnap;
 pk.data = pt;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* ----------------------------- CllPntPt  --------------------------*/

static void CllPntPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllPntP  *pt = (CllPntP *)pk->data;
 void     *iup_cv = pt->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = pt->end_action;

/* Check to see if collected point is valid.  If not valid, 
 * just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_QUIT;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllPntDel( pt );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }
 
/* Get button down point as target point.
 */
 pt->p.x = pk->p0.x;
 pt->p.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * point using the keyboard. In this case, prompt the user for
 * point coordinates and return;
 */
 if( pk->button == 2 )
 {
  float x, y;
  x = pt->p.x;  y = pt->p.y;
  if( DlgCapturePPoint( &x, &y ) )
  {
   pt->p.x = x;  pt->p.y = y;
   if( pt->put != 0L )
    (*pt->put )( pt->iup_cv, &pt->p, pk->toler );
   CllPntDel( pt );
   if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
  }
  else
  {
   CllPntDel( pt );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
  }
  return;
 }

/* Call the client function to put the valid collected point
 * and finish collection calling end action function.
 */
 if( pt->put != 0L )
  (*pt->put )( pt->iup_cv, &pt->p, pk->toler );
 CllPntDel( pt );
 if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
}


/* ========================  CllKbdPntCreate  ====================== */

static CllPnt  kbdpnt;
static double  kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados do ponto. " };
static char *kbdprompt_msge = { "Use the keyboard to enter point data. " };

int  CllKbdPntCreate( Ihandle *iup_cv,
              void (*put)( Ihandle *iup_cv, void *pt, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllPnt   pnt;
 float    x, y;
 float    toler;
 int      status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 pnt = kbdpnt;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 x = pnt.x;
 y = pnt.y;
 toler = kbdtol;
 if( DlgCapturePoint( &x, &y, &toler ) )
 {
  pnt.x = x;
  pnt.y = y;
  kbdpnt = pnt;
  kbdtol = toler;
  if( put != 0L )
   (*put)( iup_cv, &pnt, kbdtol );
  status = CLL_NORMAL;
 }

 return( status );
}


