
/*
** dlgarc.c - Functions that establish the pre-defined dialogs 
**            of the library to capture arcs.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureArc( float *cx, float *cy, float *p1x, float *p1y, 
                                            float *p2x, float *p2y, float *tol )
**
** cx  - coordinate (X direction) of center.       (IN/OUT)
** cy  - coordinate (Y direction) of center.       (IN/OUT)
** p1x - coordinate (X direction) of first point.  (IN/OUT)
** p1y - coordinate (X direction) of first point.  (IN/OUT)
** p2x - coordinate (X direction) of second point. (IN/OUT)
** p2y - coordinate (X direction) of second point. (IN/OUT)
** tol - tolerance used.                           (IN/OUT)
**
** This function creates a dialog to capture the parameters of a arc. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureFArc( float *cx, float *cy )
**
** cx  - coordinate (X direction) of center.       (IN/OUT)
** cy  - coordinate (Y direction) of center.       (IN/OUT)
**
** This function creates a dialog to capture the parameters of a arc. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureSArc( float *cx, float *cy, float *p1x, float *p1y )
**
** cx  - coordinate (X direction) of center.       (IN/OUT)
** cy  - coordinate (Y direction) of center.       (IN/OUT)
** p1x - coordinate (X direction) of first point.  (IN/OUT)
** p1y - coordinate (X direction) of first point.  (IN/OUT)
**
** This function creates a dialog to capture the parameters of a arc. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureTArc( float *cx, float *cy, float *p1x, float *p1y,
		                             float *p2x, float *p2y )
**
** cx  - coordinate (X direction) of center.       (IN/OUT)
** cy  - coordinate (Y direction) of center.       (IN/OUT)
** p1x - coordinate (X direction) of first point.  (IN/OUT)
** p1y - coordinate (X direction) of first point.  (IN/OUT)
** p2x - coordinate (X direction) of second point. (IN/OUT)
** p2y - coordinate (X direction) of second point. (IN/OUT)
**
** This function creates a dialog to capture the parameters of a arc. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    10-Sep-01    William W. M. Lira.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgarc.h"


static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure */

typedef struct _arcdlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;
  char *pointlabel5;
  char *pointlabel6;
  char *pointlabel7;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} ArcDlg;


static void  dlgInitArc      ( ArcDlg *dial );
static void* dlgCreateArc    ( void );
static void  dlgValueToArc   ( float cx,  float cy, 
                               float p1x, float p1y,
                               float p2x, float p2y, float tol );
static int   dlgValidateArc  ( void );
static void  dlgValueFromArc ( float *cx,  float *cy, 
                               float *p1x, float *p1y, 
                               float *p2x, float *p2y, float *tol );

static void  dlgInitFArc      ( ArcDlg *dial );
static void* dlgCreateFArc    ( void );
static void  dlgValueToFArc   ( float cx, float cy );
static int   dlgValidateFArc  ( void );
static void  dlgValueFromFArc ( float *cx, float *cy );

static void  dlgInitSArc     ( ArcDlg *dial );
static void* dlgCreateSArc   ( void );
static void  dlgValueToSArc  ( float p1x, float p1y );
static int   dlgValidateSArc ( void );
static void  dlgValueFromSArc( float *p1x, float *p1y );

static void  dlgInitTArc     ( ArcDlg *dial );
static void* dlgCreateTArc   ( void );
static void  dlgValueToTArc  ( float cx, float cy, float p1x, float p1y,
				                   float p2x, float p2y );
static int   dlgValidateTArc ( void );
static void  dlgValueFromTArc( float *cx,  float *cy, 
                               float *p1x, float *p1y,
			       float *p2x, float *p2y );

static int  callbackArcDialogOk       ( Ihandle *bt );
static int  callbackArcDialogCancel   ( Ihandle *bt );
static int  callbackArcDialogCloseCB  ( Ihandle *bt );

static int  callbackSArcDialogOk      ( Ihandle *bt );
static int  callbackSArcDialogCancel  ( Ihandle *bt );
static int  callbackSArcDialogCloseCB ( Ihandle *bt );

static int  callbackFArcDialogOk      ( Ihandle *bt );
static int  callbackFArcDialogCancel  ( Ihandle *bt );
static int  callbackFArcDialogCloseCB ( Ihandle *bt );

static int  callbackTArcDialogOk      ( Ihandle *bt );
static int  callbackTArcDialogCancel  ( Ihandle *bt );
static int  callbackTArcDialogCloseCB ( Ihandle *bt );

static ArcDlg  arc;
static ArcDlg farc;
static ArcDlg sarc;
static ArcDlg tarc;

static char *size_butt = {"55x12"};
static char *size_text = {"65"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with arc parameters: " };
static char *llabel1e = { "Center X:  " };
static char *llabel2e = { "Center Y:  " };
static char *llabel3e = { "Point1 X:  " };
static char *llabel4e = { "Point1 Y:  " };
static char *llabel5e = { "Point2 X:  " };
static char *llabel6e = { "Point2 Y:  " };
static char *llabel7e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel8e  = { "Enter with center coordinates:" };
static char *llabel9e  = { "Enter with first point coordinates:" };
static char *llabel10e = { "Enter with second point coordinates:" };

static char *llabel0p = { "Entre com os parametros do arco:" };
static char *llabel1p = { "Centro X:  " };
static char *llabel2p = { "Centro Y:  " };
static char *llabel3p = { "Ponto1 X:  " };
static char *llabel4p = { "Ponto1 Y:  " };
static char *llabel5p = { "Ponto2 X:  " };
static char *llabel6p = { "Ponto2 Y:  " };
static char *llabel7p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

static char *llabel8p  = { "Entre com as coordenadas do centro:" };
static char *llabel9p  = { "Entre com as coordenadas do primeiro ponto:" };
static char *llabel10p = { "Entre com as coordenadas do segundo ponto:" };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitArc  ======================= */

static void dlgInitArc ( ArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointlabel4 = llabel4e; 
   dial->pointlabel5 = llabel5e; 
   dial->pointlabel6 = llabel6e; 
   dial->pointlabel7 = llabel7e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p; 
   dial->pointlabel5 = llabel5p; 
   dial->pointlabel6 = llabel6p; 
   dial->pointlabel7 = llabel7p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateArc  ======================= */

static void* dlgCreateArc ( void )
{
  if ( arc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( arc.pointbuttn1, "call_arc_ok" );
    Ihandle *butt2 = IupButton ( arc.pointbuttn2, "call_arc_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;
    Ihandle *labl7 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;
    Ihandle *text6 = NULL;
    Ihandle *text7 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;
    Ihandle *h6 = NULL;
    Ihandle *h7 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( arc.pointlabel1 );
    labl2 = IupLabel  ( arc.pointlabel2 );
    labl3 = IupLabel  ( arc.pointlabel3 );
    labl4 = IupLabel  ( arc.pointlabel4 );
    labl5 = IupLabel  ( arc.pointlabel5 );
    labl6 = IupLabel  ( arc.pointlabel6 );
    labl7 = IupLabel  ( arc.pointlabel7 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );
    text6 = IupText   ( NULL );
    text7 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);
    h6 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl6,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text6,IupFill(),NULL), 
		  IupFill(), NULL);
    h7 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl7,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text7,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
		h6,
	        IupFill(),
		h7,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text7, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text7, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( text7, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );
    IupSetAttribute ( labl7, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  arc.pointlabel0 );

    /* sets handles */
    IupSetHandle("cx0_arc1_handle", text1);
    IupSetHandle("cy0_arc1_handle", text2);
    IupSetHandle("p1x_arc1_handle", text3);
    IupSetHandle("p1y_arc1_handle", text4);
    IupSetHandle("p2x_arc1_handle", text5);
    IupSetHandle("p2y_arc1_handle", text6);
    IupSetHandle("tol_arc1_handle", text7);
    iupmaskSet(IupGetHandle("cx0_arc1_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("cy0_arc1_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p1x_arc1_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p1y_arc1_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p2x_arc1_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p2y_arc1_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("tol_arc1_handle"), IUPMASK_UFLOAT , 0, 1);

    /* creates dialog */
    arc.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_arc_ok", (Icallback)callbackArcDialogOk );
    IupSetFunction ( "call_arc_cn", (Icallback)callbackArcDialogCancel );
    IupSetFunction ( "call_arc_close", (Icallback)callbackArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( arc.dialog, "GAP",    "3" );
    IupSetAttribute ( arc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( arc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( arc.dialog, "RESIZE", "NO");
    IupSetAttribute ( arc.dialog, "CLOSE_CB", "call_arc_close");
#ifdef _UNIX_
    IupSetAttribute ( arc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( arc.dialog, "TITLE",  "Arc" );

    IupMap ( arc.dialog );
  }

  return arc.dialog;
}

/* ======================  dlgValueToArc  ======================= */

static void dlgValueToArc ( float cx,  float cy, 
                            float p1x, float p1y,
                            float p2x, float p2y, float tol )
{
 char cx0[50];
 char cy0[50];
 char cp1x[50];
 char cp1y[50];
 char cp2x[50];
 char cp2y[50];
 char ctol[50];

 if ( arc.dialog == NULL ) return;

 sprintf(cx0,  format, cx);
 sprintf(cy0,  format, cy);
 sprintf(cp1x, format, p1x);
 sprintf(cp1y, format, p1y);
 sprintf(cp2x, format, p2x);
 sprintf(cp2y, format, p2y);
 sprintf(ctol, format, tol);

 IupSetAttribute (IupGetHandle("cx0_arc1_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("cy0_arc1_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("p1x_arc1_handle"), "VALUE", cp1x);
 IupSetAttribute (IupGetHandle("p1y_arc1_handle"), "VALUE", cp1y);
 IupSetAttribute (IupGetHandle("p2x_arc1_handle"), "VALUE", cp2x);
 IupSetAttribute (IupGetHandle("p2y_arc1_handle"), "VALUE", cp2y);
 IupSetAttribute (IupGetHandle("tol_arc1_handle"), "VALUE", ctol);
}

/* =====================  dlgValueFromArc  ====================== */

static void dlgValueFromArc ( float *cx,  float *cy, 
                              float *p1x, float *p1y,
                              float *p2x, float *p2y, float *tol )
{
  if ( arc.dialog == NULL ) return;

  /* normal case */
  *cx  = IupGetFloat (IupGetHandle("cx0_arc1_handle"), "VALUE");
  *cy  = IupGetFloat (IupGetHandle("cy0_arc1_handle"), "VALUE");
  *p1x = IupGetFloat (IupGetHandle("p1x_arc1_handle"), "VALUE");
  *p1y = IupGetFloat (IupGetHandle("p1y_arc1_handle"), "VALUE");
  *p2x = IupGetFloat (IupGetHandle("p2x_arc1_handle"), "VALUE");
  *p2y = IupGetFloat (IupGetHandle("p2y_arc1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tol_arc1_handle"), "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateArc  ======================= */

static int dlgValidateArc ( void )
{
 char *cx0;
 char *cy0;
 char *p1x;
 char *p1y;
 char *p2x;
 char *p2y;
 char *tol;

 if ( arc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(arc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 cx0 = IupGetAttribute (IupGetHandle("cx0_arc1_handle"), "VALUE");
 cy0 = IupGetAttribute (IupGetHandle("cy0_arc1_handle"), "VALUE");
 p1x = IupGetAttribute (IupGetHandle("p1x_arc1_handle"), "VALUE");
 p1y = IupGetAttribute (IupGetHandle("p1y_arc1_handle"), "VALUE");
 p2x = IupGetAttribute (IupGetHandle("p2x_arc1_handle"), "VALUE");
 p2y = IupGetAttribute (IupGetHandle("p2y_arc1_handle"), "VALUE");
 tol = IupGetAttribute (IupGetHandle("tol_arc1_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitFArc  ==================== */

static void dlgInitFArc ( ArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel8e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel8p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateFArc  =================== */

static void* dlgCreateFArc ( void )
{
  if ( farc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( farc.pointbuttn1, "call_farc_ok" );
    Ihandle *butt2 = IupButton ( farc.pointbuttn2, "call_farc_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( farc.pointlabel1 );
    labl2 = IupLabel  ( farc.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  farc.pointlabel0 );

    /* sets handles */
    IupSetHandle("cx_arc2_handle", text1);
    IupSetHandle("cy_arc2_handle", text2);
    iupmaskSet(IupGetHandle("cx_arc2_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("cy_arc2_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    farc.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_farc_ok", (Icallback)callbackFArcDialogOk );
    IupSetFunction ( "call_farc_cn", (Icallback)callbackFArcDialogCancel );
    IupSetFunction ( "call_farc_close", (Icallback)callbackFArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( farc.dialog, "GAP",    "3" );
    IupSetAttribute ( farc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( farc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( farc.dialog, "RESIZE", "NO");
    IupSetAttribute ( farc.dialog, "CLOSE_CB", "call_farc_close");
#ifdef _UNIX_
    IupSetAttribute ( farc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( farc.dialog, "TITLE",  "Arc" );

    IupMap ( farc.dialog );
  }

  return farc.dialog;
}

/* ===================  dlgValueToFArc  =================== */

static void dlgValueToFArc ( float cx, float cy )
{
 char cx0[50];
 char cy0[50];

 if ( farc.dialog == NULL ) return;

 sprintf(cx0, format, cx);
 sprintf(cy0, format, cy);

 IupSetAttribute (IupGetHandle("cx_arc2_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("cy_arc2_handle"), "VALUE", cy0);
}

/* =================  dlgValueFromFArc  =================== */

static void dlgValueFromFArc ( float *cx, float *cy )
{
  if ( sarc.dialog == NULL ) return;
 
  *cx = IupGetFloat (IupGetHandle("cx_arc2_handle"), "VALUE");
  *cy = IupGetFloat (IupGetHandle("cy_arc2_handle"), "VALUE");
}

/* ==================  dlgValidateFArc  =================== */

static int dlgValidateFArc ( void )
{
 char *cx0;
 char *cy0;

 if ( farc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(farc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 cx0 = IupGetAttribute (IupGetHandle("cx_arc2_handle"), "VALUE");
 cy0 = IupGetAttribute (IupGetHandle("cy_arc2_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitSArc  ==================== */

static void dlgInitSArc ( ArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel9e; 
   dial->pointlabel1 = llabel3e; 
   dial->pointlabel2 = llabel4e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel9p; 
   dial->pointlabel1 = llabel3p; 
   dial->pointlabel2 = llabel4p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateSArc  =================== */

static void* dlgCreateSArc ( void )
{
  if ( sarc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( sarc.pointbuttn1, "call_sarc_ok" );
    Ihandle *butt2 = IupButton ( sarc.pointbuttn2, "call_sarc_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( sarc.pointlabel1 );
    labl2 = IupLabel  ( sarc.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  sarc.pointlabel0);

    /* sets handles */
    IupSetHandle("p1x_arc3_handle", text1);
    IupSetHandle("p1y_arc3_handle", text2);
    iupmaskSet(IupGetHandle("p1x_arc3_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p1y_arc3_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    sarc.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_sarc_ok", (Icallback)callbackSArcDialogOk );
    IupSetFunction ( "call_sarc_cn", (Icallback)callbackSArcDialogCancel );
    IupSetFunction ( "call_sarc_close", (Icallback)callbackSArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( sarc.dialog, "GAP",    "3" );
    IupSetAttribute ( sarc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( sarc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( sarc.dialog, "RESIZE", "NO");
    IupSetAttribute ( sarc.dialog, "CLOSE_CB", "call_sarc_close");
#ifdef _UNIX_
    IupSetAttribute ( sarc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( sarc.dialog, "TITLE",  "Arc" );

    IupMap ( sarc.dialog );
  }

  return sarc.dialog;
}

/* ===================  dlgValueToSArc  =================== */

static void dlgValueToSArc ( float p1x, float p1y )
{
 char cp1x[50];
 char cp1y[50];

 if ( sarc.dialog == NULL ) return;

 sprintf(cp1x, format, p1x);
 sprintf(cp1y, format, p1y);

 IupSetAttribute (IupGetHandle("p1x_arc3_handle"), "VALUE", cp1x);
 IupSetAttribute (IupGetHandle("p1y_arc3_handle"), "VALUE", cp1y);
}

/* =================  dlgValueFromSArc  =================== */

static void dlgValueFromSArc ( float *p1x, float *p1y )
{
  if ( sarc.dialog == NULL ) return;

  *p1x = IupGetFloat (IupGetHandle("p1x_arc3_handle"), "VALUE");
  *p1y = IupGetFloat (IupGetHandle("p1y_arc3_handle"), "VALUE");
}

/* ==================  dlgValidateSArc  =================== */

static int dlgValidateSArc ( void )
{
 char *p1x;
 char *p1y;

 if ( sarc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(sarc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 p1x = IupGetAttribute (IupGetHandle("p1x_arc3_handle"), "VALUE");
 p1y = IupGetAttribute (IupGetHandle("p1y_arc3_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitTArc  ==================== */

static void dlgInitTArc ( ArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel10e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointlabel4 = llabel4e; 
   dial->pointlabel5 = llabel5e; 
   dial->pointlabel6 = llabel6e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel10p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p; 
   dial->pointlabel5 = llabel5p; 
   dial->pointlabel6 = llabel6p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateSArc  =================== */

static void* dlgCreateTArc ( void )
{
  if ( tarc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( tarc.pointbuttn1, "call_tarc_ok" );
    Ihandle *butt2 = IupButton ( tarc.pointbuttn2, "call_tarc_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;
    Ihandle *text6 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;
    Ihandle *h6 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( tarc.pointlabel1 );
    labl2 = IupLabel  ( tarc.pointlabel2 );
    labl3 = IupLabel  ( tarc.pointlabel3 );
    labl4 = IupLabel  ( tarc.pointlabel4 );
    labl5 = IupLabel  ( tarc.pointlabel5 );
    labl6 = IupLabel  ( tarc.pointlabel6 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );
    text6 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);
    h6 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl6,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text6,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, 
		       IupFill(), h3, IupFill(), h4, 
		       IupFill(), h5, IupFill(), h6, 
		       IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  tarc.pointlabel0);

    /* sets handles */
    IupSetHandle("cx0_arc4_handle", text1);
    IupSetHandle("cy0_arc4_handle", text2);
    IupSetHandle("p1x_arc4_handle", text3);
    IupSetHandle("p1y_arc4_handle", text4);
    IupSetHandle("p2x_arc4_handle", text5);
    IupSetHandle("p2y_arc4_handle", text6);
    iupmaskSet(IupGetHandle("cx0_arc4_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("cy0_arc4_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p1x_arc4_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p1y_arc4_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p2x_arc4_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p2y_arc4_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    tarc.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_tarc_ok", (Icallback)callbackTArcDialogOk );
    IupSetFunction ( "call_tarc_cn", (Icallback)callbackTArcDialogCancel );
    IupSetFunction ( "call_tarc_close", (Icallback)callbackTArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( tarc.dialog, "GAP",    "3" );
    IupSetAttribute ( tarc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( tarc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( tarc.dialog, "RESIZE", "NO");
    IupSetAttribute ( tarc.dialog, "CLOSE_CB", "call_tarc_close");
#ifdef _UNIX_
    IupSetAttribute ( tarc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( tarc.dialog, "TITLE",  "Arc" );

    IupMap ( tarc.dialog );
  }

  return tarc.dialog;
}

/* ===================  dlgValueToTArc  =================== */

static void dlgValueToTArc ( float cx, float cy, float p1x, float p1y,
			                         float p2x, float p2y )
{
 char  cx0[50];
 char  cy0[50];
 char cp1x[50];
 char cp1y[50];
 char cp2x[50];
 char cp2y[50];

 if ( tarc.dialog == NULL ) return;

 sprintf(cx0,  format, cx);
 sprintf(cy0,  format, cy);
 sprintf(cp1x, format, p1x);
 sprintf(cp1y, format, p1y);
 sprintf(cp2x, format, p2x);
 sprintf(cp2y, format, p2y);

 IupSetAttribute (IupGetHandle("cx0_arc4_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("cy0_arc4_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("p1x_arc4_handle"), "VALUE", cp1x);
 IupSetAttribute (IupGetHandle("p1y_arc4_handle"), "VALUE", cp1y);
 IupSetAttribute (IupGetHandle("p2x_arc4_handle"), "VALUE", cp2x);
 IupSetAttribute (IupGetHandle("p2y_arc4_handle"), "VALUE", cp2y);
}

/* =================  dlgValueFromTArc  =================== */

static void dlgValueFromTArc ( float *cx,  float *cy, 
                               float *p1x, float *p1y,
			       float *p2x ,float *p2y )
{
  if ( tarc.dialog == NULL ) return;

  *cx  = IupGetFloat (IupGetHandle("cx0_arc4_handle"), "VALUE");
  *cy  = IupGetFloat (IupGetHandle("cy0_arc4_handle"), "VALUE");
  *p1x = IupGetFloat (IupGetHandle("p1x_arc4_handle"), "VALUE");
  *p1y = IupGetFloat (IupGetHandle("p1y_arc4_handle"), "VALUE");
  *p2x = IupGetFloat (IupGetHandle("p2x_arc4_handle"), "VALUE");
  *p2y = IupGetFloat (IupGetHandle("p2y_arc4_handle"), "VALUE");
}

/* ==================  dlgValidateTArc  =================== */

static int dlgValidateTArc ( void )
{
 char *cx0;
 char *cy0;
 char *p1x;
 char *p1y;
 char *p2x;
 char *p2y;

 if ( tarc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(tarc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 cx0 = IupGetAttribute (IupGetHandle("cx0_arc4_handle"), "VALUE");
 cy0 = IupGetAttribute (IupGetHandle("cy0_arc4_handle"), "VALUE");
 p1x = IupGetAttribute (IupGetHandle("p1x_arc4_handle"), "VALUE");
 p1y = IupGetAttribute (IupGetHandle("p1y_arc4_handle"), "VALUE");
 p2x = IupGetAttribute (IupGetHandle("p2x_arc4_handle"), "VALUE");
 p2y = IupGetAttribute (IupGetHandle("p2y_arc4_handle"), "VALUE");

 return 1;
}

/* =========================  callbackArcDialogOk  ======================== */

static int callbackArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(arc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackArcDialogCancel  ====================== */

static int callbackArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(arc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackArcDialogCloseCB  ==================== */

static int callbackArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(arc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackFArcDialogOk  ====================== */

static int callbackFArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(farc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackFArcDialogCancel  ==================== */

static int callbackFArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(farc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackFArcDialogCloseCB  ==================== */

static int callbackFArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(farc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackSArcDialogOk  ====================== */

static int callbackSArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(sarc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackSArcDialogCancel  ==================== */

static int callbackSArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(sarc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackSArcDialogCloseCB  =================== */

static int callbackSArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(sarc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackTArcDialogOk  ====================== */

static int callbackTArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(tarc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackTArcDialogCancel  ==================== */

static int callbackTArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(tarc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackTArcDialogCloseCB  ==================== */

static int callbackTArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(tarc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureArc  ======================== */

int DlgCaptureArc ( float *cx, float *cy, float *p1x, float *p1y, 
                                          float *p2x, float *p2y, float *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitArc( &arc );

  /* create dialog to capture point data. */
  d = dlgCreateArc();

  /* sets point data to dialog. */
  dlgValueToArc ( *cx, *cy, *p1x, *p1y, *p2x, *p2y, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromArc ( cx, cy, p1x, p1y, p2x, p2y, tol );

  return 1;
}

/* ======================  DlgCaptureFArc  ===================== */

int DlgCaptureFArc ( float *cx, float *cy )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitFArc( &farc );

  /* create dialog to capture point data. */
  d = dlgCreateFArc();

  /* sets point data to dialog. */
  dlgValueToFArc ( *cx, *cy );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateFArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromFArc ( cx, cy );

  return 1;
}

/* ======================  DlgCaptureSArc  ===================== */

int DlgCaptureSArc ( float *p1x, float *p1y )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitSArc( &sarc );

  /* create dialog to capture point data. */
  d = dlgCreateSArc();

  /* sets point data to dialog. */
  dlgValueToSArc ( *p1x, *p1y );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateSArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromSArc ( p1x, p1y );

  return 1;
}

/* ======================  DlgCaptureTArc  ===================== */

int DlgCaptureTArc ( float *cx, float *cy, float *p1x, float *p1y, 
                                           float *p2x, float *p2y )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitTArc( &tarc );

  /* create dialog to capture point data. */
  d = dlgCreateTArc();

  /* sets point data to dialog. */
  dlgValueToTArc ( *cx, *cy, *p1x, *p1y, *p2x, *p2y );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateTArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromTArc ( cx, cy, p1x, p1y, p2x, p2y );

  return 1;
}

