  
/*
** dlgbez.c - Functions that establish the pre-defined dialogs 
**            of the library to capture beziers.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureBezier( float *px1, float *py1, float *px2, float *py2,
                         float *px3, float *py3, float *px4, float *py4, 
			 float *tol )
**
** px1 - coordinate (X direction) of first bezier.       (IN/OUT)
** py1 - coordinate (Y direction) of first bezier.       (IN/OUT)
** px2 - coordinate (X direction) of second bezier.      (IN/OUT)
** py2 - coordinate (Y direction) of second bezier.      (IN/OUT)
** px3 - coordinate (X direction) of third bezier.       (IN/OUT)
** py3 - coordinate (Y direction) of third bezier.       (IN/OUT)
** px4 - coordinate (X direction) of forth bezier.       (IN/OUT)
** py4 - coordinate (Y direction) of forth bezier.       (IN/OUT)
** tol - tolerance used.                          (IN/OUT)
**
** This function creates a dialog to capture the parameters of a bezier. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapturePBezier( float *px1, float *py1 )
**
** px1 - coordinate (X direction) for each bezier of bezier.  (IN/OUT)
** py1 - coordinate (Y direction) for each bezier of bezier.  (IN/OUT)
**
** This function creates a dialog to capture the parameters of a bezier. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureLBezier( float *px1, float *py1, float *px2, float *py2,
                          float *px3, float *py3, float *px4, float *py4 )
**
** px1 - coordinate (X direction) of first bezier.       (IN/OUT)
** py1 - coordinate (Y direction) of first bezier.       (IN/OUT)
** px2 - coordinate (X direction) of second bezier.      (IN/OUT)
** py2 - coordinate (Y direction) of second bezier.      (IN/OUT)
** px3 - coordinate (X direction) of third bezier.       (IN/OUT)
** py3 - coordinate (Y direction) of third bezier.       (IN/OUT)
** px4 - coordinate (X direction) of forth bezier.       (IN/OUT)
** py4 - coordinate (Y direction) of forth bezier.       (IN/OUT)
**
** This function creates a dialog to capture the parameters of a bezier. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
** Version:    1-000
**
** Created:    10-Sep-01    William W. M. Lira.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgbez.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store bezier dialog */

typedef struct _bezierdlg
{

  char *bezlabel0;
  char *bezlabel1;
  char *bezlabel2;
  char *bezlabel3;
  char *bezlabel4;
  char *bezlabel5;
  char *bezlabel6;
  char *bezlabel7;
  char *bezlabel8;
  char *bezlabel9;
  char *bezierbuttn1;
  char *bezierbuttn2;

  Ihandle *dialog;

} BezDlg;


static void  dlgInitBezier      ( BezDlg *dial );
static void* dlgCreateBezier    ( void );
static void  dlgValueToBezier   ( float px1, float py1, float px2, 
                                  float py2, float px3, float py3, 
				  float px4, float py4, float tol );
static int   dlgValidateBezier  ( void );
static void  dlgValueFromBezier ( float *px1, float *py1, float *px2,
                                  float *py2, float *px3, float *py3,
				  float *px4, float *py4, float *tol );

static void  dlgInitPBezier      ( BezDlg *dial );
static void* dlgCreatePBezier    ( void );
static void  dlgValueToPBezier   ( float px1, float py1 );
static int   dlgValidatePBezier  ( void );
static void  dlgValueFromPBezier ( float *px1, float *py1 );

static void  dlgInitLBezier      ( BezDlg *dial );
static void* dlgCreateLBezier    ( void );
static void  dlgValueToLBezier   ( float px1, float py1, float px2, 
                                   float py2, float px3, float py3, 
				   float px4, float py4 );
static int   dlgValidateLBezier  ( void );
static void  dlgValueFromLBezier ( float *px1, float *py1, float *px2,
                                   float *py2, float *px3, float *py3,
				   float *px4, float *py4 );

static int  callbackBezierDialogOk     ( Ihandle *bt );
static int  callbackBezierDialogCancel ( Ihandle *bt );
static int  callbackBezierDialogCloseCB( Ihandle *bt );

static int  callbackPBezierDialogOk     ( Ihandle *bt );
static int  callbackPBezierDialogCancel ( Ihandle *bt );
static int  callbackPBezierDialogCloseCB( Ihandle *bt );

static int  callbackLBezierDialogOk     ( Ihandle *bt );
static int  callbackLBezierDialogCancel ( Ihandle *bt );
static int  callbackLBezierDialogCloseCB( Ihandle *bt );

static BezDlg  bez;
static BezDlg pbez;
static BezDlg lbez;

static char *size_butt = {"55x12"};
static char *size_text = {"55"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with interpolation points:" };
static char *llabel1e = { "Point1 X:  " };
static char *llabel2e = { "Point1 Y:  " };
static char *llabel3e = { "Point2 X:  " };
static char *llabel4e = { "Point2 Y:  " };
static char *llabel5e = { "Point3 X:  " };
static char *llabel6e = { "Point3 Y:  " };
static char *llabel7e = { "Point4 X:  " };
static char *llabel8e = { "Point4 Y:  " };
static char *llabel9e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel0p = { "Entre com os pontos de interpolacao:" };
static char *llabel1p = { "Ponto1 X:  " };
static char *llabel2p = { "Ponto1 Y:  " };
static char *llabel3p = { "Ponto2 X:  " };
static char *llabel4p = { "Ponto2 Y:  " };
static char *llabel5p = { "Ponto3 X:  " };
static char *llabel6p = { "Ponto3 Y:  " };
static char *llabel7p = { "Ponto4 X:  " };
static char *llabel8p = { "Ponto4 Y:  " };
static char *llabel9p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitBezier  ======================= */

static void dlgInitBezier ( BezDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->bezlabel0 = llabel0e; 
   dial->bezlabel1 = llabel1e; 
   dial->bezlabel2 = llabel2e; 
   dial->bezlabel3 = llabel3e; 
   dial->bezlabel4 = llabel4e; 
   dial->bezlabel5 = llabel5e; 
   dial->bezlabel6 = llabel6e; 
   dial->bezlabel7 = llabel7e; 
   dial->bezlabel8 = llabel8e; 
   dial->bezlabel9 = llabel9e; 
   dial->bezierbuttn1 = lbuttn1e; 
   dial->bezierbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->bezlabel0 = llabel0p; 
   dial->bezlabel1 = llabel1p; 
   dial->bezlabel2 = llabel2p; 
   dial->bezlabel3 = llabel3p; 
   dial->bezlabel4 = llabel4p; 
   dial->bezlabel5 = llabel5p; 
   dial->bezlabel6 = llabel6p; 
   dial->bezlabel7 = llabel7p; 
   dial->bezlabel8 = llabel8p; 
   dial->bezlabel9 = llabel9p; 
   dial->bezierbuttn1 = lbuttn1p; 
   dial->bezierbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateBezier  ======================= */

static void* dlgCreateBezier ( void )
{
  if ( bez.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( bez.bezierbuttn1, "call_bez_ok" );
    Ihandle *butt2 = IupButton ( bez.bezierbuttn2, "call_bez_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;
    Ihandle *labl7 = NULL;
    Ihandle *labl8 = NULL;
    Ihandle *labl9 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;
    Ihandle *text6 = NULL;
    Ihandle *text7 = NULL;
    Ihandle *text8 = NULL;
    Ihandle *text9 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;
    Ihandle *h6 = NULL;
    Ihandle *h7 = NULL;
    Ihandle *h8 = NULL;
    Ihandle *h9 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( bez.bezlabel1 );
    labl2 = IupLabel  ( bez.bezlabel2 );
    labl3 = IupLabel  ( bez.bezlabel3 );
    labl4 = IupLabel  ( bez.bezlabel4 );
    labl5 = IupLabel  ( bez.bezlabel5 );
    labl6 = IupLabel  ( bez.bezlabel6 );
    labl7 = IupLabel  ( bez.bezlabel7 );
    labl8 = IupLabel  ( bez.bezlabel8 );
    labl9 = IupLabel  ( bez.bezlabel9 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );
    text6 = IupText   ( NULL );
    text7 = IupText   ( NULL );
    text8 = IupText   ( NULL );
    text9 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);
    h6 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl6,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text6,IupFill(),NULL), 
		  IupFill(), NULL);
    h7 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl7,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text7,IupFill(),NULL), 
		  IupFill(), NULL);
    h8 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl8,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text8,IupFill(),NULL), 
		  IupFill(), NULL);
    h9 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl9,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text9,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(), h1, IupFill(), h2, IupFill(), h3, IupFill(), h4,
	        IupFill(), h5, IupFill(), h6, IupFill(), h7, IupFill(), h8,
	        IupFill(), h9, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text7, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text8, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text9, "FGCOLOR", "000 000 000" );

    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text7, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text8, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text9, "BGCOLOR", "255 255 255" );

    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( text7, "SIZE", size_text );
    IupSetAttribute ( text8, "SIZE", size_text );
    IupSetAttribute ( text9, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );
    IupSetAttribute ( labl7, "SIZE", size_text );
    IupSetAttribute ( labl8, "SIZE", size_text );
    IupSetAttribute ( labl9, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP",    "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  bez.bezlabel0 );

    /* sets handles */
    IupSetHandle("px1_bez1_handle", text1);
    IupSetHandle("py1_bez1_handle", text2);
    IupSetHandle("px2_bez1_handle", text3);
    IupSetHandle("py2_bez1_handle", text4);
    IupSetHandle("px3_bez1_handle", text5);
    IupSetHandle("py3_bez1_handle", text6);
    IupSetHandle("px4_bez1_handle", text7);
    IupSetHandle("py4_bez1_handle", text8);
    IupSetHandle("tol_bez1_handle", text9);
    iupmaskSet(IupGetHandle("px1_bez1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py1_bez1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("px2_bez1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py2_bez1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("px3_bez1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py3_bez1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("px4_bez1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py4_bez1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tol_bez1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    bez.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_bez_ok",  (Icallback)callbackBezierDialogOk );
    IupSetFunction ( "call_bez_cn",  (Icallback)callbackBezierDialogCancel );
    IupSetFunction ( "call_bez_close", (Icallback)callbackBezierDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( bez.dialog, "GAP",    "3" );
    IupSetAttribute ( bez.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( bez.dialog, "MAXBOX", "NO");
    IupSetAttribute ( bez.dialog, "RESIZE", "NO");
    IupSetAttribute ( bez.dialog, "CLOSE_CB", "call_bez_close");
#ifdef _UNIX_
    IupSetAttribute ( bez.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( bez.dialog, "TITLE",  "Bezier" );

    IupMap ( bez.dialog );
  }

  return bez.dialog;
}

/* ======================  dlgValueToBezier  ======================= */

static void dlgValueToBezier ( float px1, float py1, float px2,
                               float py2, float px3, float py3,
                               float px4, float py4, float tol )
{
 char cpx1[50];
 char cpy1[50];
 char cpx2[50];
 char cpy2[50];
 char cpx3[50];
 char cpy3[50];
 char cpx4[50];
 char cpy4[50];
 char ctol[50];

 if ( bez.dialog == NULL ) return;

 sprintf(cpx1, format, px1);
 sprintf(cpy1, format, py1);
 sprintf(cpx2, format, px2);
 sprintf(cpy2, format, py2);
 sprintf(cpx3, format, px3);
 sprintf(cpy3, format, py3);
 sprintf(cpx4, format, px4);
 sprintf(cpy4, format, py4);
 sprintf(ctol, format, tol);

 IupSetAttribute (IupGetHandle("px1_bez1_handle"), "VALUE", cpx1);
 IupSetAttribute (IupGetHandle("py1_bez1_handle"), "VALUE", cpy1);
 IupSetAttribute (IupGetHandle("px2_bez1_handle"), "VALUE", cpx2);
 IupSetAttribute (IupGetHandle("py2_bez1_handle"), "VALUE", cpy2);
 IupSetAttribute (IupGetHandle("px3_bez1_handle"), "VALUE", cpx3);
 IupSetAttribute (IupGetHandle("py3_bez1_handle"), "VALUE", cpy3);
 IupSetAttribute (IupGetHandle("px4_bez1_handle"), "VALUE", cpx4);
 IupSetAttribute (IupGetHandle("py4_bez1_handle"), "VALUE", cpy4);
 IupSetAttribute (IupGetHandle("tol_bez1_handle"), "VALUE", ctol);
}

/* =====================  dlgValueFromBezier  ====================== */

static void dlgValueFromBezier ( float *px1, float *py1, float *px2,
                                 float *py2, float *px3, float *py3,
                                 float *px4, float *py4, float *tol )
{
  if ( bez.dialog == NULL ) return;

  /* normal case */
  *px1 = IupGetFloat (IupGetHandle("px1_bez1_handle"), "VALUE");
  *py1 = IupGetFloat (IupGetHandle("py1_bez1_handle"), "VALUE");
  *px2 = IupGetFloat (IupGetHandle("px2_bez1_handle"), "VALUE");
  *py2 = IupGetFloat (IupGetHandle("py2_bez1_handle"), "VALUE");
  *px3 = IupGetFloat (IupGetHandle("px3_bez1_handle"), "VALUE");
  *py3 = IupGetFloat (IupGetHandle("py3_bez1_handle"), "VALUE");
  *px4 = IupGetFloat (IupGetHandle("px4_bez1_handle"), "VALUE");
  *py4 = IupGetFloat (IupGetHandle("py4_bez1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tol_bez1_handle"), "VALUE");
  
  if (*tol == 0.0)
   *tol = 0.001;
}

/* =====================  dlgValidateBezier  ======================= */

static int dlgValidateBezier ( void )
{
 char *px1;
 char *py1;
 char *px2;
 char *py2;
 char *px3;
 char *py3;
 char *px4;
 char *py4;
 char *tol;

 if ( bez.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(bez.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 px1 = IupGetAttribute (IupGetHandle("px1_bez1_handle"), "VALUE");
 py1 = IupGetAttribute (IupGetHandle("py1_bez1_handle"), "VALUE");
 px2 = IupGetAttribute (IupGetHandle("px2_bez1_handle"), "VALUE");
 py2 = IupGetAttribute (IupGetHandle("py2_bez1_handle"), "VALUE");
 px3 = IupGetAttribute (IupGetHandle("px3_bez1_handle"), "VALUE");
 py3 = IupGetAttribute (IupGetHandle("py3_bez1_handle"), "VALUE");
 px4 = IupGetAttribute (IupGetHandle("px4_bez1_handle"), "VALUE");
 py4 = IupGetAttribute (IupGetHandle("py4_bez1_handle"), "VALUE");
 tol = IupGetAttribute (IupGetHandle("tol_bez1_handle"), "VALUE");
 
 return 1;
}

/* ====================  dlgInitPBezier  ==================== */

static void dlgInitPBezier ( BezDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   char *llabel0 = { "Enter with first interpotation point:" };
   char *llabel1 = { "Point X: " };
   char *llabel2 = { "Point Y: " };
   char *lbuttn1 = { " OK " };
   char *lbuttn2 = { " Cancel " };

   dial->bezlabel0 = llabel0; 
   dial->bezlabel1 = llabel1; 
   dial->bezlabel2 = llabel2; 
   dial->bezierbuttn1 = lbuttn1; 
   dial->bezierbuttn2 = lbuttn2; 
 }
 else
 {
   char *llabel0 = { "Entre com o primeiro ponto de interpolacao:" };
   char *llabel1 = { "Ponto X: " };
   char *llabel2 = { "Ponto Y: " };
   char *lbuttn1 = { "Confirma" };
   char *lbuttn2 = { "Cancela " };

   dial->bezlabel0 = llabel0; 
   dial->bezlabel1 = llabel1; 
   dial->bezlabel2 = llabel2; 
   dial->bezierbuttn1 = lbuttn1; 
   dial->bezierbuttn2 = lbuttn2; 
 }
}

/* ====================  dlgCreatePBezier  =================== */

static void* dlgCreatePBezier ( void )
{
  if ( pbez.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( pbez.bezierbuttn1, "call_pbez_ok" );
    Ihandle *butt2 = IupButton ( pbez.bezierbuttn2, "call_pbez_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( pbez.bezlabel1 );
    labl2 = IupLabel  ( pbez.bezlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  pbez.bezlabel0 );

    /* sets handles */
    IupSetHandle("px1_bez2_handle", text1);
    IupSetHandle("py1_bez2_handle", text2);
    iupmaskSet(IupGetHandle("px1_bez2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py1_bez2_handle"), IUPMASK_FLOAT, 0, 1); 

    /* creates dialog */
    pbez.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_pbez_ok", (Icallback)callbackPBezierDialogOk );
    IupSetFunction ( "call_pbez_cn", (Icallback)callbackPBezierDialogCancel );
    IupSetFunction ( "call_pbez_close",(Icallback)callbackPBezierDialogCloseCB);

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( pbez.dialog, "GAP",    "3" );
    IupSetAttribute ( pbez.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( pbez.dialog, "MAXBOX", "NO");
    IupSetAttribute ( pbez.dialog, "RESIZE", "NO");
    IupSetAttribute ( pbez.dialog, "CLOSE_CB", "call_pbez_close");
#ifdef _UNIX_
    IupSetAttribute ( pbez.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( pbez.dialog, "TITLE",  "Bezier" );

    IupMap ( pbez.dialog );
  }

  return pbez.dialog;
}

/* ===================  dlgValueToPBezier  =================== */

static void dlgValueToPBezier ( float px1, float py1 )
{
 char cx0[50];
 char cy0[50];

 if ( pbez.dialog == NULL ) return;

 sprintf(cx0, format, px1);
 sprintf(cy0, format, py1);

 IupSetAttribute (IupGetHandle("px1_bez2_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("py1_bez2_handle"), "VALUE", cy0);
}

/* =================  dlgValueFromPBezier  =================== */

static void dlgValueFromPBezier ( float *px1, float *py1 )
{
  if ( pbez.dialog == NULL ) return;

  *px1  = IupGetFloat (IupGetHandle("px1_bez2_handle"), "VALUE");
  *py1  = IupGetFloat (IupGetHandle("py1_bez2_handle"), "VALUE");
}

/* ==================  dlgValidatePBezier  =================== */

static int dlgValidatePBezier ( void )
{
 char *px1;
 char *py1;

 if ( pbez.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(pbez.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 px1 = IupGetAttribute (IupGetHandle("px1_bez2_handle"), "VALUE");
 py1 = IupGetAttribute (IupGetHandle("py1_bez2_handle"), "VALUE");

 return 1;
}

/* =======================  dlgInitLBezier  ======================= */

static void dlgInitLBezier ( BezDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->bezlabel0 = llabel0e; 
   dial->bezlabel1 = llabel1e; 
   dial->bezlabel2 = llabel2e; 
   dial->bezlabel3 = llabel3e; 
   dial->bezlabel4 = llabel4e; 
   dial->bezlabel5 = llabel5e; 
   dial->bezlabel6 = llabel6e; 
   dial->bezlabel7 = llabel7e; 
   dial->bezlabel8 = llabel8e; 
   dial->bezierbuttn1 = lbuttn1e; 
   dial->bezierbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->bezlabel0 = llabel0p; 
   dial->bezlabel1 = llabel1p; 
   dial->bezlabel2 = llabel2p; 
   dial->bezlabel3 = llabel3p; 
   dial->bezlabel4 = llabel4p; 
   dial->bezlabel5 = llabel5p; 
   dial->bezlabel6 = llabel6p; 
   dial->bezlabel7 = llabel7p; 
   dial->bezlabel8 = llabel8p; 
   dial->bezierbuttn1 = lbuttn1p; 
   dial->bezierbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateLBezier  ======================= */

static void* dlgCreateLBezier ( void )
{
  if ( lbez.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( lbez.bezierbuttn1, "call_lbez_ok" );
    Ihandle *butt2 = IupButton ( lbez.bezierbuttn2, "call_lbez_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;
    Ihandle *labl7 = NULL;
    Ihandle *labl8 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;
    Ihandle *text6 = NULL;
    Ihandle *text7 = NULL;
    Ihandle *text8 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;
    Ihandle *h6 = NULL;
    Ihandle *h7 = NULL;
    Ihandle *h8 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( lbez.bezlabel1 );
    labl2 = IupLabel  ( lbez.bezlabel2 );
    labl3 = IupLabel  ( lbez.bezlabel3 );
    labl4 = IupLabel  ( lbez.bezlabel4 );
    labl5 = IupLabel  ( lbez.bezlabel5 );
    labl6 = IupLabel  ( lbez.bezlabel6 );
    labl7 = IupLabel  ( lbez.bezlabel7 );
    labl8 = IupLabel  ( lbez.bezlabel8 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );
    text6 = IupText   ( NULL );
    text7 = IupText   ( NULL );
    text8 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);
    h6 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl6,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text6,IupFill(),NULL), 
		  IupFill(), NULL);
    h7 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl7,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text7,IupFill(),NULL), 
		  IupFill(), NULL);
    h8 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl8,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text8,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(), h1, IupFill(), h2, IupFill(), h3, IupFill(), h4,
	        IupFill(), h5, IupFill(), h6, IupFill(), h7, IupFill(), h8,
	        IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text7, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text8, "FGCOLOR", "000 000 000" );

    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text7, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text8, "BGCOLOR", "255 255 255" );

    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( text7, "SIZE", size_text );
    IupSetAttribute ( text8, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );
    IupSetAttribute ( labl7, "SIZE", size_text );
    IupSetAttribute ( labl8, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP",    "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  lbez.bezlabel0 );

    /* sets handles */
    IupSetHandle("px1_bez3_handle", text1);
    IupSetHandle("py1_bez3_handle", text2);
    IupSetHandle("px2_bez3_handle", text3);
    IupSetHandle("py2_bez3_handle", text4);
    IupSetHandle("px3_bez3_handle", text5);
    IupSetHandle("py3_bez3_handle", text6);
    IupSetHandle("px4_bez3_handle", text7);
    IupSetHandle("py4_bez3_handle", text8);
    iupmaskSet(IupGetHandle("px1_bez3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py1_bez3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("px2_bez3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py2_bez3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("px3_bez3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py3_bez3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("px4_bez3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("py4_bez3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    lbez.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_lbez_ok", (Icallback)callbackLBezierDialogOk );
    IupSetFunction ( "call_lbez_cn", (Icallback)callbackLBezierDialogCancel );
    IupSetFunction ( "call_lbez_close",(Icallback)callbackLBezierDialogCloseCB);

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( lbez.dialog, "GAP",    "3" );
    IupSetAttribute ( lbez.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( lbez.dialog, "MAXBOX", "NO");
    IupSetAttribute ( lbez.dialog, "RESIZE", "NO");
    IupSetAttribute ( lbez.dialog, "CLOSE_CB", "call_lbez_close");
#ifdef _UNIX_
    IupSetAttribute ( lbez.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( lbez.dialog, "TITLE",  "Bezier" );

    IupMap ( lbez.dialog );
  }

  return lbez.dialog;
}

/* ======================  dlgValueToLBezier  ======================= */

static void dlgValueToLBezier ( float px1, float py1, float px2,
                                float py2, float px3, float py3,
                                float px4, float py4 )
{
 char cpx1[50];
 char cpy1[50];
 char cpx2[50];
 char cpy2[50];
 char cpx3[50];
 char cpy3[50];
 char cpx4[50];
 char cpy4[50];

 if ( lbez.dialog == NULL ) return;

 sprintf(cpx1, format, px1);
 sprintf(cpy1, format, py1);
 sprintf(cpx2, format, px2);
 sprintf(cpy2, format, py2);
 sprintf(cpx3, format, px3);
 sprintf(cpy3, format, py3);
 sprintf(cpx4, format, px4);
 sprintf(cpy4, format, py4);

 IupSetAttribute (IupGetHandle("px1_bez3_handle"), "VALUE", cpx1);
 IupSetAttribute (IupGetHandle("py1_bez3_handle"), "VALUE", cpy1);
 IupSetAttribute (IupGetHandle("px2_bez3_handle"), "VALUE", cpx2);
 IupSetAttribute (IupGetHandle("py2_bez3_handle"), "VALUE", cpy2);
 IupSetAttribute (IupGetHandle("px3_bez3_handle"), "VALUE", cpx3);
 IupSetAttribute (IupGetHandle("py3_bez3_handle"), "VALUE", cpy3);
 IupSetAttribute (IupGetHandle("px4_bez3_handle"), "VALUE", cpx4);
 IupSetAttribute (IupGetHandle("py4_bez3_handle"), "VALUE", cpy4);
}

/* =====================  dlgValueFromLBezier  ====================== */

static void dlgValueFromLBezier ( float *px1, float *py1, float *px2,
                                  float *py2, float *px3, float *py3,
                                  float *px4, float *py4 )
{
  if ( lbez.dialog == NULL ) return;

  /* normal case */
  *px1 = IupGetFloat (IupGetHandle("px1_bez3_handle"), "VALUE");
  *py1 = IupGetFloat (IupGetHandle("py1_bez3_handle"), "VALUE");
  *px2 = IupGetFloat (IupGetHandle("px2_bez3_handle"), "VALUE");
  *py2 = IupGetFloat (IupGetHandle("py2_bez3_handle"), "VALUE");
  *px3 = IupGetFloat (IupGetHandle("px3_bez3_handle"), "VALUE");
  *py3 = IupGetFloat (IupGetHandle("py3_bez3_handle"), "VALUE");
  *px4 = IupGetFloat (IupGetHandle("px4_bez3_handle"), "VALUE");
  *py4 = IupGetFloat (IupGetHandle("py4_bez3_handle"), "VALUE");
}

/* =====================  dlgValidateLBezier  ======================= */

static int dlgValidateLBezier ( void )
{
 char *px1;
 char *py1;
 char *px2;
 char *py2;
 char *px3;
 char *py3;
 char *px4;
 char *py4;

 if ( lbez.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(lbez.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 px1 = IupGetAttribute (IupGetHandle("px1_bez3_handle"), "VALUE");
 py1 = IupGetAttribute (IupGetHandle("py1_bez3_handle"), "VALUE");
 px2 = IupGetAttribute (IupGetHandle("px2_bez3_handle"), "VALUE");
 py2 = IupGetAttribute (IupGetHandle("py2_bez3_handle"), "VALUE");
 px3 = IupGetAttribute (IupGetHandle("px3_bez3_handle"), "VALUE");
 py3 = IupGetAttribute (IupGetHandle("py3_bez3_handle"), "VALUE");
 px4 = IupGetAttribute (IupGetHandle("px4_bez3_handle"), "VALUE");
 py4 = IupGetAttribute (IupGetHandle("py4_bez3_handle"), "VALUE");

 return 1;
}

/* =========================  callbackBezierDialogOk  ======================== */

static int callbackBezierDialogOk ( Ihandle *bt )
{
 IupSetAttribute(bez.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackBezierDialogCancel  ====================== */

static int callbackBezierDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(bez.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackBezierDialogCloseCB  ====================== */

static int callbackBezierDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(bez.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackPBezierDialogOk  ====================== */

static int callbackPBezierDialogOk ( Ihandle *bt )
{
 IupSetAttribute(pbez.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackPBezierDialogCancel  ==================== */

static int callbackPBezierDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(pbez.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackPBezierDialogCloseCB  ==================== */

static int callbackPBezierDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(pbez.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackLBezierDialogOk  ====================== */

static int callbackLBezierDialogOk ( Ihandle *bt )
{
 IupSetAttribute(lbez.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackLBezierDialogCancel  ==================== */

static int callbackLBezierDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(lbez.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackLBezierDialogCloseCB  ==================== */

static int callbackLBezierDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(lbez.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureBezier  ======================== */

int DlgCaptureBezier ( float *px1, float *py1, float *px2, float *py2,
                       float *px3, float *py3, float *px4, float *py4, 
		       float *tol )
{
  Ihandle *d;

  /* initializes bezier dialog. */
  dlgInitBezier( &bez );

  /* create dialog to capture bezier data. */
  d = dlgCreateBezier();

  /* sets bezier data to dialog. */
  dlgValueToBezier ( *px1, *py1, *px2, *py2, *px3, *py3, *px4, *py4, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks bezier data. */
  if ( dlgValidateBezier ( ) == 0 ) return 0;

  /* get bezier data from dialog. */
  dlgValueFromBezier ( px1, py1, px2, py2, px3, py3, px4, py4, tol );

  return 1;
}

/* ======================  DlgCapturePBezier  ===================== */

int DlgCapturePBezier ( float *px1, float *py1 )
{
  Ihandle *d;

  /* initializes bezier dialog. */
  dlgInitPBezier( &pbez );

  /* create dialog to capture bezier data. */
  d = dlgCreatePBezier();

  /* sets bezier data to dialog. */
  dlgValueToPBezier ( *px1, *py1 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks bezier data. */
  if ( dlgValidatePBezier ( ) == 0 ) return 0;

  /* get bezier data from dialog. */
  dlgValueFromPBezier ( px1, py1 );

  return 1;
}

/* =========================  DlgCaptureLBezier  ======================== */

int DlgCaptureLBezier ( float *px1, float *py1, float *px2, float *py2,
                        float *px3, float *py3, float *px4, float *py4 )
{
  Ihandle *d;

  /* initializes bezier dialog. */
  dlgInitLBezier( &lbez );

  /* create dialog to capture bezier data. */
  d = dlgCreateLBezier();

  /* sets bezier data to dialog. */
  dlgValueToLBezier ( *px1, *py1, *px2, *py2, *px3, *py3, *px4, *py4 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks bezier data. */
  if ( dlgValidateLBezier ( ) == 0 ) return 0;

  /* get bezier data from dialog. */
  dlgValueFromLBezier ( px1, py1, px2, py2, px3, py3, px4, py4 );

  return 1;
}
