
/*
** dlgcircl.c - Functions that establish the pre-defined dialogs 
**              of the library to capture circles.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureCircle( float *x0, float *y0, float &r, float *tol )
**
** x0  - coordinate (X direction) of center.      (IN/OUT)
** y0  - coordinate (Y direction) of center.      (IN/OUT)
** r   - radius.                                  (IN/OUT)
** tol - tolerance used.                          (IN/OUT)
**
** This function creates a dialog to capture the parameters of a circle. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureFCircle( float *x0, float *y0 )
**
** x0  - coordinate (X direction) of center.      (IN/OUT)
** y0  - coordinate (Y direction) of center.      (IN/OUT)
**
** This function creates a dialog to capture the parameters of a circle. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureSCircle( float *x0, float *y0, float *r )
**
** x0  - coordinate (X direction) of center.      (IN/OUT)
** y0  - coordinate (Y direction) of center.      (IN/OUT)
** r   - radius.                                  (IN/OUT)
**
** This function creates a dialog to capture the parameters of a circle. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    10-Sep-01    William W. M. Lira.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgcircl.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store point dialog */

typedef struct _circledlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} CircleDlg;


static void  dlgInitCircle      ( CircleDlg *dial );
static void* dlgCreateCircle    ( void );
static void  dlgValueToCircle   ( float x0, float y0, 
                                  float  r, float tol );
static int   dlgValidateCircle  ( void );
static void  dlgValueFromCircle ( float *x0, float *y0, 
                                  float *r,  float *tol );

static void  dlgInitFCircle      ( CircleDlg *dial );
static void* dlgCreateFCircle    ( void );
static void  dlgValueToFCircle   ( float x0, float y0 );
static int   dlgValidateFCircle  ( void );
static void  dlgValueFromFCircle ( float *x0, float *y0 );

static void  dlgInitSCircle      ( CircleDlg *dial );
static void* dlgCreateSCircle    ( void );
static void  dlgValueToSCircle   ( float x0, float y0, float r );
static int   dlgValidateSCircle  ( void );
static void  dlgValueFromSCircle ( float *x0, float *y0, float *r );

static int  callbackCircleDialogOk      ( Ihandle *bt );
static int  callbackCircleDialogCancel  ( Ihandle *bt );
static int  callbackCircleDialogCloseCB ( Ihandle *bt );

static int  callbackSCircleDialogOk     ( Ihandle *bt );
static int  callbackSCircleDialogCancel ( Ihandle *bt );
static int  callbackSCircleDialogCloseCB( Ihandle *bt );

static int  callbackFCircleDialogOk     ( Ihandle *bt );
static int  callbackFCircleDialogCancel ( Ihandle *bt );
static int  callbackFCircleDialogCloseCB( Ihandle *bt );

static CircleDlg  circl;
static CircleDlg fcircl;
static CircleDlg scircl;

static char *size_butt = {"55x12"};
static char *size_text = {"65"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with center coordinates and radius: " };
static char *llabel1e = { "Center X:  " };
static char *llabel2e = { "Center Y:  " };
static char *llabel3e = { "Radius:    " };
static char *llabel4e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };
   
static char *llabel0p = { "Entre com as coordenadas do centro e o raio:" };
static char *llabel1p = { "Centro X:  " };
static char *llabel2p = { "Centro Y:  " };
static char *llabel3p = { "Raio:      " };
static char *llabel4p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitCircle  ======================= */

static void dlgInitCircle ( CircleDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointlabel4 = llabel4e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateCircle  ======================= */

static void* dlgCreateCircle ( void )
{
  if ( circl.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( circl.pointbuttn1, "call_circl_ok" );
    Ihandle *butt2 = IupButton ( circl.pointbuttn2, "call_circl_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( circl.pointlabel1 );
    labl2 = IupLabel  ( circl.pointlabel2 );
    labl3 = IupLabel  ( circl.pointlabel3 );
    labl4 = IupLabel  ( circl.pointlabel4 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  circl.pointlabel0 );

    /* sets handles */
    IupSetHandle("x0_circl1_handle", text1);
    IupSetHandle("y0_circl1_handle", text2);
    IupSetHandle("rd_circl1_handle", text3);
    IupSetHandle("tl_circl1_handle", text4);
    iupmaskSet(IupGetHandle("x0_circl1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_circl1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("rd_circl1_handle"), IUPMASK_UFLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_circl1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    circl.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_circl_ok", (Icallback)callbackCircleDialogOk );
    IupSetFunction ( "call_circl_cn", (Icallback)callbackCircleDialogCancel );
    IupSetFunction ( "call_circl_close",(Icallback)callbackCircleDialogCloseCB);

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( circl.dialog, "GAP",    "3" );
    IupSetAttribute ( circl.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( circl.dialog, "MAXBOX", "NO");
    IupSetAttribute ( circl.dialog, "RESIZE", "NO");
    IupSetAttribute ( circl.dialog, "CLOSE_CB", "call_circl_close");
#ifdef _UNIX_
    IupSetAttribute ( circl.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( circl.dialog, "TITLE",  "Circle" );

    IupMap ( circl.dialog );
  }

  return circl.dialog;
}

/* ======================  dlgValueToCircle  ======================= */

static void dlgValueToCircle ( float x0, float y0, float r, float tol )
{
 char cx0[50];
 char cy0[50];
 char rd0[50];
 char ctl[50];

 if ( circl.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);
 sprintf(rd0, format, r);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("x0_circl1_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_circl1_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("rd_circl1_handle"), "VALUE", rd0);
 IupSetAttribute (IupGetHandle("tl_circl1_handle"), "VALUE", ctl);
}

/* =====================  dlgValueFromCircle  ====================== */

static void dlgValueFromCircle ( float *x0, float *y0, 
                                 float *r, float *tol )
{
  if ( circl.dialog == NULL ) return;

  /* normal case */
  *x0  = IupGetFloat (IupGetHandle("x0_circl1_handle"), "VALUE");
  *y0  = IupGetFloat (IupGetHandle("y0_circl1_handle"), "VALUE");
  *r   = IupGetFloat (IupGetHandle("rd_circl1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tl_circl1_handle"), "VALUE");
  
  if (*tol == 0.0)
   *tol = 0.001;
}

/* =====================  dlgValidateCircle  ======================= */

static int dlgValidateCircle ( void )
{
 char *x0;
 char *y0;
 char *rd;
 char *tl;

 if ( circl.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(circl.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 x0 = IupGetAttribute (IupGetHandle("x0_circl1_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_circl1_handle"), "VALUE");
 rd = IupGetAttribute (IupGetHandle("rd_circl1_handle"), "VALUE");
 tl = IupGetAttribute (IupGetHandle("tl_circl1_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitFCircle  ==================== */

static void dlgInitFCircle ( CircleDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   char *llabel0 = { "Enter with center coordinates:" };
   char *llabel1 = { "Center X:  " };
   char *llabel2 = { "Center Y:  " };
   char *lbuttn1 = { " OK " };
   char *lbuttn2 = { " Cancel " };

   dial->pointlabel0 = llabel0; 
   dial->pointlabel1 = llabel1; 
   dial->pointlabel2 = llabel2; 
   dial->pointbuttn1 = lbuttn1; 
   dial->pointbuttn2 = lbuttn2; 
 }
 else
 {
   char *llabel0 = { "Entre com as coordenadas do centro:" };
   char *llabel1 = { "Centro X: " };
   char *llabel2 = { "Centro Y: " };
   char *lbuttn1 = { "Confirma" };
   char *lbuttn2 = { "Cancela " };

   dial->pointlabel0 = llabel0; 
   dial->pointlabel1 = llabel1; 
   dial->pointlabel2 = llabel2; 
   dial->pointbuttn1 = lbuttn1; 
   dial->pointbuttn2 = lbuttn2; 
 }
}

/* ====================  dlgCreateFCircle  =================== */

static void* dlgCreateFCircle ( void )
{
  if ( fcircl.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( fcircl.pointbuttn1, "call_fcircl_ok" );
    Ihandle *butt2 = IupButton ( fcircl.pointbuttn2, "call_fcircl_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( fcircl.pointlabel1 );
    labl2 = IupLabel  ( fcircl.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  fcircl.pointlabel0 );

    /* sets handles */
    IupSetHandle("x0_circl3_handle", text1);
    IupSetHandle("y0_circl3_handle", text2);
    iupmaskSet(IupGetHandle("x0_circl3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_circl3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    fcircl.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_fcircl_ok", (Icallback)callbackFCircleDialogOk );
    IupSetFunction ( "call_fcircl_cn", (Icallback)callbackFCircleDialogCancel );
    IupSetFunction ( "call_fcircl_close",(Icallback)callbackFCircleDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( fcircl.dialog, "GAP",    "3" );
    IupSetAttribute ( fcircl.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( fcircl.dialog, "MAXBOX", "NO");
    IupSetAttribute ( fcircl.dialog, "RESIZE", "NO");
    IupSetAttribute ( fcircl.dialog, "CLOSE_CB", "call_fcircl_close");
#ifdef _UNIX_
    IupSetAttribute ( fcircl.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( fcircl.dialog, "TITLE",  "Circle" );

    IupMap ( fcircl.dialog );
  }

  return fcircl.dialog;
}

/* ===================  dlgValueToFCircle  =================== */

static void dlgValueToFCircle ( float x0, float y0 )
{
 char cx0[50];
 char cy0[50];

 if ( fcircl.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);

 IupSetAttribute (IupGetHandle("x0_circl3_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_circl3_handle"), "VALUE", cy0);
}

/* =================  dlgValueFromFCircle  =================== */

static void dlgValueFromFCircle ( float *x0, float *y0 )
{
  if ( scircl.dialog == NULL ) return;

  *x0  = IupGetFloat (IupGetHandle("x0_circl3_handle"), "VALUE");
  *y0  = IupGetFloat (IupGetHandle("y0_circl3_handle"), "VALUE");
}

/* ==================  dlgValidateFCircle  =================== */

static int dlgValidateFCircle ( void )
{
 char *x0;
 char *y0;

 if ( fcircl.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(fcircl.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x0 = IupGetAttribute (IupGetHandle("x0_circl3_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_circl3_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitSCircle  ==================== */

static void dlgInitSCircle ( CircleDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateSCircle  =================== */

static void* dlgCreateSCircle ( void )
{
  if ( scircl.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( scircl.pointbuttn1, "call_scircl_ok" );
    Ihandle *butt2 = IupButton ( scircl.pointbuttn2, "call_scircl_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( scircl.pointlabel1 );
    labl2 = IupLabel  ( scircl.pointlabel2 );
    labl3 = IupLabel  ( scircl.pointlabel3 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), 
	               h1, 
		       IupFill(), 
		       h2, 
		       IupFill(), 
		       h3, 
		       IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  scircl.pointlabel0);

    /* sets handles */
    IupSetHandle("x0_circl2_handle", text1);
    IupSetHandle("y0_circl2_handle", text2);
    IupSetHandle("rd_circl2_handle", text3);
    iupmaskSet(IupGetHandle("x0_circl2_handle"), IUPMASK_FLOAT, 1, 1);
    iupmaskSet(IupGetHandle("y0_circl2_handle"), IUPMASK_FLOAT, 1, 1);
    iupmaskSet(IupGetHandle("rd_circl2_handle"), IUPMASK_UFLOAT, 1, 1);

    /* creates dialog */
    scircl.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_scircl_ok", (Icallback)callbackSCircleDialogOk );
    IupSetFunction ( "call_scircl_cn", (Icallback)callbackSCircleDialogCancel );
    IupSetFunction ( "call_scircl_close",(Icallback)callbackSCircleDialogCloseCB );
    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( scircl.dialog, "GAP",    "3" );
    IupSetAttribute ( scircl.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( scircl.dialog, "MAXBOX", "NO");
    IupSetAttribute ( scircl.dialog, "RESIZE", "NO");
    IupSetAttribute ( scircl.dialog, "CLOSE_CB", "call_scircl_close");
#ifdef _UNIX_
    IupSetAttribute ( scircl.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( scircl.dialog, "TITLE",  "Circle" );

    IupMap ( scircl.dialog );
  }

  return scircl.dialog;
}

/* ===================  dlgValueToSCircle  =================== */

static void dlgValueToSCircle ( float x0, float y0, float r )
{
 char cx0[50];
 char cy0[50];
 char rd0[50];

 if ( scircl.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);
 sprintf(rd0, format, r);

 IupSetAttribute (IupGetHandle("x0_circl2_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_circl2_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("rd_circl2_handle"), "VALUE", rd0);
}

/* =================  dlgValueFromSCircle  =================== */

static void dlgValueFromSCircle ( float *x0, float *y0, float *r )
{
  if ( scircl.dialog == NULL ) return;

  *x0  = IupGetFloat (IupGetHandle("x0_circl2_handle"), "VALUE");
  *y0  = IupGetFloat (IupGetHandle("y0_circl2_handle"), "VALUE");
  *r   = IupGetFloat (IupGetHandle("rd_circl2_handle"), "VALUE");
}

/* ==================  dlgValidateSCircle  =================== */

static int dlgValidateSCircle ( void )
{
 char *x0;
 char *y0;
 char *r;

 if ( scircl.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(scircl.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x0 = IupGetAttribute (IupGetHandle("x0_circl2_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_circl2_handle"), "VALUE");
 r  = IupGetAttribute (IupGetHandle("rd_circl2_handle"), "VALUE");

 return 1;
}

/* =========================  callbackCircleDialogOk  ======================== */

static int callbackCircleDialogOk ( Ihandle *bt )
{
 IupSetAttribute(circl.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackCircleDialogCancel  ====================== */

static int callbackCircleDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(circl.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackCircleDialogCloseCB  ====================== */

static int callbackCircleDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(circl.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackSCircleDialogOk  ====================== */

static int callbackSCircleDialogOk ( Ihandle *bt )
{
 IupSetAttribute(scircl.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackSCircleDialogCancel  ==================== */

static int callbackSCircleDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(scircl.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackCircleDialogCloseCB  ====================== */

static int callbackSCircleDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(scircl.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackFCircleDialogOk  ====================== */

static int callbackFCircleDialogOk ( Ihandle *bt )
{
 IupSetAttribute(fcircl.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackFCircleDialogCancel  ==================== */

static int callbackFCircleDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(fcircl.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackCircleDialogCloseCB  ====================== */

static int callbackFCircleDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(fcircl.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureCircle  ======================== */

int DlgCaptureCircle ( float *x0, float *y0, float *r, float *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitCircle( &circl );

  /* create dialog to capture point data. */
  d = dlgCreateCircle();

  /* sets point data to dialog. */
  dlgValueToCircle ( *x0, *y0, *r, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateCircle ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromCircle ( x0, y0, r, tol );

  return 1;
}

/* ======================  DlgCaptureSCircle  ===================== */

int DlgCaptureSCircle ( float *x0, float *y0, float *r )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitSCircle( &scircl );

  /* create dialog to capture point data. */
  d = dlgCreateSCircle();

  /* sets point data to dialog. */
  dlgValueToSCircle ( *x0, *y0, *r );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateSCircle ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromSCircle ( x0, y0, r );

  return 1;
}

/* ======================  DlgCaptureFCircle  ===================== */

int DlgCaptureFCircle ( float *x0, float *y0 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitFCircle( &fcircl );

  /* create dialog to capture point data. */
  d = dlgCreateFCircle();

  /* sets point data to dialog. */
  dlgValueToFCircle ( *x0, *y0 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateFCircle ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromFCircle ( x0, y0 );

  return 1;
}
