
/*
** dlgmirr.c - Functions that establish the pre-defined dialogs 
**              of the library to create mirror objects.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureMirror( *x1, *y1, *x2, *y2, *toler )
**
** x1  - coordinate (X direction) of first point of mirror segment.    (IN/OUT)
** y1  - coordinate (Y direction) of first point of mirror segment.    (IN/OUT)
** x2  - coordinate (X direction) of end point of mirror segment.      (IN/OUT)
** y2  - coordinate (X direction) of end point of mirror segment.      (IN/OUT)
** toler - tolerance used.                                             (IN/OUT)
**
** This function creates a dialog to capture the parameters of a 
** mirror line segment. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureFMirror( float *x1, float *y1 )
**
** x1  - coordinate (X direction) of first point of mirror segment.    (IN/OUT)
** y1  - coordinate (Y direction) of first point of mirror segment.    (IN/OUT)
**
** This function creates a dialog to capture the coordinates of the
** first point of a mirror line segment. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureSMirror( float *x2, float *y2 )
**
** x2  - coordinate (X direction) of end point of mirror segment.      (IN/OUT)
** y2  - coordinate (Y direction) of end point of mirror segment.      (IN/OUT)
**
** This function creates a dialog to capture the coordinates of the 
** end point of a mirror line segment. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    06-Aug-02    Marcos C. de Arruda.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgmirr.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store point dialog */

typedef struct _mirrordlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;
  char *pointlabel5;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} MirrorDlg;


static void  dlgInitMirror      ( MirrorDlg *dial );
static void* dlgCreateMirror    ( void );
static void  dlgValueToMirror   ( float x1, float y1, 
                                  float x2, float y2, float tol );
static int   dlgValidateMirror  ( void );
static void  dlgValueFromMirror ( float *x1, float *y1, 
                                  float *x2, float *y2, float *tol );

static void  dlgInitFMirror      ( MirrorDlg *dial );
static void* dlgCreateFMirror    ( void );
static void  dlgValueToFMirror   ( float x1, float y1 );
static int   dlgValidateFMirror  ( void );
static void  dlgValueFromFMirror ( float *x1, float *y1 );

static void  dlgInitSMirror      ( MirrorDlg *dial );
static void* dlgCreateSMirror    ( void );
static void  dlgValueToSMirror   ( float x1, float y1, float x2, float y2 );
static int   dlgValidateSMirror  ( void );
static void  dlgValueFromSMirror ( float *x1, float *y1,
                                   float *x2, float *y2 );

static int  callbackMirrorDialogOk     ( Ihandle *bt );
static int  callbackMirrorDialogCancel ( Ihandle *bt );
static int  callbackMirrorDialogCloseCB( Ihandle *bt );

static int  callbackSMirrorDialogOk     ( Ihandle *bt );
static int  callbackSMirrorDialogCancel ( Ihandle *bt );
static int  callbackSMirrorDialogCloseCB( Ihandle *bt );

static int  callbackFMirrorDialogOk     ( Ihandle *bt );
static int  callbackFMirrorDialogCancel ( Ihandle *bt );
static int  callbackFMirrorDialogCloseCB( Ihandle *bt );

static MirrorDlg  mirror;
static MirrorDlg fmirror;
static MirrorDlg smirror;

static char *size_butt = {"55x12"};
static char *size_text = {"65"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with mirror segment coordinates: " };
static char *llabel1e = { "X1:  " };
static char *llabel2e = { "Y1:  " };
static char *llabel3e = { "X2:  " };
static char *llabel4e = { "Y2:  " };
static char *llabel5e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel0p = { "Entre com as coordenadas do segmento de espelhamento:" };
static char *llabel1p = { "X1:  " };
static char *llabel2p = { "Y1:  " };
static char *llabel3p = { "X2:  " };
static char *llabel4p = { "Y2:  " };
static char *llabel5p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

char *llabel6e = { "Enter with first point coordinates:" };

char *llabel6p = { "Entre com as coordenadas do ponto inicial:" };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitMirror  ======================= */

static void dlgInitMirror ( MirrorDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointlabel4 = llabel4e;
   dial->pointlabel5 = llabel5e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p; 
   dial->pointlabel5 = llabel5p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateMirror  ======================= */

static void* dlgCreateMirror ( void )
{
  if ( mirror.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( mirror.pointbuttn1, "call_mirror_ok" );
    Ihandle *butt2 = IupButton ( mirror.pointbuttn2, "call_mirror_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( mirror.pointlabel1 );
    labl2 = IupLabel  ( mirror.pointlabel2 );
    labl3 = IupLabel  ( mirror.pointlabel3 );
    labl4 = IupLabel  ( mirror.pointlabel4 );
    labl5 = IupLabel  ( mirror.pointlabel5 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl5,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text5,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
	    NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  mirror.pointlabel0 );

    /* sets handles */
    IupSetHandle("x1_mirr1_handle", text1);
    IupSetHandle("y1_mirr1_handle", text2);
    IupSetHandle("x2_mirr1_handle", text3);
    IupSetHandle("y2_mirr1_handle", text4);
    IupSetHandle("tl_mirr1_handle", text5);
    iupmaskSet(IupGetHandle("x1_mirr1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_mirr1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x2_mirr1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y2_mirr1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_mirr1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    mirror.dialog = IupDialog (
                      IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_mirror_ok", (Icallback)callbackMirrorDialogOk );
    IupSetFunction ( "call_mirror_cn", (Icallback)callbackMirrorDialogCancel );
    IupSetFunction ( "call_mirror_close",(Icallback)callbackMirrorDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( mirror.dialog, "GAP",    "3" );
    IupSetAttribute ( mirror.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( mirror.dialog, "MAXBOX", "NO");
    IupSetAttribute ( mirror.dialog, "RESIZE", "NO");
    IupSetAttribute ( mirror.dialog, "CLOSE_CB", "call_mirror_close");
#ifdef _UNIX_
    IupSetAttribute ( mirror.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( mirror.dialog, "TITLE",  "Mirror" );

    IupMap ( mirror.dialog );
  }

  return mirror.dialog;
}

/* ======================  dlgValueToMirror  ======================= */

static void dlgValueToMirror ( float x1, float y1, float x2, float y2, 
                               float tol )
{
 char cx1[50];
 char cy1[50];
 char cx2[50];
 char cy2[50];
 char ctl[50];

 if ( mirror.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(cx2, format, x2);
 sprintf(cy2, format, y2);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("x1_mirr1_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_mirr1_handle"), "VALUE", cy1);
 IupSetAttribute (IupGetHandle("x2_mirr1_handle"), "VALUE", cx2);
 IupSetAttribute (IupGetHandle("y2_mirr1_handle"), "VALUE", cy2);
 IupSetAttribute (IupGetHandle("tl_mirr1_handle"), "VALUE", ctl);
}

/* =====================  dlgValueFromMirror  ====================== */

static void dlgValueFromMirror ( float *x1, float *y1, 
                                 float *x2, float *y2, float *tol )
{
  if ( mirror.dialog == NULL ) return;

  /* normal case */ 
  *x1  = IupGetFloat (IupGetHandle("x1_mirr1_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_mirr1_handle"), "VALUE");
  *x2  = IupGetFloat (IupGetHandle("x2_mirr1_handle"), "VALUE");
  *y2  = IupGetFloat (IupGetHandle("y2_mirr1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tl_mirr1_handle"), "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateMirror  ======================= */

static int dlgValidateMirror ( void )
{
 char *x1;
 char *y1;
 char *x2;
 char *y2;
 char *tl;

 if ( mirror.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(mirror.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 x1 = IupGetAttribute (IupGetHandle("x1_mirr1_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_mirr1_handle"), "VALUE");
 x2 = IupGetAttribute (IupGetHandle("x2_mirr1_handle"), "VALUE");
 y2 = IupGetAttribute (IupGetHandle("y2_mirr1_handle"), "VALUE");
 tl = IupGetAttribute (IupGetHandle("tl_mirr1_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitFMirror  ==================== */

static void dlgInitFMirror ( MirrorDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel6e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel6p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateFMirror  =================== */

static void* dlgCreateFMirror ( void )
{
  if ( fmirror.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( fmirror.pointbuttn1, "call_fmirror_ok" );
    Ihandle *butt2 = IupButton ( fmirror.pointbuttn2, "call_fmirror_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( fmirror.pointlabel1 );
    labl2 = IupLabel  ( fmirror.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  fmirror.pointlabel0 );

    /* sets handles */
    IupSetHandle("x1_mirr3_handle", text1);
    IupSetHandle("y1_mirr3_handle", text2);
    iupmaskSet(IupGetHandle("x1_mirr3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_mirr3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    fmirror.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_fmirror_ok", (Icallback)callbackFMirrorDialogOk );
    IupSetFunction ( "call_fmirror_cn", (Icallback)callbackFMirrorDialogCancel);
    IupSetFunction ( "call_fmirror_close",(Icallback)callbackFMirrorDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( fmirror.dialog, "GAP",    "3" );
    IupSetAttribute ( fmirror.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( fmirror.dialog, "MAXBOX", "NO");
    IupSetAttribute ( fmirror.dialog, "RESIZE", "NO");
    IupSetAttribute ( fmirror.dialog, "CLOSE_CB", "call_fmirror_close");
#ifdef _UNIX_
    IupSetAttribute ( fmirror.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( fmirror.dialog, "TITLE",  "Mirror" );

    IupMap ( fmirror.dialog );
  }

  return fmirror.dialog;
}

/* ===================  dlgValueToFMirror  =================== */

static void dlgValueToFMirror ( float x1, float y1 )
{
 char cx1[50];
 char cy1[50];

 if ( fmirror.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);

 IupSetAttribute (IupGetHandle("x1_mirr3_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_mirr3_handle"), "VALUE", cy1);
}

/* =================  dlgValueFromFMirror  =================== */

static void dlgValueFromFMirror ( float *x1, float *y1 )
{
  if ( fmirror.dialog == NULL ) return;

  *x1  = IupGetFloat (IupGetHandle("x1_mirr3_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_mirr3_handle"), "VALUE");
}

/* ==================  dlgValidateFMirror  =================== */

static int dlgValidateFMirror ( void )
{
 char *x1;
 char *y1;

 if ( fmirror.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(fmirror.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x1 = IupGetAttribute (IupGetHandle("x1_mirr3_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_mirr3_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitSMirror  ==================== */

static void dlgInitSMirror ( MirrorDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointlabel4 = llabel4e;
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p;
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateSMirror  =================== */

static void* dlgCreateSMirror ( void )
{
  if ( smirror.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( smirror.pointbuttn1, "call_smirror_ok" );
    Ihandle *butt2 = IupButton ( smirror.pointbuttn2, "call_smirror_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( smirror.pointlabel1 );
    labl2 = IupLabel  ( smirror.pointlabel2 );
    labl3 = IupLabel  ( smirror.pointlabel3 );
    labl4 = IupLabel  ( smirror.pointlabel4 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);

    h4 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl4,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text4,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), 
	               h1, 
		       IupFill(), 
		       h2, 
		       IupFill(), 
		       h3, 
		       IupFill(),
		       h4, 
		       IupFill(), NULL )
		);

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  smirror.pointlabel0);

    /* sets handles */
    IupSetHandle("x1_mirr2_handle", text1);
    IupSetHandle("y1_mirr2_handle", text2);
    IupSetHandle("x2_mirr2_handle", text3);
    IupSetHandle("y2_mirr2_handle", text4);
    iupmaskSet(IupGetHandle("x1_mirr2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_mirr2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x2_mirr2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y2_mirr2_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    smirror.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_smirror_ok", (Icallback)callbackSMirrorDialogOk );
    IupSetFunction ( "call_smirror_cn", (Icallback)callbackSMirrorDialogCancel);
    IupSetFunction ( "call_smirror_close",(Icallback)callbackSMirrorDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( smirror.dialog, "GAP",    "3" );
    IupSetAttribute ( smirror.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( smirror.dialog, "MAXBOX", "NO");
    IupSetAttribute ( smirror.dialog, "RESIZE", "NO");
    IupSetAttribute ( smirror.dialog, "CLOSE_CB", "call_smirror_close");
#ifdef _UNIX_
    IupSetAttribute ( smirror.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( smirror.dialog, "TITLE",  "Mirror" );

    IupMap ( smirror.dialog );
  }

  return smirror.dialog;
}

/* ===================  dlgValueToSMirror  =================== */

static void dlgValueToSMirror ( float x1, float y1, float x2, float y2 )
{
 char cx1[50];
 char cy1[50];
 char cx2[50];
 char cy2[50];

 if ( smirror.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(cx2, format, x2);
 sprintf(cy2, format, y2);

 IupSetAttribute (IupGetHandle("x1_mirr2_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_mirr2_handle"), "VALUE", cy1);
 IupSetAttribute (IupGetHandle("x2_mirr2_handle"), "VALUE", cx2);
 IupSetAttribute (IupGetHandle("y2_mirr2_handle"), "VALUE", cy2);
}

/* =================  dlgValueFromSMirror  =================== */

static void dlgValueFromSMirror ( float *x1, float *y1, float *x2, float *y2 )
{
  if ( smirror.dialog == NULL ) return;
  
  *x1  = IupGetFloat (IupGetHandle("x1_mirr2_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_mirr2_handle"), "VALUE");
  *x2  = IupGetFloat (IupGetHandle("x2_mirr2_handle"), "VALUE");
  *y2  = IupGetFloat (IupGetHandle("y2_mirr2_handle"), "VALUE");
}

/* ==================  dlgValidateSMirror  =================== */

static int dlgValidateSMirror ( void )
{
 char *x1;
 char *y1;
 char *x2;
 char *y2;

 if ( smirror.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(smirror.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x1 = IupGetAttribute (IupGetHandle("x1_mirr2_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_mirr2_handle"), "VALUE");
 x2 = IupGetAttribute (IupGetHandle("x2_mirr2_handle"), "VALUE");
 y2 = IupGetAttribute (IupGetHandle("y2_mirr2_handle"), "VALUE");

 return 1;
}

/* =========================  callbackMirrorDialogOk  ======================== */

static int callbackMirrorDialogOk ( Ihandle *bt )
{
 IupSetAttribute(mirror.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackMirrorDialogCancel  ====================== */

static int callbackMirrorDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(mirror.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackMirrorDialogCloseCB  ====================== */

static int callbackMirrorDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(mirror.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackSMirrorDialogOk  ====================== */

static int callbackSMirrorDialogOk ( Ihandle *bt )
{
 IupSetAttribute(smirror.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackSMirrorDialogCancel  ==================== */

static int callbackSMirrorDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(smirror.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackSMirrorDialogCloseCB  ====================== */

static int callbackSMirrorDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(smirror.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackFMirrorDialogOk  ====================== */

static int callbackFMirrorDialogOk ( Ihandle *bt )
{
 IupSetAttribute(fmirror.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackFMirrorDialogCancel  ==================== */

static int callbackFMirrorDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(fmirror.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackFMirrorDialogCloseCB  ====================== */

static int callbackFMirrorDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(fmirror.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureMirror  ======================== */

int DlgCaptureMirror ( float *x1, float *y1, float *x2, float *y2, float *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitMirror( &mirror );

  /* create dialog to capture point data. */
  d = dlgCreateMirror();

  /* sets point data to dialog. */
  dlgValueToMirror ( *x1, *y1, *x2, *y2, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateMirror ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromMirror ( x1, y1, x2, y2, tol );

  return 1;
}

/* ======================  DlgCaptureSMirror  ===================== */

int DlgCaptureSMirror ( float *x1, float *y1, float *x2, float *y2 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitSMirror( &smirror );

  /* create dialog to capture point data. */
  d = dlgCreateSMirror();

  /* sets point data to dialog. */
  dlgValueToSMirror ( *x1, *y1, *x2, *y2 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateSMirror ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromSMirror ( x1, y1, x2, y2 );

  return 1;
}

/* ======================  DlgCaptureFMirror  ===================== */

int DlgCaptureFMirror ( float *x1, float *y1 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitFMirror( &fmirror );

  /* create dialog to capture point data. */
  d = dlgCreateFMirror();

  /* sets point data to dialog. */
  dlgValueToFMirror ( *x1, *y1 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateFMirror ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromFMirror ( x1, y1 );

  return 1;
}

