
/*
** dlgmove.c - Functions that establish the pre-defined dialogs 
**              of the library to move objects.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureMove( *dx, *dy, *toler )
**
** dx  - variation of points coordinates (X direction)    (IN/OUT)
** dy  - variation of points coordinates (Y direction)    (IN/OUT)
** toler - tolerance used.                                (IN/OUT)
**
** This function creates a dialog to capture the translation data. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureFMove( float *x1, float *y1 )
**
** x1  - coordinate (X direction) of first point of translation axis. (IN/OUT)
** y1  - coordinate (Y direction) of first point of translation axis. (IN/OUT)
**
** This function creates a dialog to capture the coordinates of the
** first point of a translation axis. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureSMove( float *x1, float *y1, float *x2, float *y2 )
**
** x1  - coordinate (X direction) of first point of translation axis. (IN/OUT)
** y1  - coordinate (Y direction) of first point of translation axis. (IN/OUT)
** x2  - coordinate (X direction) of end point of translation axis.   (IN/OUT)
** y2  - coordinate (Y direction) of end point of translation axis.   (IN/OUT)
**
** This function creates a dialog to capture the coordinates of the 
** end point of a translation axis. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    08-Aug-02    Marcos C. de Arruda.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgmove.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store point dialog */

typedef struct _movedlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;  
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} MoveDlg;


static void  dlgInitMove      ( MoveDlg *dial );
static void* dlgCreateMove    ( void );
static void  dlgValueToMove   ( float dx, float dy, float tol );
                                
static int   dlgValidateMove  ( void );
static void  dlgValueFromMove ( float *dx, float *dy, float *tol );                                  

static void  dlgInitFMove      ( MoveDlg *dial );
static void* dlgCreateFMove    ( void );
static void  dlgValueToFMove   ( float x1, float y1 );
static int   dlgValidateFMove  ( void );
static void  dlgValueFromFMove ( float *x1, float *y1 );

static void  dlgInitSMove      ( MoveDlg *dial );
static void* dlgCreateSMove    ( void );
static void  dlgValueToSMove   ( float x1, float y1, float x2, float y2 );
static int   dlgValidateSMove  ( void );
static void  dlgValueFromSMove ( float *x1, float *y1,
                                 float *x2, float *y2 );

static int  callbackMoveDialogOk     ( Ihandle *bt );
static int  callbackMoveDialogCancel ( Ihandle *bt );
static int  callbackMoveDialogCloseCB( Ihandle *bt );

static int  callbackSMoveDialogOk     ( Ihandle *bt );
static int  callbackSMoveDialogCancel ( Ihandle *bt );
static int  callbackSMoveDialogCloseCB( Ihandle *bt );

static int  callbackFMoveDialogOk     ( Ihandle *bt );
static int  callbackFMoveDialogCancel ( Ihandle *bt );
static int  callbackFMoveDialogCloseCB( Ihandle *bt );

static MoveDlg  move;
static MoveDlg fmove;
static MoveDlg smove;

static char *size_butt = {"55x12"};
static char *size_text = {"65"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with translation data: " };
static char *llabel1e = { "DX:  " };
static char *llabel2e = { "DY:  " };
static char *llabel3e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel0p = { "Entre com as informaes de translao:" };
static char *llabel1p = { "DX:  " };
static char *llabel2p = { "DY:  " };
static char *llabel3p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

static char *llabel4e = { "Enter with first point coordinates:" };

static char *llabel4p = { "Entre com as coordenadas do ponto inicial:" };

static char *llabel5 = { "X1:  " };
static char *llabel6 = { "Y1:  " };
static char *llabel7 = { "X2: " };
static char *llabel8 = { "Y2: " };

char *llabel9e = { "Enter with translation axis coordinates:" };

char *llabel9p = { "Entre com as coordenadas do eixo de translao:" };
   

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitMove  ======================= */

static void dlgInitMove ( MoveDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e;    
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p;    
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateMove  ======================= */

static void* dlgCreateMove ( void )
{
  if ( move.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( move.pointbuttn1, "call_move_ok" );
    Ihandle *butt2 = IupButton ( move.pointbuttn2, "call_move_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( move.pointlabel1 );
    labl2 = IupLabel  ( move.pointlabel2 );
    labl3 = IupLabel  ( move.pointlabel3 );    
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );    
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );    
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );    
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  move.pointlabel0 );

    /* sets handles */
    IupSetHandle("dx_move1_handle", text1);
    IupSetHandle("dy_move1_handle", text2);    
    IupSetHandle("tl_move1_handle", text3);
    iupmaskSet(IupGetHandle("dx_move1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("dy_move1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_move1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    move.dialog = IupDialog (
                      IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_move_ok", (Icallback)callbackMoveDialogOk );
    IupSetFunction ( "call_move_cn", (Icallback)callbackMoveDialogCancel );
    IupSetFunction ( "call_move_close",(Icallback)callbackMoveDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( move.dialog, "GAP",    "3" );
    IupSetAttribute ( move.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( move.dialog, "MAXBOX", "NO");
    IupSetAttribute ( move.dialog, "RESIZE", "NO");
    IupSetAttribute ( move.dialog, "CLOSE_CB", "call_move_close");
#ifdef _UNIX_
    IupSetAttribute ( move.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( move.dialog, "TITLE",  "Move" );

    IupMap ( move.dialog );
  }

  return move.dialog;
}

/* ======================  dlgValueToMove  ======================= */

static void dlgValueToMove ( float dx, float dy, float tol )
{
 char cdx[50];
 char cdy[50]; 
 char ctl[50];

 if ( move.dialog == NULL ) return;

 sprintf(cdx, format, dx);
 sprintf(cdy, format, dy);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("dx_move1_handle"), "VALUE", cdx);
 IupSetAttribute (IupGetHandle("dy_move1_handle"), "VALUE", cdy);
 IupSetAttribute (IupGetHandle("tl_move1_handle"), "VALUE", ctl);
}

/* =====================  dlgValueFromMove  ====================== */

static void dlgValueFromMove ( float *dx, float *dy, float *tol )                               
{
  if ( move.dialog == NULL ) return;

  /* normal case */
  *dx  = IupGetFloat (IupGetHandle("dx_move1_handle"), "VALUE");
  *dy  = IupGetFloat (IupGetHandle("dy_move1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tl_move1_handle"), "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateMove  ======================= */

static int dlgValidateMove ( void )
{
 char *dx;
 char *dy;
 char *tl;

 if ( move.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(move.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 dx = IupGetAttribute (IupGetHandle("dx_move1_handle"), "VALUE");
 dy = IupGetAttribute (IupGetHandle("dy_move1_handle"), "VALUE");
 tl = IupGetAttribute (IupGetHandle("tl_move1_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitFMove  ==================== */

static void dlgInitFMove ( MoveDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel4e; 
   dial->pointlabel1 = llabel5; 
   dial->pointlabel2 = llabel6; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel4p; 
   dial->pointlabel1 = llabel5; 
   dial->pointlabel2 = llabel6; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
}

/* ====================  dlgCreateFMove  =================== */

static void* dlgCreateFMove ( void )
{
  if ( fmove.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( fmove.pointbuttn1, "call_fmove_ok" );
    Ihandle *butt2 = IupButton ( fmove.pointbuttn2, "call_fmove_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( fmove.pointlabel1 );
    labl2 = IupLabel  ( fmove.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  fmove.pointlabel0 );

    /* sets handles */
    IupSetHandle("x1_move3_handle", text1);
    IupSetHandle("y1_move3_handle", text2);
    iupmaskSet(IupGetHandle("x1_move3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_move3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    fmove.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_fmove_ok", (Icallback)callbackFMoveDialogOk );
    IupSetFunction ( "call_fmove_cn", (Icallback)callbackFMoveDialogCancel );
    IupSetFunction ( "call_fmove_close",(Icallback)callbackFMoveDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( fmove.dialog, "GAP",    "3" );
    IupSetAttribute ( fmove.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( fmove.dialog, "MAXBOX", "NO");
    IupSetAttribute ( fmove.dialog, "RESIZE", "NO");
    IupSetAttribute ( fmove.dialog, "CLOSE_CB", "call_fmove_close");
#ifdef _UNIX_
    IupSetAttribute ( fmove.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( fmove.dialog, "TITLE",  "Move" );

    IupMap ( fmove.dialog );
  }

  return fmove.dialog;
}

/* ===================  dlgValueToFMove  =================== */

static void dlgValueToFMove ( float x1, float y1 )
{
 char cx1[50];
 char cy1[50];

 if ( fmove.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);

 IupSetAttribute (IupGetHandle("x1_move3_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_move3_handle"), "VALUE", cy1);
}

/* =================  dlgValueFromFMove  =================== */

static void dlgValueFromFMove ( float *x1, float *y1 )
{
  if ( fmove.dialog == NULL ) return;

  *x1  = IupGetFloat (IupGetHandle("x1_move3_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_move3_handle"), "VALUE");
}

/* ==================  dlgValidateFMove  =================== */

static int dlgValidateFMove ( void )
{
 char *x1;
 char *y1;

 if ( fmove.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(fmove.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x1 = IupGetAttribute (IupGetHandle("x1_move3_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_move3_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitSMove  ==================== */

static void dlgInitSMove ( MoveDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel9e; 
   dial->pointlabel1 = llabel5; 
   dial->pointlabel2 = llabel6; 
   dial->pointlabel3 = llabel7; 
   dial->pointlabel4 = llabel8;
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel9p; 
   dial->pointlabel1 = llabel5; 
   dial->pointlabel2 = llabel6; 
   dial->pointlabel3 = llabel7; 
   dial->pointlabel4 = llabel8;
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateSMove  =================== */

static void* dlgCreateSMove ( void )
{
  if ( smove.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( smove.pointbuttn1, "call_smove_ok" );
    Ihandle *butt2 = IupButton ( smove.pointbuttn2, "call_smove_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( smove.pointlabel1 );
    labl2 = IupLabel  ( smove.pointlabel2 );
    labl3 = IupLabel  ( smove.pointlabel3 );
    labl4 = IupLabel  ( smove.pointlabel4 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);

    h4 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl4,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text4,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), 
	               h1, 
		       IupFill(), 
		       h2, 
		       IupFill(), 
		       h3, 
		       IupFill(),
		       h4, 
		       IupFill(), NULL )
		);

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  smove.pointlabel0);

    /* sets handles */
    IupSetHandle("x1_move2_handle", text1);
    IupSetHandle("y1_move2_handle", text2);
    IupSetHandle("x2_move2_handle", text3);
    IupSetHandle("y2_move2_handle", text4);
    iupmaskSet(IupGetHandle("x1_move2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_move2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x2_move2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y2_move2_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    smove.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_smove_ok", (Icallback)callbackSMoveDialogOk );
    IupSetFunction ( "call_smove_cn", (Icallback)callbackSMoveDialogCancel );
    IupSetFunction ( "call_smove_close",(Icallback)callbackSMoveDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( smove.dialog, "GAP",    "3" );
    IupSetAttribute ( smove.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( smove.dialog, "MAXBOX", "NO");
    IupSetAttribute ( smove.dialog, "RESIZE", "NO");
    IupSetAttribute ( smove.dialog, "CLOSE_CB", "call_smove_close");
#ifdef _UNIX_
    IupSetAttribute ( smove.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( smove.dialog, "TITLE",  "Move" );

    IupMap ( smove.dialog );
  }

  return smove.dialog;
}

/* ===================  dlgValueToSMove  =================== */

static void dlgValueToSMove ( float x1, float y1, float x2, float y2 )
{
 char cx1[50];
 char cy1[50];
 char cx2[50];
 char cy2[50];

 if ( smove.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(cx2, format, x2);
 sprintf(cy2, format, y2);

 IupSetAttribute (IupGetHandle("x1_move2_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_move2_handle"), "VALUE", cy1);
 IupSetAttribute (IupGetHandle("x2_move2_handle"), "VALUE", cx2);
 IupSetAttribute (IupGetHandle("y2_move2_handle"), "VALUE", cy2);
}

/* =================  dlgValueFromSMove  =================== */

static void dlgValueFromSMove ( float *x1, float *y1, float *x2, float *y2 )
{
  if ( smove.dialog == NULL ) return;

  *x1  = IupGetFloat (IupGetHandle("x1_move2_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_move2_handle"), "VALUE");
  *x2  = IupGetFloat (IupGetHandle("x2_move2_handle"), "VALUE");
  *y2  = IupGetFloat (IupGetHandle("y2_move2_handle"), "VALUE");
}

/* ==================  dlgValidateSMove  =================== */

static int dlgValidateSMove ( void )
{
 char *x1;
 char *y1;
 char *x2;
 char *y2;

 if ( smove.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(smove.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x1 = IupGetAttribute (IupGetHandle("x1_move2_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_move2_handle"), "VALUE");
 x2 = IupGetAttribute (IupGetHandle("x2_move2_handle"), "VALUE");
 y2 = IupGetAttribute (IupGetHandle("y2_move2_handle"), "VALUE");

 return 1;
}

/* =========================  callbackMoveDialogOk  ======================== */

static int callbackMoveDialogOk ( Ihandle *bt )
{
 IupSetAttribute(move.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackMoveDialogCancel  ====================== */

static int callbackMoveDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(move.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackMoveDialogCloseCB  ====================== */

static int callbackMoveDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(move.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackSMoveDialogOk  ====================== */

static int callbackSMoveDialogOk ( Ihandle *bt )
{
 IupSetAttribute(smove.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackSMoveDialogCancel  ==================== */

static int callbackSMoveDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(smove.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackSMoveDialogCloseCB  ====================== */

static int callbackSMoveDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(smove.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackFMoveDialogOk  ====================== */

static int callbackFMoveDialogOk ( Ihandle *bt )
{
 IupSetAttribute(fmove.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackFMoveDialogCancel  ==================== */

static int callbackFMoveDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(fmove.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackMoveDialogCloseCB  ====================== */

static int callbackFMoveDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(fmove.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureMove  ======================== */

int DlgCaptureMove ( float *dx, float *dy, float *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitMove( &move );

  /* create dialog to capture point data. */
  d = dlgCreateMove();

  /* sets point data to dialog. */
  dlgValueToMove ( *dx, *dy, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateMove ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromMove ( dx, dy, tol );

  return 1;
}

/* ======================  DlgCaptureSMove  ===================== */

int DlgCaptureSMove ( float *x1, float *y1, float *x2, float *y2 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitSMove( &smove );

  /* create dialog to capture point data. */
  d = dlgCreateSMove();

  /* sets point data to dialog. */
  dlgValueToSMove ( *x1, *y1, *x2, *y2 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateSMove ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromSMove ( x1, y1, x2, y2 );

  return 1;
}

/* ======================  DlgCaptureFMove  ===================== */

int DlgCaptureFMove ( float *x1, float *y1 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitFMove( &fmove );

  /* create dialog to capture point data. */
  d = dlgCreateFMove();

  /* sets point data to dialog. */
  dlgValueToFMove ( *x1, *y1 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateFMove ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromFMove ( x1, y1 );

  return 1;
}

