  
/*
** dlgpline.c - Functions that establish the pre-defined dialogs 
**              of the library to capture polylines.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureNPline( int *np, float *tol )
**
** np  - number of points of polygonal.           (IN/OUT)
** tol - tolerance used.                          (IN/OUT)
**
** This function creates a dialog to capture the number of points
** of polygonal. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureXPline( float *x0, float *y0 )
**
** x0 - coordinate (X direction) for each point.    (IN/OUT)
** y0 - coordinate (Y direction) for each point.    (IN/OUT)
**
** This function creates a dialog to capture the points of polygonal. 
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    23-Sep-01    William W. M. Lira.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgpline.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure */

typedef struct _pointdlg
{

  char *plinelabel0;
  char *plinelabel1;
  char *plinelabel2;
  char *plinelabel3;
  char *plinelabel4;
  char *plinelabel5;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} PLineDlg;


static void  dlgInitNPLine      ( PLineDlg *dial );
static void* dlgCreateNPLine    ( void );
static void  dlgValueToNPLine   ( int np, float tol );
static int   dlgValidateNPLine  ( void );
static void  dlgValueFromNPLine ( int *np, float *tol );

static void  dlgInitXPLine      ( PLineDlg *dial );
static void* dlgCreateXPLine    ( void );
static void  dlgValueToXPLine   ( float x0, float y0 );
static int   dlgValidateXPLine  ( void );
static void  dlgValueFromXPLine ( float *x0, float *y0 );

static int  callbackNPLineDialogOk     ( Ihandle *bt );
static int  callbackNPLineDialogCancel ( Ihandle *bt );
static int  callbackNPLineDialogCloseCB( Ihandle *bt );

static int  callbackXPLineDialogOk     ( Ihandle *bt );
static int  callbackXPLineDialogCancel ( Ihandle *bt );
static int  callbackXPLineDialogCloseCB( Ihandle *bt );

static PLineDlg npline;
static PLineDlg xpline;

static char *size_butt = {"55x12"};
static char *size_text = {"55"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with number of points:" };
static char *llabel1e = { "N. Points: " };
static char *llabel2e = { "Point X: " };
static char *llabel3e = { "Point Y: " };
static char *llabel4e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel0p = { "Entre com o numero de pontos:" };
static char *llabel1p = { "N. Pontos: " };
static char *llabel2p = { "Ponto X: " };
static char *llabel3p = { "Ponto Y: " };
static char *llabel4p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

static char *llabel5e = { "Enter with point coordinates:" }; 

static char *llabel5p = { "Entre com as coordenadas do ponto:" };
 
/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitNPLine  ======================= */

static void dlgInitNPLine ( PLineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->plinelabel0 = llabel0e; 
   dial->plinelabel1 = llabel1e; 
   dial->plinelabel4 = llabel4e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->plinelabel0 = llabel0p; 
   dial->plinelabel1 = llabel1p; 
   dial->plinelabel4 = llabel4p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateNPLine  ======================= */

static void* dlgCreateNPLine ( void )
{
  if ( npline.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( npline.pointbuttn1, "call_pline_ok" );
    Ihandle *butt2 = IupButton ( npline.pointbuttn2, "call_pline_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( npline.plinelabel1 );
    labl2 = IupLabel  ( npline.plinelabel4 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP",    "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  npline.plinelabel0 );

    /* sets handles */
    IupSetHandle("np_pline1_handle", text1);
    IupSetHandle("tl_pline1_handle", text2);
    iupmaskSet(IupGetHandle("np_pline1_handle"), IUPMASK_UINT, 0, 1);
    iupmaskSet(IupGetHandle("tl_pline1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    npline.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_pline_ok", (Icallback)callbackNPLineDialogOk );
    IupSetFunction ( "call_pline_cn", (Icallback)callbackNPLineDialogCancel );
    IupSetFunction ( "call_pline_close",(Icallback)callbackNPLineDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( npline.dialog, "GAP",    "3" );
    IupSetAttribute ( npline.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( npline.dialog, "MAXBOX", "NO");
    IupSetAttribute ( npline.dialog, "RESIZE", "NO");
    IupSetAttribute ( npline.dialog, "CLOSE_CB", "call_pline_close");
#ifdef _UNIX_
    IupSetAttribute ( npline.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( npline.dialog, "TITLE",  "PolyLine" );

    IupMap ( npline.dialog );
  }

  return npline.dialog;
}

/* ======================  dlgValueToNPLine  ======================= */

static void dlgValueToNPLine ( int np, float tol )
{
 char cnp[50];
 char ctl[50];

 if ( npline.dialog == NULL ) return;

 sprintf(cnp, "%d",    np);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("np_pline1_handle"), "VALUE", cnp);
 IupSetAttribute (IupGetHandle("tl_pline1_handle"), "VALUE", ctl);
}

/* =====================  dlgValueFromNPLine  ====================== */

static void dlgValueFromNPLine ( int *np, float *tol )
{
  if ( npline.dialog == NULL ) return;

  /* normal case */
  *np  = IupGetInt (IupGetHandle("np_pline1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tl_pline1_handle"), "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateNPLine  ======================= */

static int dlgValidateNPLine ( void )
{
 char *np;
 char *tl;

 if ( npline.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(npline.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 np = IupGetAttribute (IupGetHandle("np_pline1_handle"), "VALUE");
 tl = IupGetAttribute (IupGetHandle("tl_pline1_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitXPLine  ==================== */

static void dlgInitXPLine ( PLineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->plinelabel0 = llabel5e; 
   dial->plinelabel2 = llabel2e; 
   dial->plinelabel3 = llabel3e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->plinelabel0 = llabel5p; 
   dial->plinelabel2 = llabel2p; 
   dial->plinelabel3 = llabel3p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateXPLine  =================== */

static void* dlgCreateXPLine ( void )
{
  if ( xpline.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( xpline.pointbuttn1, "call_xpline_ok" );
    Ihandle *butt2 = IupButton ( xpline.pointbuttn2, "call_xpline_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( xpline.plinelabel2 );
    labl2 = IupLabel  ( xpline.plinelabel3 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  xpline.plinelabel0 );

    /* sets handles */
    IupSetHandle("x0_pline2_handle", text1);
    IupSetHandle("y0_pline2_handle", text2);
    iupmaskSet(IupGetHandle("x0_pline2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_pline2_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    xpline.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_xpline_ok", (Icallback)callbackXPLineDialogOk );
    IupSetFunction ( "call_xpline_cn", (Icallback)callbackXPLineDialogCancel );
    IupSetFunction ( "call_xpline_close", (Icallback)callbackXPLineDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( xpline.dialog, "GAP",    "3" );
    IupSetAttribute ( xpline.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( xpline.dialog, "MAXBOX", "NO");
    IupSetAttribute ( xpline.dialog, "RESIZE", "NO");
    IupSetAttribute ( xpline.dialog, "CLOSE_CB", "call_xpline_close");
#ifdef _UNIX_
    IupSetAttribute ( xpline.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( xpline.dialog, "TITLE",  "PolyLine" );

    IupMap ( xpline.dialog );
  }

  return xpline.dialog;
}

/* ===================  dlgValueToXPLine  =================== */

static void dlgValueToXPLine ( float x0, float y0 )
{
 char cx0[50];
 char cy0[50];

 if ( xpline.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);

 IupSetAttribute (IupGetHandle("x0_pline2_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_pline2_handle"), "VALUE", cy0);
}

/* =================  dlgValueFromXPLine  =================== */

static void dlgValueFromXPLine ( float *x0, float *y0 )
{
  if ( xpline.dialog == NULL ) return;

  *x0  = IupGetFloat (IupGetHandle("x0_pline2_handle"), "VALUE");
  *y0  = IupGetFloat (IupGetHandle("y0_pline2_handle"), "VALUE");
}

/* ==================  dlgValidateXPLine  =================== */

static int dlgValidateXPLine ( void )
{
 char *x0;
 char *y0;

 if ( xpline.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(xpline.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x0 = IupGetAttribute (IupGetHandle("x0_pline2_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_pline2_handle"), "VALUE");

 return 1;
}

/* =========================  callbackNPLineDialogOk  ======================= */

static int callbackNPLineDialogOk ( Ihandle *bt )
{
 IupSetAttribute(npline.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackNPLineDialogCancel  ===================== */

static int callbackNPLineDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(npline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackNPLineDialogCloseCB  ===================== */

static int callbackNPLineDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(npline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackXPLineDialogOk  ====================== */

static int callbackXPLineDialogOk ( Ihandle *bt )
{
 IupSetAttribute(xpline.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackXPLineDialogCancel  ==================== */

static int callbackXPLineDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(xpline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackXPLineDialogCloseCB  ===================== */

static int callbackXPLineDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(xpline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureNPLine  ======================== */

int DlgCaptureNPLine ( int *np, float *tol )
{
  Ihandle *d;

  /* initializes dialog. */
  dlgInitNPLine( &npline );

  /* create dialog to capture the number of points of polygonal. */
  d = dlgCreateNPLine();

  /* sets polygonal data to dialog. */
  dlgValueToNPLine ( *np, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks polygonal data. */
  if ( dlgValidateNPLine ( ) == 0 ) return 0;

  /* get polygonal data from dialog. */
  dlgValueFromNPLine ( np, tol );

  return 1;
}

/* ======================  DlgCaptureXPLine  ===================== */

int DlgCaptureXPLine ( float *x0, float *y0 )
{
  Ihandle *d;

  /* initializes dialog. */
  dlgInitXPLine( &xpline );

  /* create dialog to capture points of polygonal. */
  d = dlgCreateXPLine();

  /* sets polygonal data to dialog. */
  dlgValueToXPLine ( *x0, *y0 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks polygonal data. */
  if ( dlgValidateXPLine ( ) == 0 ) return 0;

  /* gets polygonal data from dialog. */
  dlgValueFromXPLine ( x0, y0 );

  return 1;
}

