  
/*
** dlgpnt.c - Functions that establish the pre-defined dialogs 
**            of the library to capture points.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCapturePoint( float *x0, float *y0, float *tol )
**
** x0  - coordinate (X direction) of point.       (IN/OUT)
** y0  - coordinate (Y direction) of point.       (IN/OUT)
** tol - tolerance used.                          (IN/OUT)
**
** This function creates a dialog to capture the parameters of a point. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapturePPoint( float *x0, float *y0 )
**
** x0  - coordinate (X direction) of point.       (IN/OUT)
** y0  - coordinate (Y direction) of point.       (IN/OUT)
**
** This function creates a dialog to capture the parameters of a point. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    10-Sep-01    William W. M. Lira.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgpnt.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store point dialog */

typedef struct _pointdlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} PointDlg;


static void  dlgInitPoint      ( PointDlg *dial );
static void* dlgCreatePoint    ( void );
static void  dlgValueToPoint   ( float x0, float y0, float tol );
static int   dlgValidatePoint  ( void );
static void  dlgValueFromPoint ( float *x0, float *y0, float *tol );

static void  dlgInitPPoint      ( PointDlg *dial );
static void* dlgCreatePPoint    ( void );
static void  dlgValueToPPoint   ( float x0, float y0 );
static int   dlgValidatePPoint  ( void );
static void  dlgValueFromPPoint ( float *x0, float *y0 );

static int  callbackPointDialogOk     ( Ihandle *bt );
static int  callbackPointDialogCancel ( Ihandle *bt );
static int  callbackPointDialogCloseCB( Ihandle *bt );

static int  callbackPPointDialogOk     ( Ihandle *bt );
static int  callbackPPointDialogCancel ( Ihandle *bt );
static int  callbackPPointDialogCloseCB( Ihandle *bt );

static PointDlg  pnt;
static PointDlg ppnt;

static char *size_butt = {"55x12"};
static char *size_text = {"55"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with point coordinates:" };
static char *llabel1e = { "Point X:   " };
static char *llabel2e = { "Point Y:   " };
static char *llabel3e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel0p = { "Entre com as coordenadas do ponto:" };
static char *llabel1p = { "Ponto X:   " };
static char *llabel2p = { "Ponto Y:   " };
static char *llabel3p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitPoint  ======================= */

static void dlgInitPoint ( PointDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreatePoint  ======================= */

static void* dlgCreatePoint ( void )
{
  if ( pnt.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( pnt.pointbuttn1, "call_pnt_ok" );
    Ihandle *butt2 = IupButton ( pnt.pointbuttn2, "call_pnt_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( pnt.pointlabel1 );
    labl2 = IupLabel  ( pnt.pointlabel2 );
    labl3 = IupLabel  ( pnt.pointlabel3 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP",    "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  pnt.pointlabel0 );

    /* sets handles */
    IupSetHandle("x0_pnt1_handle", text1);
    IupSetHandle("y0_pnt1_handle", text2);
    IupSetHandle("tl_pnt1_handle", text3);
    iupmaskSet(IupGetHandle("x0_pnt1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_pnt1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_pnt1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    pnt.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_pnt_ok", (Icallback)callbackPointDialogOk );
    IupSetFunction ( "call_pnt_cn", (Icallback)callbackPointDialogCancel );
    IupSetFunction ( "call_pnt_close",(Icallback)callbackPointDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( pnt.dialog, "GAP",    "3" );
    IupSetAttribute ( pnt.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( pnt.dialog, "MAXBOX", "NO");
    IupSetAttribute ( pnt.dialog, "RESIZE", "NO");
    IupSetAttribute ( pnt.dialog, "CLOSE_CB", "call_pnt_close");
#ifdef _UNIX_
    IupSetAttribute ( pnt.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( pnt.dialog, "TITLE",  "Point" );

    IupMap ( pnt.dialog );
  }

  return pnt.dialog;
}

/* ======================  dlgValueToPoint  ======================= */

static void dlgValueToPoint ( float x0, float y0, float tol )
{
 char cx0[50];
 char cy0[50];
 char ctl[50];

 if ( pnt.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("x0_pnt1_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_pnt1_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("tl_pnt1_handle"), "VALUE", ctl);
}

/* =====================  dlgValueFromPoint  ====================== */

static void dlgValueFromPoint ( float *x0, float *y0, float *tol )
{
  if ( pnt.dialog == NULL ) return;

  /* normal case */
  *x0  = IupGetFloat (IupGetHandle("x0_pnt1_handle"), "VALUE");
  *y0  = IupGetFloat (IupGetHandle("y0_pnt1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tl_pnt1_handle"), "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidatePoint  ======================= */

static int dlgValidatePoint ( void )
{
 char *x0;
 char *y0;
 char *tl;

 if ( pnt.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(pnt.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 x0 = IupGetAttribute (IupGetHandle("x0_pnt1_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_pnt1_handle"), "VALUE");
 tl = IupGetAttribute (IupGetHandle("tl_pnt1_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitPPoint  ==================== */

static void dlgInitPPoint ( PointDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreatePPoint  =================== */

static void* dlgCreatePPoint ( void )
{
  if ( ppnt.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( ppnt.pointbuttn1, "call_ppnt_ok" );
    Ihandle *butt2 = IupButton ( ppnt.pointbuttn2, "call_ppnt_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( ppnt.pointlabel1 );
    labl2 = IupLabel  ( ppnt.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  ppnt.pointlabel0 );

    /* sets handles */
    IupSetHandle("x0_pnt2_handle", text1);
    IupSetHandle("y0_pnt2_handle", text2);
    iupmaskSet(IupGetHandle("x0_pnt2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_pnt2_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    ppnt.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_ppnt_ok", (Icallback)callbackPPointDialogOk );
    IupSetFunction ( "call_ppnt_cn", (Icallback)callbackPPointDialogCancel );
    IupSetFunction ( "call_ppnt_close",(Icallback)callbackPPointDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( ppnt.dialog, "GAP",    "3" );
    IupSetAttribute ( ppnt.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( ppnt.dialog, "MAXBOX", "NO");
    IupSetAttribute ( ppnt.dialog, "RESIZE", "NO");
    IupSetAttribute ( ppnt.dialog, "CLOSE_CB", "call_ppnt_close");
#ifdef _UNIX_
    IupSetAttribute ( ppnt.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( ppnt.dialog, "TITLE",  "Point" );

    IupMap ( ppnt.dialog );
  }

  return ppnt.dialog;
}

/* ===================  dlgValueToPPoint  =================== */

static void dlgValueToPPoint ( float x0, float y0 )
{
 char cx0[50];
 char cy0[50];

 if ( ppnt.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);

 IupSetAttribute (IupGetHandle("x0_pnt2_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_pnt2_handle"), "VALUE", cy0);
}

/* =================  dlgValueFromPPoint  =================== */

static void dlgValueFromPPoint ( float *x0, float *y0 )
{
  if ( ppnt.dialog == NULL ) return;

  *x0  = IupGetFloat (IupGetHandle("x0_pnt2_handle"), "VALUE");
  *y0  = IupGetFloat (IupGetHandle("y0_pnt2_handle"), "VALUE");
}

/* ==================  dlgValidatePPoint  =================== */

static int dlgValidatePPoint ( void )
{
 char *x0;
 char *y0;

 if ( ppnt.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(ppnt.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x0 = IupGetAttribute (IupGetHandle("x0_pnt2_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_pnt2_handle"), "VALUE");

 return 1;
}

/* =========================  callbackPointDialogOk  ======================== */

static int callbackPointDialogOk ( Ihandle *bt )
{
 IupSetAttribute(pnt.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackPointDialogCancel  ====================== */

static int callbackPointDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(pnt.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackPointDialogCloseCB  ====================== */

static int callbackPointDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(pnt.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackPPointDialogOk  ====================== */

static int callbackPPointDialogOk ( Ihandle *bt )
{
 IupSetAttribute(ppnt.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackPPointDialogCancel  ==================== */

static int callbackPPointDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(ppnt.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackPPointDialogCloseCB  ====================== */

static int callbackPPointDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(ppnt.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCapturePoint  ======================== */

int DlgCapturePoint ( float *x0, float *y0, float *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitPoint( &pnt );

  /* create dialog to capture point data. */
  d = dlgCreatePoint();

  /* sets point data to dialog. */
  dlgValueToPoint ( *x0, *y0, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidatePoint ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromPoint ( x0, y0, tol );

  return 1;
}

/* ======================  DlgCapturePPoint  ===================== */

int DlgCapturePPoint ( float *x0, float *y0 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitPPoint( &ppnt );

  /* create dialog to capture point data. */
  d = dlgCreatePPoint();

  /* sets point data to dialog. */
  dlgValueToPPoint ( *x0, *y0 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidatePPoint ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromPPoint ( x0, y0 );

  return 1;
}

