
/*
** dlgrotat.c - Functions that establish the pre-defined dialogs 
**              of the library to rotate objects.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureRotate( *cx, *cy, *ang, *toler )
**
** cx    - coordinate (X direction) of rotation center.    (IN/OUT)
** cy    - coordinate (Y direction) of rotation center.    (IN/OUT)
** ang   - rotation angle.                                 (IN/OUT)
** toler - tolerance used.                                 (IN/OUT)
**
** This function creates a dialog to capture the parameters to 
** rotate an image. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureFRotate( float *cx, float *cy )
**
** cx  - coordinate (X direction) of rotation center.      (IN/OUT)
** cy  - coordinate (Y direction) of rotation center.      (IN/OUT)
**
** This function creates a dialog to capture the parameters of the 
** rotation center. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureSRotate( float *x1, float *y1 )
**
** x1  - coordinate (X direction) of arc first point.      (IN/OUT)
** y1  - coordinate (Y direction) of arc first point.      (IN/OUT)
**
** This function creates a dialog to capture the parameters of the 
** arc first point.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureTRotate( float *cx, float *cy, float *x1, float *y1, 
**                        float *x2, float *y2 )
**
** cx  - coordinate (X direction) of rotation center.      (IN/OUT)
** cy  - coordinate (Y direction) of rotation center.      (IN/OUT)
** x1  - coordinate (X direction) of arc first point.      (IN/OUT)
** y1  - coordinate (Y direction) of arc first point.      (IN/OUT)
** x2  - coordinate (X direction) of arc second point.     (IN/OUT)
** y2  - coordinate (Y direction) of arc second point.     (IN/OUT)
**
** This function creates a dialog to capture the parameters of arc 
** second point. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    08-Aug-02    Marcos C. de Arruda.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgrotat.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store point dialog */

typedef struct _rotatedlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;
  char *pointlabel5;
  char *pointlabel6;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} RotateDlg;


static void  dlgInitRotate      ( RotateDlg *dial );
static void* dlgCreateRotate    ( void );
static void  dlgValueToRotate   ( float cx,  float cy, 
                                  float ang, float tol );
static int   dlgValidateRotate  ( void );
static void  dlgValueFromRotate ( float *cx,  float *cy, 
                                  float *ang, float *tol );

static void  dlgInitFRotate      ( RotateDlg *dial );
static void* dlgCreateFRotate    ( void );
static void  dlgValueToFRotate   ( float cx, float cy );
static int   dlgValidateFRotate  ( void );
static void  dlgValueFromFRotate ( float *cx, float *cy );

static void  dlgInitSRotate      ( RotateDlg *dial );
static void* dlgCreateSRotate    ( void );
static void  dlgValueToSRotate   ( float x1, float y1 );
static int   dlgValidateSRotate  ( void );
static void  dlgValueFromSRotate ( float *x1, float *y1 );

static void  dlgInitTRotate      ( RotateDlg *dial );
static void* dlgCreateTRotate    ( void );
static void  dlgValueToTRotate   ( float cx, float cy,
                                   float x1, float y1,
                                   float x2, float y2 );
static int   dlgValidateTRotate  ( void );
static void  dlgValueFromTRotate ( float *cx, float *cy,
                                   float *x1, float *y1,
                                   float *x2, float *y2 );

static int  callbackRotateDialogOk     ( Ihandle *bt );
static int  callbackRotateDialogCancel ( Ihandle *bt );
static int  callbackRotateDialogCloseCB( Ihandle *bt );

static int  callbackFRotateDialogOk     ( Ihandle *bt );
static int  callbackFRotateDialogCancel ( Ihandle *bt );
static int  callbackFRotateDialogCloseCB( Ihandle *bt );

static int  callbackSRotateDialogOk     ( Ihandle *bt );
static int  callbackSRotateDialogCancel ( Ihandle *bt );
static int  callbackSRotateDialogCloseCB( Ihandle *bt );

static int  callbackTRotateDialogOk     ( Ihandle *bt );
static int  callbackTRotateDialogCancel ( Ihandle *bt );
static int  callbackTRotateDialogCloseCB( Ihandle *bt );

static RotateDlg  rotate;
static RotateDlg frotate;
static RotateDlg srotate;
static RotateDlg trotate;

static char *size_butt = {"55x12"};
static char *size_text = {"65"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with rotation parameters: " };
static char *llabel1e = { "Center X:  " };
static char *llabel2e = { "Center Y:  " };
static char *llabel3e = { "Rotation angle:  " };
static char *llabel4e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel5e  = { "Enter with rotation center coordinates:" };

static char *llabel6e  = { "Enter with arc first point coordinates:" };

static char *llabel11e = { "Enter with coordinates of rotation parameters:" };

static char *llabel0p = { "Entre com os parmetros de rotao :" };
static char *llabel1p = { "Centro X:  " };
static char *llabel2p = { "Centro Y:  " };
static char *llabel3p = { "ngulo de rotao:  " };
static char *llabel4p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

static char *llabel5p = { "Entre com as coordenadas do centro de rotao:" };

static char *llabel6p = { "Entre com as coordenadas do ponto inicial do arco:" };

static char *llabel11p = { "Entre com as coordenadas dos parmetros de rotao:" };

static char *llabel7  = { "X1:  " };
static char *llabel8  = { "Y1:  " };
static char *llabel9  = { "X2:  " };
static char *llabel10 = { "Y2:  " };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitRotate  ======================= */

static void dlgInitRotate ( RotateDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointlabel4 = llabel4e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p;    
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateRotate  ======================= */

static void* dlgCreateRotate ( void )
{
  if ( rotate.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( rotate.pointbuttn1, "call_rotate_ok" );
    Ihandle *butt2 = IupButton ( rotate.pointbuttn2, "call_rotate_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;	

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( rotate.pointlabel1 );
    labl2 = IupLabel  ( rotate.pointlabel2 );
    labl3 = IupLabel  ( rotate.pointlabel3 );
    labl4 = IupLabel  ( rotate.pointlabel4 );    
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    
    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );	
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );	
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );	
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  rotate.pointlabel0 );

    /* sets handles */
    IupSetHandle("cx_rotat1_handle",  text1);
    IupSetHandle("cy_rotat1_handle",  text2);
    IupSetHandle("ang_rotat1_handle", text3);    
    IupSetHandle("tl_rotat1_handle",  text4);
    iupmaskSet(IupGetHandle("cx_rotat1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("cy_rotat1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("ang_rotat1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_rotat1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    rotate.dialog = IupDialog (
                      IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_rotate_ok", (Icallback)callbackRotateDialogOk );
    IupSetFunction ( "call_rotate_cn", (Icallback)callbackRotateDialogCancel );
    IupSetFunction ( "call_rotate_close", (Icallback)callbackRotateDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( rotate.dialog, "GAP",    "3" );
    IupSetAttribute ( rotate.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( rotate.dialog, "MAXBOX", "NO");
    IupSetAttribute ( rotate.dialog, "RESIZE", "NO");
    IupSetAttribute ( rotate.dialog, "CLOSE_CB", "call_rotate_close");
#ifdef _UNIX_
    IupSetAttribute ( rotate.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( rotate.dialog, "TITLE",  "Rotate" );

    IupMap ( rotate.dialog );
  }

  return rotate.dialog;
}

/* ======================  dlgValueToRotate  ======================= */

static void dlgValueToRotate ( float cx, float cy, float ang, float tol )
{
 char cx1[50];
 char cy1[50];
 char cang[50]; 
 char ctl[50];

 if ( rotate.dialog == NULL ) return;

 sprintf(cx1, format, cx);
 sprintf(cy1, format, cy);
 sprintf(cang, format, ang); 
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("cx_rotat1_handle"),  "VALUE", cx1);
 IupSetAttribute (IupGetHandle("cy_rotat1_handle"),  "VALUE", cy1);
 IupSetAttribute (IupGetHandle("ang_rotat1_handle"), "VALUE", cang); 
 IupSetAttribute (IupGetHandle("tl_rotat1_handle"),  "VALUE", ctl);
}

/* =====================  dlgValueFromRotate  ====================== */

static void dlgValueFromRotate ( float *cx, float *cy, 
                                 float *ang, float *tol )
{
  if ( rotate.dialog == NULL ) return;

  /* normal case */
  *cx  = IupGetFloat (IupGetHandle("cx_rotat1_handle"),  "VALUE");
  *cy  = IupGetFloat (IupGetHandle("cy_rotat1_handle"),  "VALUE");
  *ang = IupGetFloat (IupGetHandle("ang_rotat1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tl_rotat1_handle"),  "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateRotate  ======================= */

static int dlgValidateRotate ( void )
{
 char *cx;
 char *cy;
 char *ang; 
 char *tl;

 if ( rotate.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(rotate.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 cx = IupGetAttribute (IupGetHandle("cx_rotat1_handle"),  "VALUE");
 cy = IupGetAttribute (IupGetHandle("cy_rotat1_handle"),  "VALUE");
 ang = IupGetAttribute (IupGetHandle("ang_rotat1_handle"), "VALUE"); 
 tl = IupGetAttribute (IupGetHandle("tl_rotat1_handle"),  "VALUE");

 return 1;
}

/* ====================  dlgInitFRotate  ==================== */

static void dlgInitFRotate ( RotateDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel5e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel5p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateFRotate  =================== */

static void* dlgCreateFRotate ( void )
{
  if ( frotate.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( frotate.pointbuttn1, "call_frotate_ok" );
    Ihandle *butt2 = IupButton ( frotate.pointbuttn2, "call_frotate_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( frotate.pointlabel1 );
    labl2 = IupLabel  ( frotate.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  frotate.pointlabel0 );

    /* sets handles */
    IupSetHandle("cx_rotat2_handle", text1);
    IupSetHandle("cy_rotat2_handle", text2);
    iupmaskSet(IupGetHandle("cx_rotat2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("cy_rotat2_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    frotate.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_frotate_ok", (Icallback)callbackFRotateDialogOk );
    IupSetFunction ( "call_frotate_cn", (Icallback)callbackFRotateDialogCancel);
    IupSetFunction ( "call_frotate_close", (Icallback)callbackFRotateDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( frotate.dialog, "GAP",    "3" );
    IupSetAttribute ( frotate.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( frotate.dialog, "MAXBOX", "NO");
    IupSetAttribute ( frotate.dialog, "RESIZE", "NO");
    IupSetAttribute ( frotate.dialog, "CLOSE_CB", "call_frotate_close");
#ifdef _UNIX_
    IupSetAttribute ( frotate.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( frotate.dialog, "TITLE",  "Rotate" );

    IupMap ( frotate.dialog );
  }

  return frotate.dialog;
}

/* ===================  dlgValueToFRotate  =================== */

static void dlgValueToFRotate ( float cx, float cy )
{
 char cx1[50];
 char cy1[50];

 if ( frotate.dialog == NULL ) return;

 sprintf(cx1, format, cx);
 sprintf(cy1, format, cy);

 IupSetAttribute (IupGetHandle("cx_rotat2_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("cy_rotat2_handle"), "VALUE", cy1);
}

/* =================  dlgValueFromFRotate  =================== */

static void dlgValueFromFRotate ( float *cx, float *cy )
{
  if ( frotate.dialog == NULL ) return;
 
  *cx  = IupGetFloat (IupGetHandle("cx_rotat2_handle"), "VALUE");
  *cy  = IupGetFloat (IupGetHandle("cy_rotat2_handle"), "VALUE");
}

/* ==================  dlgValidateFRotate  =================== */

static int dlgValidateFRotate ( void )
{
 char *cx;
 char *cy;

 if ( frotate.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(frotate.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 cx = IupGetAttribute (IupGetHandle("cx_rotat2_handle"), "VALUE");
 cy = IupGetAttribute (IupGetHandle("cy_rotat2_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitSRotate  ==================== */

static void dlgInitSRotate ( RotateDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel6e;
   dial->pointlabel1 = llabel7;
   dial->pointlabel2 = llabel8;
   dial->pointbuttn1 = lbuttn1e;
   dial->pointbuttn2 = lbuttn2e;
 }
 else
 {
   dial->pointlabel0 = llabel6p;
   dial->pointlabel1 = llabel7;
   dial->pointlabel2 = llabel8;
   dial->pointbuttn1 = lbuttn1p;
   dial->pointbuttn2 = lbuttn2p;
 }
}

/* ====================  dlgCreateSRotate  =================== */

static void* dlgCreateSRotate ( void )
{
  if ( srotate.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( srotate.pointbuttn1, "call_srotate_ok" );
    Ihandle *butt2 = IupButton ( srotate.pointbuttn2, "call_srotate_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( srotate.pointlabel1 );
    labl2 = IupLabel  ( srotate.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl1,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text1,IupFill(),NULL),
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl2,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text2,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  srotate.pointlabel0 );

    /* sets handles */
    IupSetHandle("x1_rotat3_handle", text1);
    IupSetHandle("y1_rotat3_handle", text2);
    iupmaskSet(IupGetHandle("x1_rotat3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_rotat3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    srotate.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_srotate_ok", (Icallback)callbackSRotateDialogOk );
    IupSetFunction ( "call_srotate_cn", (Icallback)callbackSRotateDialogCancel);
    IupSetFunction ( "call_srotate_close", (Icallback)callbackSRotateDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( srotate.dialog, "GAP",    "3" );
    IupSetAttribute ( srotate.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( srotate.dialog, "MAXBOX", "NO");
    IupSetAttribute ( srotate.dialog, "RESIZE", "NO");
    IupSetAttribute ( srotate.dialog, "CLOSE_CB", "call_srotate_close");
#ifdef _UNIX_
    IupSetAttribute ( srotate.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( srotate.dialog, "TITLE",  "Rotate" );

    IupMap ( srotate.dialog );
  }

  return srotate.dialog;
}

/* ===================  dlgValueToSRotate  =================== */

static void dlgValueToSRotate ( float x1, float y1 )
{
 char cx1[50];
 char cy1[50];

 if ( frotate.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);

 IupSetAttribute (IupGetHandle("x1_rotat3_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_rotat3_handle"), "VALUE", cy1);
}

/* =================  dlgValueFromSRotate  =================== */

static void dlgValueFromSRotate ( float *x1, float *y1 )
{
  if ( srotate.dialog == NULL ) return;

  *x1  = IupGetFloat (IupGetHandle("x1_rotat3_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_rotat3_handle"), "VALUE");
}

/* ==================  dlgValidateSRotate  =================== */

static int dlgValidateSRotate ( void )
{
 char *x1;
 char *y1;

 if ( srotate.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(srotate.dialog, "CONFIRM"), "1") != 0 )
    return 0;

 x1 = IupGetAttribute (IupGetHandle("x1_rotat3_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_rotat3_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitTRotate  ==================== */

static void dlgInitTRotate ( RotateDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel11e;
   dial->pointlabel1 = llabel1e;
   dial->pointlabel2 = llabel2e;
   dial->pointlabel3 = llabel7;
   dial->pointlabel4 = llabel8;
   dial->pointlabel5 = llabel9;
   dial->pointlabel6 = llabel10;
   dial->pointbuttn1 = lbuttn1e;
   dial->pointbuttn2 = lbuttn2e;
 }
 else
 {
   dial->pointlabel0 = llabel11p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel7; 
   dial->pointlabel4 = llabel8;
   dial->pointlabel5 = llabel9;
   dial->pointlabel6 = llabel10;
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateTRotate  =================== */

static void* dlgCreateTRotate ( void )
{
  if ( trotate.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( trotate.pointbuttn1, "call_trotate_ok" );
    Ihandle *butt2 = IupButton ( trotate.pointbuttn2, "call_trotate_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;
    Ihandle *text6 = NULL; 

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;
    Ihandle *h6 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( trotate.pointlabel1 );
    labl2 = IupLabel  ( trotate.pointlabel2 );
    labl3 = IupLabel  ( trotate.pointlabel3 );
    labl4 = IupLabel  ( trotate.pointlabel4 );
    labl5 = IupLabel  ( trotate.pointlabel5 );
    labl6 = IupLabel  ( trotate.pointlabel6 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );
    text6 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);

    h4 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl4,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text4,IupFill(),NULL),
		  IupFill(), NULL);

    h5 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl5,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text5,IupFill(),NULL),
		  IupFill(), NULL);

    h6 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl6,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text6,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), 
	               h1, 
		       IupFill(), 
		       h2, 
		       IupFill(), 
		       h3, 
		       IupFill(),
		       h4, 
		       IupFill(),
		       h5, 
		       IupFill(),
		       h6,
		       IupFill(), NULL )
		);

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text ); 

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  trotate.pointlabel0);

    /* sets handles */
    IupSetHandle("cx_rotat4_handle", text1);
    IupSetHandle("cy_rotat4_handle", text2);
    IupSetHandle("x1_rotat4_handle", text3);
    IupSetHandle("y1_rotat4_handle", text4);
    IupSetHandle("x2_rotat4_handle", text5);
    IupSetHandle("y2_rotat4_handle", text6);
    iupmaskSet(IupGetHandle("cx_rotat4_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("cy_rotat4_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x1_rotat4_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_rotat4_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x2_rotat4_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y2_rotat4_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    trotate.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_trotate_ok", (Icallback)callbackTRotateDialogOk );
    IupSetFunction ( "call_trotate_cn", (Icallback)callbackTRotateDialogCancel);
    IupSetFunction ( "call_trotate_close", (Icallback)callbackTRotateDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( trotate.dialog, "GAP",    "3" );
    IupSetAttribute ( trotate.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( trotate.dialog, "MAXBOX", "NO");
    IupSetAttribute ( trotate.dialog, "RESIZE", "NO");
    IupSetAttribute ( trotate.dialog, "CLOSE_CB", "call_trotate_close");
#ifdef _UNIX_
    IupSetAttribute ( trotate.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( trotate.dialog, "TITLE",  "Rotate" );

    IupMap ( trotate.dialog );
  }

  return trotate.dialog;
}

/* ===================  dlgValueToTRotate  =================== */

static void dlgValueToTRotate ( float cx, float cy, float x1, float y1, 
                                float x2, float y2 )
{
 char ccx[50];
 char ccy[50];
 char cx1[50];
 char cy1[50];
 char cx2[50];
 char cy2[50];

 if ( trotate.dialog == NULL ) return;

 sprintf(ccx, format, cx);
 sprintf(ccy, format, cy);
 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(cx2, format, x2);
 sprintf(cy2, format, y2);

 IupSetAttribute (IupGetHandle("cx_rotat4_handle"), "VALUE", ccx);
 IupSetAttribute (IupGetHandle("cy_rotat4_handle"), "VALUE", ccy);
 IupSetAttribute (IupGetHandle("x1_rotat4_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_rotat4_handle"), "VALUE", cy1);
 IupSetAttribute (IupGetHandle("x2_rotat4_handle"), "VALUE", cx2);
 IupSetAttribute (IupGetHandle("y2_rotat4_handle"), "VALUE", cy2);
}

/* =================  dlgValueFromTRotate  =================== */

static void dlgValueFromTRotate ( float *cx, float *cy, float *x1, float *y1, 
                                  float *x2, float *y2 )
{
  if ( trotate.dialog == NULL ) return;
  
  *cx  = IupGetFloat (IupGetHandle("cx_rotat4_handle"), "VALUE");
  *cy  = IupGetFloat (IupGetHandle("cy_rotat4_handle"), "VALUE");
  *x1  = IupGetFloat (IupGetHandle("x1_rotat4_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_rotat4_handle"), "VALUE");
  *x2  = IupGetFloat (IupGetHandle("x2_rotat4_handle"), "VALUE");
  *y2  = IupGetFloat (IupGetHandle("y2_rotat4_handle"), "VALUE");
}

/* ==================  dlgValidateTRotate  =================== */

static int dlgValidateTRotate ( void )
{
 char *cx;
 char *cy;
 char *x1;
 char *y1;
 char *x2;
 char *y2;

 if ( trotate.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(trotate.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 cx = IupGetAttribute (IupGetHandle("cx_rotat4_handle"), "VALUE");
 cy = IupGetAttribute (IupGetHandle("cy_rotat4_handle"), "VALUE");
 x1 = IupGetAttribute (IupGetHandle("x1_rotat4_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_rotat4_handle"), "VALUE");
 x2 = IupGetAttribute (IupGetHandle("x2_rotat4_handle"), "VALUE");
 y2 = IupGetAttribute (IupGetHandle("y2_rotat4_handle"), "VALUE");

 return 1;
}

/* =========================  callbackRotateDialogOk  ======================== */

static int callbackRotateDialogOk ( Ihandle *bt )
{
 IupSetAttribute(rotate.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackRotateDialogCancel  ====================== */

static int callbackRotateDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(rotate.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackRotateDialogCloseCB  ====================== */

static int callbackRotateDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(rotate.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackFRotateDialogOk  ====================== */

static int callbackFRotateDialogOk ( Ihandle *bt )
{
 IupSetAttribute(frotate.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackFRotateDialogCancel  ==================== */

static int callbackFRotateDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(frotate.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackFRotateDialogCloseCB  ====================== */

static int callbackFRotateDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(frotate.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackSRotateDialogOk  ====================== */

static int callbackSRotateDialogOk ( Ihandle *bt )
{
 IupSetAttribute(srotate.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackSRotateDialogCancel  ==================== */

static int callbackSRotateDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(srotate.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackSRotateDialogCloseCB  ====================== */

static int callbackSRotateDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(srotate.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackTRotateDialogOk  ====================== */

static int callbackTRotateDialogOk ( Ihandle *bt )
{
 IupSetAttribute(trotate.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackTRotateDialogCancel  ==================== */

static int callbackTRotateDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(trotate.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackTRotateDialogCloseCB  ====================== */

static int callbackTRotateDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(trotate.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureRotate  ======================== */

int DlgCaptureRotate ( float *cx, float *cy, float *ang, float *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitRotate( &rotate );

  /* create dialog to capture point data. */
  d = dlgCreateRotate();

  /* sets point data to dialog. */
  dlgValueToRotate ( *cx, *cy, *ang, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateRotate ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromRotate ( cx, cy, ang, tol );

  return 1;
}

/* ======================  DlgCaptureFRotate  ===================== */

int DlgCaptureFRotate ( float *cx, float *cy )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitFRotate( &frotate );

  /* create dialog to capture point data. */
  d = dlgCreateFRotate();

  /* sets point data to dialog. */
  dlgValueToFRotate ( *cx, *cy );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateFRotate ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromFRotate ( cx, cy );

  return 1;
}

/* ======================  DlgCaptureSRotate  ===================== */

int DlgCaptureSRotate ( float *x1, float *y1 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitSRotate( &srotate );

  /* create dialog to capture point data. */
  d = dlgCreateSRotate();

  /* sets point data to dialog. */
  dlgValueToSRotate ( *x1, *y1 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateSRotate ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromSRotate ( x1, y1 );

  return 1;
}

/* ======================  DlgCaptureTRotate  ===================== */

int DlgCaptureTRotate ( float *cx, float *cy, float *x1, float *y1, 
                        float *x2, float *y2 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitTRotate( &trotate );

  /* create dialog to capture point data. */
  d = dlgCreateTRotate();

  /* sets point data to dialog. */
  dlgValueToTRotate ( *cx, *cy, *x1, *y1, *x2, *y2 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateTRotate ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromTRotate ( cx, cy, x1, y1, x2, y2 );

  return 1;
}

