
/*
** dlgscale.c - Functions that establish the pre-defined dialogs 
**              of the library to scale objects.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureScale( *cx, *cy, *fac, *toler )
**
** cx    - coordinate (X direction) of scaling center.     (IN/OUT)
** cy    - coordinate (Y direction) of scaling center.     (IN/OUT)
** fac   - scaling factor.                                 (IN/OUT)
** toler - tolerance used.                                 (IN/OUT)
**
** This function creates a dialog to capture the parameters to scale an image. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureFScale( float *cx, float *cx )
**
** cx  - coordinate (X direction) of scaling center.      (IN/OUT)
** cy  - coordinate (Y direction) of scaling center.      (IN/OUT)
**
** This function creates a dialog to capture the parameters of the 
** scaling center.  It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureSScale( float *x1, float *y1 )
**
** x1  - coordinate (X direction) of first scaling line point.      (IN/OUT)
** y1  - coordinate (Y direction) of first scaling line point.      (IN/OUT)
**
** This function creates a dialog to capture the parameters of the 
** first scaling line point.  It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureTScale( float *cx, float *cy, float *x1, float *y1, 
**                       float *x2, float *y2 )
**
** cx  - coordinate (X direction) of scaling center.              (IN/OUT)
** cy  - coordinate (Y direction) of scaling center.              (IN/OUT)
** x1  - coordinate (X direction) of first  scaling line point.   (IN/OUT)
** y1  - coordinate (Y direction) of first  scaling line point.   (IN/OUT)
** x2  - coordinate (X direction) of second scaling line point.   (IN/OUT)
** y2  - coordinate (Y direction) of second scaling line point.   (IN/OUT)
**
** This function creates a dialog to capture the parameters of the 
** second scaling line point.  It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    08-Aug-02    Marcos C. de Arruda.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgscale.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store point dialog */

typedef struct _scaledlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;
  char *pointlabel5;
  char *pointlabel6;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} ScaleDlg;


static void  dlgInitScale      ( ScaleDlg *dial );
static void* dlgCreateScale    ( void );
static void  dlgValueToScale   ( float cx,  float cy, 
                                 float fac, float tol );
static int   dlgValidateScale  ( void );
static void  dlgValueFromScale ( float *cx,  float *cy, 
                                 float *fac, float *tol );

static void  dlgInitFScale      ( ScaleDlg *dial );
static void* dlgCreateFScale    ( void );
static void  dlgValueToFScale   ( float cx, float cy );
static int   dlgValidateFScale  ( void );
static void  dlgValueFromFScale ( float *cx, float *cy );

static void  dlgInitSScale      ( ScaleDlg *dial );
static void* dlgCreateSScale    ( void );
static void  dlgValueToSScale   ( float x1, float y1 );
static int   dlgValidateSScale  ( void );
static void  dlgValueFromSScale ( float *x1, float *y1 );

static void  dlgInitTScale      ( ScaleDlg *dial );
static void* dlgCreateTScale    ( void );
static void  dlgValueToTScale   ( float cx, float cy,
                                  float x1, float y1,
                                  float x2, float y2 );
static int   dlgValidateTScale  ( void );
static void  dlgValueFromTScale ( float *cx, float *cy,	
                                  float *x1, float *y1,
                                  float *x2, float *y2 );

static int  callbackScaleDialogOk       ( Ihandle *bt );
static int  callbackScaleDialogCancel   ( Ihandle *bt );
static int  callbackScaleDialogCloseCB  ( Ihandle *bt );

static int  callbackFScaleDialogOk      ( Ihandle *bt );
static int  callbackFScaleDialogCancel  ( Ihandle *bt );
static int  callbackFScaleDialogCloseCB ( Ihandle *bt );

static int  callbackSScaleDialogOk      ( Ihandle *bt );
static int  callbackSScaleDialogCancel  ( Ihandle *bt );
static int  callbackSScaleDialogCloseCB ( Ihandle *bt );

static int  callbackTScaleDialogOk      ( Ihandle *bt );
static int  callbackTScaleDialogCancel  ( Ihandle *bt );
static int  callbackTScaleDialogCloseCB ( Ihandle *bt );

static ScaleDlg  scale;
static ScaleDlg fscale;
static ScaleDlg sscale;
static ScaleDlg tscale;

static char *size_butt = {"55x12"};
static char *size_text = {"65"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with scaling parameters: " };
static char *llabel1e = { "Center X:  " };
static char *llabel2e = { "Center Y:  " };
static char *llabel3e = { "Factor:  " };
static char *llabel4e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel5e  = { "Enter with scaling center coordinates:" };

static char *llabel6e  = { "Enter with first scaling line point coordinates:" };

static char *llabel11e = { "Enter with coordinates of scaling parameters:" };

static char *llabel0p = { "Entre com os parmetros de escala :" };
static char *llabel1p = { "Centro X:  " };
static char *llabel2p = { "Centro Y:  " };
static char *llabel3p = { "Fator:  " };
static char *llabel4p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

static char *llabel5p  = { "Entre com as coordenadas do centro de escala:" };

static char *llabel6p  = { "Entre com as coordenadas do ponto inicial do segmento de escala:" };

static char *llabel11p = { "Entre com as coordenadas dos parmetros de escala:" };

static char *llabel7  = { "X1:  " };
static char *llabel8  = { "Y1:  " };
static char *llabel9  = { "X2:  " };
static char *llabel10 = { "Y2:  " };


/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitScale  ======================= */

static void dlgInitScale ( ScaleDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointlabel4 = llabel4e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p;    
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateScale  ======================= */

static void* dlgCreateScale ( void )
{
  if ( scale.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( scale.pointbuttn1, "call_scale_ok" );
    Ihandle *butt2 = IupButton ( scale.pointbuttn2, "call_scale_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;	

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( scale.pointlabel1 );
    labl2 = IupLabel  ( scale.pointlabel2 );
    labl3 = IupLabel  ( scale.pointlabel3 );
    labl4 = IupLabel  ( scale.pointlabel4 );    
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    
    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );	
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );	
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );	
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  scale.pointlabel0 );

    /* sets handles */
    IupSetHandle("cx_scale1_handle",  text1);
    IupSetHandle("cy_scale1_handle",  text2);
    IupSetHandle("fac_scale1_handle", text3);    
    IupSetHandle("tl_scale1_handle",  text4);
    iupmaskSet(IupGetHandle("cx_scale1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("cy_scale1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("fac_scale1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_scale1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    scale.dialog = IupDialog (
                      IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_scale_ok", (Icallback)callbackScaleDialogOk );
    IupSetFunction ( "call_scale_cn", (Icallback)callbackScaleDialogCancel );
    IupSetFunction ( "call_scale_close",(Icallback)callbackScaleDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( scale.dialog, "GAP",    "3" );
    IupSetAttribute ( scale.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( scale.dialog, "MAXBOX", "NO");
    IupSetAttribute ( scale.dialog, "RESIZE", "NO");
    IupSetAttribute ( scale.dialog, "CLOSE_CB", "call_scale_close");
#ifdef _UNIX_
    IupSetAttribute ( scale.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( scale.dialog, "TITLE",  "Scale" );

    IupMap ( scale.dialog );
  }

  return scale.dialog;
}

/* ======================  dlgValueToScale  ======================= */

static void dlgValueToScale ( float cx, float cy, float fac, float tol )
{
 char cx1[50];
 char cy1[50];
 char cfac[50]; 
 char ctl[50];

 if ( scale.dialog == NULL ) return;

 sprintf(cx1, format, cx);
 sprintf(cy1, format, cy);
 sprintf(cfac, format, fac); 
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("cx_scale1_handle"),  "VALUE", cx1);
 IupSetAttribute (IupGetHandle("cy_scale1_handle"),  "VALUE", cy1);
 IupSetAttribute (IupGetHandle("fac_scale1_handle"), "VALUE", cfac); 
 IupSetAttribute (IupGetHandle("tl_scale1_handle"),  "VALUE", ctl);
}

/* =====================  dlgValueFromScale  ====================== */

static void dlgValueFromScale ( float *cx, float *cy, 
                                 float *fac, float *tol )
{
  if ( scale.dialog == NULL ) return;

  /* normal case */
  
  *cx  = IupGetFloat (IupGetHandle("cx_scale1_handle"),  "VALUE");
  *cy  = IupGetFloat (IupGetHandle("cy_scale1_handle"),  "VALUE");
  *fac  = IupGetFloat (IupGetHandle("fac_scale1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tl_scale1_handle"),  "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateScale  ======================= */

static int dlgValidateScale ( void )
{
 char *cx;
 char *cy;
 char *fac; 
 char *tl;

 if ( scale.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(scale.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 cx = IupGetAttribute (IupGetHandle("cx_scale1_handle"),  "VALUE");
 cy = IupGetAttribute (IupGetHandle("cy_scale1_handle"),  "VALUE");
 fac = IupGetAttribute (IupGetHandle("fac_scale1_handle"), "VALUE"); 
 tl = IupGetAttribute (IupGetHandle("tl_scale1_handle"),  "VALUE");

 return 1;
}

/* ====================  dlgInitFScale  ==================== */

static void dlgInitFScale ( ScaleDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel5e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel5p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateFScale  =================== */

static void* dlgCreateFScale ( void )
{
  if ( fscale.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( fscale.pointbuttn1, "call_fscale_ok" );
    Ihandle *butt2 = IupButton ( fscale.pointbuttn2, "call_fscale_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( fscale.pointlabel1 );
    labl2 = IupLabel  ( fscale.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  fscale.pointlabel0 );

    /* sets handles */
    IupSetHandle("cx_scale2_handle", text1);
    IupSetHandle("cy_scale2_handle", text2);
    iupmaskSet(IupGetHandle("cx_scale2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("cy_scale2_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    fscale.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_fscale_ok", (Icallback)callbackFScaleDialogOk );
    IupSetFunction ( "call_fscale_cn", (Icallback)callbackFScaleDialogCancel );
    IupSetFunction ( "call_fscale_close",(Icallback)callbackFScaleDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( fscale.dialog, "GAP",    "3" );
    IupSetAttribute ( fscale.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( fscale.dialog, "MAXBOX", "NO");
    IupSetAttribute ( fscale.dialog, "RESIZE", "NO");
    IupSetAttribute ( fscale.dialog, "CLOSE_CB", "call_fscale_close");
#ifdef _UNIX_
    IupSetAttribute ( fscale.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( fscale.dialog, "TITLE",  "Scale" );

    IupMap ( fscale.dialog );
  }

  return fscale.dialog;
}

/* ===================  dlgValueToFScale  =================== */

static void dlgValueToFScale ( float cx, float cy )
{
 char cx1[50];
 char cy1[50];

 if ( fscale.dialog == NULL ) return;

 sprintf(cx1, format, cx);
 sprintf(cy1, format, cy);

 IupSetAttribute (IupGetHandle("cx_scale2_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("cy_scale2_handle"), "VALUE", cy1);
}

/* =================  dlgValueFromFScale  =================== */

static void dlgValueFromFScale ( float *cx, float *cy )
{
  if ( fscale.dialog == NULL ) return;
  
  *cx  = IupGetFloat (IupGetHandle("cx_scale2_handle"), "VALUE");
  *cy  = IupGetFloat (IupGetHandle("cy_scale2_handle"), "VALUE");
}

/* ==================  dlgValidateFScale  =================== */

static int dlgValidateFScale ( void )
{
 char *cx;
 char *cy;

 if ( fscale.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(fscale.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 cx = IupGetAttribute (IupGetHandle("cx_scale2_handle"), "VALUE");
 cy = IupGetAttribute (IupGetHandle("cy_scale2_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitSScale  ==================== */

static void dlgInitSScale ( ScaleDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel6e;
   dial->pointlabel1 = llabel7;
   dial->pointlabel2 = llabel8;
   dial->pointbuttn1 = lbuttn1e;
   dial->pointbuttn2 = lbuttn2e;
 }
 else
 {
   dial->pointlabel0 = llabel6p;
   dial->pointlabel1 = llabel7;
   dial->pointlabel2 = llabel8;
   dial->pointbuttn1 = lbuttn1p;
   dial->pointbuttn2 = lbuttn2p;
 }
}

/* ====================  dlgCreateSScale  =================== */

static void* dlgCreateSScale ( void )
{
  if ( sscale.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( sscale.pointbuttn1, "call_sscale_ok" );
    Ihandle *butt2 = IupButton ( sscale.pointbuttn2, "call_sscale_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( sscale.pointlabel1 );
    labl2 = IupLabel  ( sscale.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl1,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text1,IupFill(),NULL),
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl2,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text2,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  sscale.pointlabel0 );

    /* sets handles */
    IupSetHandle("x1_scale3_handle", text1);
    IupSetHandle("y1_scale3_handle", text2);
    iupmaskSet(IupGetHandle("x1_scale3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_scale3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    sscale.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_sscale_ok", (Icallback)callbackSScaleDialogOk );
    IupSetFunction ( "call_sscale_cn", (Icallback)callbackSScaleDialogCancel );
    IupSetFunction ( "call_sscale_close",(Icallback)callbackSScaleDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( sscale.dialog, "GAP",    "3" );
    IupSetAttribute ( sscale.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( sscale.dialog, "MAXBOX", "NO");
    IupSetAttribute ( sscale.dialog, "RESIZE", "NO");
    IupSetAttribute ( sscale.dialog, "CLOSE_CB", "call_sscale_close");
#ifdef _UNIX_
    IupSetAttribute ( sscale.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( sscale.dialog, "TITLE",  "Scale" );

    IupMap ( sscale.dialog );
  }

  return sscale.dialog;
}

/* ===================  dlgValueToSScale  =================== */

static void dlgValueToSScale ( float x1, float y1 )
{
 char cx1[50];
 char cy1[50];

 if ( fscale.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);

 IupSetAttribute (IupGetHandle("x1_scale3_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_scale3_handle"), "VALUE", cy1);
}

/* =================  dlgValueFromSScale  =================== */

static void dlgValueFromSScale ( float *x1, float *y1 )
{
  if ( sscale.dialog == NULL ) return;

  *x1  = IupGetFloat (IupGetHandle("x1_scale3_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_scale3_handle"), "VALUE");
}

/* ==================  dlgValidateSScale  =================== */

static int dlgValidateSScale ( void )
{
 char *x1;
 char *y1;

 if ( sscale.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(sscale.dialog, "CONFIRM"), "1") != 0 )
    return 0;

 x1 = IupGetAttribute (IupGetHandle("x1_scale3_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_scale3_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitTScale  ==================== */

static void dlgInitTScale ( ScaleDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel11e;
   dial->pointlabel1 = llabel1e;
   dial->pointlabel2 = llabel2e;
   dial->pointlabel3 = llabel7;
   dial->pointlabel4 = llabel8;
   dial->pointlabel5 = llabel9;
   dial->pointlabel6 = llabel10;
   dial->pointbuttn1 = lbuttn1e;
   dial->pointbuttn2 = lbuttn2e;
 }
 else
 {
   dial->pointlabel0 = llabel11p;
   dial->pointlabel1 = llabel1p;
   dial->pointlabel2 = llabel2p;
   dial->pointlabel3 = llabel7;
   dial->pointlabel4 = llabel8;
   dial->pointlabel5 = llabel9;
   dial->pointlabel6 = llabel10;
   dial->pointbuttn1 = lbuttn1p;
   dial->pointbuttn2 = lbuttn2p;
 }
}

/* ====================  dlgCreateTScale  =================== */

static void* dlgCreateTScale ( void )
{
  if ( tscale.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( tscale.pointbuttn1, "call_tscale_ok" );
    Ihandle *butt2 = IupButton ( tscale.pointbuttn2, "call_tscale_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;
    Ihandle *text6 = NULL; 

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;
    Ihandle *h6 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( tscale.pointlabel1 );
    labl2 = IupLabel  ( tscale.pointlabel2 );
    labl3 = IupLabel  ( tscale.pointlabel3 );
    labl4 = IupLabel  ( tscale.pointlabel4 );
    labl5 = IupLabel  ( tscale.pointlabel5 );
    labl6 = IupLabel  ( tscale.pointlabel6 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );
    text6 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);

    h4 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl4,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text4,IupFill(),NULL),
		  IupFill(), NULL);

    h5 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl5,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text5,IupFill(),NULL),
		  IupFill(), NULL);

    h6 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl6,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text6,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), 
	               h1, 
		       IupFill(), 
		       h2, 
		       IupFill(), 
		       h3, 
		       IupFill(),
		       h4, 
		       IupFill(),
		       h5, 
		       IupFill(),
		       h6,
		       IupFill(), NULL )
		);

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text ); 

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  tscale.pointlabel0);

    /* sets handles */
    IupSetHandle("cx_scale3_handle", text1);
    IupSetHandle("cy_scale3_handle", text2);
    IupSetHandle("x1_scale3_handle", text3);
    IupSetHandle("y1_scale3_handle", text4);
    IupSetHandle("x2_scale3_handle", text5);
    IupSetHandle("y2_scale3_handle", text6);
    iupmaskSet(IupGetHandle("cx_scale3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("cy_scale3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x1_scale3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_scale3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x2_scale3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y2_scale3_handle"), IUPMASK_FLOAT, 0, 1);
 
    /* creates dialog */
    tscale.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_tscale_ok", (Icallback)callbackTScaleDialogOk );
    IupSetFunction ( "call_tscale_cn", (Icallback)callbackTScaleDialogCancel );
    IupSetFunction ( "call_tscale_close",(Icallback)callbackTScaleDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( tscale.dialog, "GAP",    "3" );
    IupSetAttribute ( tscale.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( tscale.dialog, "MAXBOX", "NO");
    IupSetAttribute ( tscale.dialog, "RESIZE", "NO");
    IupSetAttribute ( tscale.dialog, "CLOSE_CB", "call_tscale_close");
#ifdef _UNIX_
    IupSetAttribute ( tscale.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( tscale.dialog, "TITLE",  "Scale" );

    IupMap ( tscale.dialog );
  }

  return tscale.dialog;
}

/* ===================  dlgValueToTScale  =================== */

static void dlgValueToTScale ( float cx, float cy, float x1, float y1, 
                               float x2, float y2 )
{
 char ccx[50];
 char ccy[50];
 char cx1[50];
 char cy1[50];
 char cx2[50];
 char cy2[50];

 if ( tscale.dialog == NULL ) return;

 sprintf(ccx, format, cx);
 sprintf(ccy, format, cy);
 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(cx2, format, x2);
 sprintf(cy2, format, y2);

 IupSetAttribute (IupGetHandle("cx_scale3_handle"), "VALUE", ccx);
 IupSetAttribute (IupGetHandle("cy_scale3_handle"), "VALUE", ccy);
 IupSetAttribute (IupGetHandle("x1_scale3_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_scale3_handle"), "VALUE", cy1);
 IupSetAttribute (IupGetHandle("x2_scale3_handle"), "VALUE", cx2);
 IupSetAttribute (IupGetHandle("y2_scale3_handle"), "VALUE", cy2);
}

/* =================  dlgValueFromTScale  =================== */

static void dlgValueFromTScale ( float *cx, float *cy, float *x1, float *y1, 
                                 float *x2, float *y2 )
{
  if ( tscale.dialog == NULL ) return;
  
  *cx  = IupGetFloat (IupGetHandle("cx_scale3_handle"), "VALUE");
  *cy  = IupGetFloat (IupGetHandle("cy_scale3_handle"), "VALUE");
  *x1  = IupGetFloat (IupGetHandle("x1_scale3_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_scale3_handle"), "VALUE");
  *x2  = IupGetFloat (IupGetHandle("x2_scale3_handle"), "VALUE");
  *y2  = IupGetFloat (IupGetHandle("y2_scale3_handle"), "VALUE");
}

/* ==================  dlgValidateTScale  =================== */

static int dlgValidateTScale ( void )
{
 char *cx;
 char *cy;
 char *x1;
 char *y1;
 char *x2;
 char *y2;

 if ( tscale.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(tscale.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 cx = IupGetAttribute (IupGetHandle("cx_scale3_handle"), "VALUE");
 cy = IupGetAttribute (IupGetHandle("cy_scale3_handle"), "VALUE");
 x1 = IupGetAttribute (IupGetHandle("x1_scale3_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_scale3_handle"), "VALUE");
 x2 = IupGetAttribute (IupGetHandle("x2_scale3_handle"), "VALUE");
 y2 = IupGetAttribute (IupGetHandle("y2_scale3_handle"), "VALUE");

 return 1;
}

/* =========================  callbackScaleDialogOk  ======================== */

static int callbackScaleDialogOk ( Ihandle *bt )
{
 IupSetAttribute(scale.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackScaleDialogCancel  ====================== */

static int callbackScaleDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(scale.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackScaleDialogCloseCB  ====================== */

static int callbackScaleDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(scale.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackFScaleDialogOk  ====================== */

static int callbackFScaleDialogOk ( Ihandle *bt )
{
 IupSetAttribute(fscale.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackFScaleDialogCancel  ==================== */

static int callbackFScaleDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(fscale.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackFScaleDialogCloseCB  ==================== */

static int callbackFScaleDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(fscale.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackSScaleDialogOk  ====================== */

static int callbackSScaleDialogOk ( Ihandle *bt )
{
 IupSetAttribute(sscale.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackSScaleDialogCancel  ==================== */

static int callbackSScaleDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(sscale.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackSScaleDialogCloseCB  ==================== */

static int callbackSScaleDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(sscale.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackTScaleDialogOk  ====================== */

static int callbackTScaleDialogOk ( Ihandle *bt )
{
 IupSetAttribute(tscale.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackTScaleDialogCancel  ==================== */

static int callbackTScaleDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(tscale.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callbackTScaleDialogCloseCB  ==================== */

static int callbackTScaleDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(tscale.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureScale  ======================== */

int DlgCaptureScale ( float *cx, float *cy, float *fac, float *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitScale( &scale );

  /* create dialog to capture point data. */
  d = dlgCreateScale();

  /* sets point data to dialog. */
  dlgValueToScale ( *cx, *cy, *fac, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateScale ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromScale ( cx, cy, fac, tol );

  return 1;
}


/* ======================  DlgCaptureFScale  ===================== */

int DlgCaptureFScale ( float *cx, float *cy )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitFScale( &fscale );

  /* create dialog to capture point data. */
  d = dlgCreateFScale();

  /* sets point data to dialog. */
  dlgValueToFScale ( *cx, *cy );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateFScale ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromFScale ( cx, cy );

  return 1;
}

/* ======================  DlgCaptureSScale  ===================== */

int DlgCaptureSScale ( float *x1, float *y1 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitSScale( &sscale );

  /* create dialog to capture point data. */
  d = dlgCreateSScale();

  /* sets point data to dialog. */
  dlgValueToSScale ( *x1, *y1 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateSScale ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromSScale ( x1, y1 );

  return 1;
}

/* ======================  DlgCaptureTScale  ===================== */

int DlgCaptureTScale ( float *cx, float *cy, float *x1, float *y1, 
                       float *x2, float *y2 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitTScale( &tscale );

  /* create dialog to capture point data. */
  d = dlgCreateTScale();

  /* sets point data to dialog. */
  dlgValueToTScale ( *cx, *cy, *x1, *y1, *x2, *y2 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateTScale ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromTScale ( cx, cy, x1, y1, x2, y2 );

  return 1;
}

