/*
** dlgspline.c - Functions that establish the pre-defined dialogs 
**               of the library to capture splines.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureNSpline( int *np, float *tol )
**
** np  - number of points of spline.              (IN/OUT)
** tol - tolerance used.                          (IN/OUT)
**
** This function creates a dialog to capture the number of points
** of spline. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureXSpline( float *x0, float *y0 )
**
** x0 - coordinate (X direction) for each point.    (IN/OUT)
** y0 - coordinate (Y direction) for each point.    (IN/OUT)
**
** This function creates a dialog to capture the points of spline. 
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    30-Dec-02    Marcos C. de Arruda.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgspline.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure */

typedef struct _pointdlg
{

  char *splinelabel0;
  char *splinelabel1;
  char *splinelabel2;
  char *splinelabel3;
  char *splinelabel4;
  char *splinelabel5;
  char *splinelabel6;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} SplineDlg;


static void  dlgInitNSpline      ( SplineDlg *dial );
static void* dlgCreateNSpline    ( void );
static void  dlgValueToNSpline   ( int np, float tol );
static int   dlgValidateNSpline  ( void );
static void  dlgValueFromNSpline ( int *np, float *tol );

static void  dlgInitXSpline      ( SplineDlg *dial );
static void* dlgCreateXSpline    ( void );
static void  dlgValueToXSpline   ( float x0, float y0 );
static int   dlgValidateXSpline  ( void );
static void  dlgValueFromXSpline ( float *x0, float *y0 );

static void  dlgInitDerSpline      ( SplineDlg *dial );
static void* dlgCreateDerSpline    ( void );
static void  dlgValueToDerSpline   ( float fderx, float fdery, float lderx, float ldery );
static int   dlgValidateDerSpline  ( void );
static void  dlgValueFromDerSpline ( float *fderx, float *fdery, float *lderx, float *ldery );

static int  callbackNSplineDialogOk     ( Ihandle *bt );
static int  callbackNSplineDialogCancel ( Ihandle *bt );
static int  callbackNSplineDialogCloseCB( Ihandle *bt );

static int  callbackXSplineDialogOk     ( Ihandle *bt );
static int  callbackXSplineDialogCancel ( Ihandle *bt );
static int  callbackXSplineDialogCloseCB( Ihandle *bt );

static int  callbackDerSplineDialogOk     ( Ihandle *bt );
static int  callbackDerSplineDialogCancel ( Ihandle *bt );
static int  callbackDerSplineDialogCloseCB( Ihandle *bt );

static SplineDlg nspline;
static SplineDlg xspline;
static SplineDlg dspline;

static char *size_butt = {"55x12"};
static char *size_text = {"55"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with number of points:" };
static char *llabel1e = { "N. Points: " };
static char *llabel2e = { "Point X: " };
static char *llabel3e = { "Point Y: " };
static char *llabel4e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel5e = { "Enter with point coordinates:" };

static char *llabel6e = { "Enter with tangent vectors:" };
static char *llabel7e = { "First point:" };
static char *llabel8e = { "Last point:" };

static char *llabel0p = { "Entre com o numbero de pontos:" };
static char *llabel1p = { "N. Pontos: " };
static char *llabel2p = { "Ponto X: " };
static char *llabel3p = { "Ponto Y: " };
static char *llabel4p = { "Toleranc.: " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

static char *llabel5p = { "Entre com as coordenadas do ponto:" };

static char *llabel6p = { "Enter com os vetores tangentes:" };
static char *llabel7p = { "Primeiro ponto:" };
static char *llabel8p = { "ltimo ponto:" };

static char *llabel9  = { "Dx: " };
static char *llabel10 = { "Dy: " };
static char *llabel11 = { "Dx: " };
static char *llabel12 = { "Dy: " };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitNSpline  ======================= */

static void dlgInitNSpline ( SplineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->splinelabel0 = llabel0e; 
   dial->splinelabel1 = llabel1e; 
   dial->splinelabel4 = llabel4e; 
   dial->pointbuttn1  = lbuttn1e; 
   dial->pointbuttn2  = lbuttn2e; 
 }
 else
 {
   dial->splinelabel0 = llabel0p; 
   dial->splinelabel1 = llabel1p; 
   dial->splinelabel4 = llabel4p; 
   dial->pointbuttn1  = lbuttn1p; 
   dial->pointbuttn2  = lbuttn2p; 
 }
}

/* =======================  dlgCreateNSpline  ======================= */

static void* dlgCreateNSpline ( void )
{
  if ( nspline.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( nspline.pointbuttn1, "call_spline_ok" );
    Ihandle *butt2 = IupButton ( nspline.pointbuttn2, "call_spline_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( nspline.splinelabel1 );
    labl2 = IupLabel  ( nspline.splinelabel4 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP",    "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  nspline.splinelabel0 );

    /* sets handles */
    IupSetHandle("np_spline1_handle", text1);
    IupSetHandle("tl_spline1_handle", text2);
    iupmaskSet(IupGetHandle("np_spline1_handle"), IUPMASK_UINT , 0, 1);
    iupmaskSet(IupGetHandle("tl_spline1_handle"), IUPMASK_UFLOAT , 0, 1); 

    /* creates dialog */
    nspline.dialog = IupDialog (
                      IupFrame (
		       IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_spline_ok", (Icallback)callbackNSplineDialogOk );
    IupSetFunction ( "call_spline_cn", (Icallback)callbackNSplineDialogCancel );
    IupSetFunction ( "call_spline_close",(Icallback)callbackNSplineDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( nspline.dialog, "GAP",    "3" );
    IupSetAttribute ( nspline.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( nspline.dialog, "MAXBOX", "NO");
    IupSetAttribute ( nspline.dialog, "RESIZE", "NO");
    IupSetAttribute ( nspline.dialog, "CONFIRM", "0");
    IupSetAttribute ( nspline.dialog, "CLOSE_CB", "call_spline_close");
#ifdef _UNIX_
    IupSetAttribute ( nspline.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( nspline.dialog, "TITLE",  "Spline" );

    IupMap ( nspline.dialog );
  }

  return nspline.dialog;
}

/* ======================  dlgValueToNSpline  ======================= */

static void dlgValueToNSpline ( int np, float tol )
{
 char cnp[50];
 char ctl[50];

 if ( nspline.dialog == NULL ) return;

 sprintf(cnp, "%d",    np);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("np_spline1_handle"), "VALUE", cnp);
 IupSetAttribute (IupGetHandle("tl_spline1_handle"), "VALUE", ctl);
}

/* =====================  dlgValueFromNSpline  ====================== */

static void dlgValueFromNSpline ( int *np, float *tol )
{
  if ( nspline.dialog == NULL ) return;

  /* normal case */
  *np  = IupGetInt (IupGetHandle("np_spline1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tl_spline1_handle"), "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateNSpline  ======================= */

static int dlgValidateNSpline ( void )
{
 char *np;
 char *tl;

 if ( nspline.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(nspline.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 np = IupGetAttribute (IupGetHandle("np_spline1_handle"), "VALUE");
 tl = IupGetAttribute (IupGetHandle("tl_spline1_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitXSpline  ==================== */

static void dlgInitXSpline ( SplineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->splinelabel0 = llabel5e; 
   dial->splinelabel2 = llabel2e; 
   dial->splinelabel3 = llabel3e; 
   dial->pointbuttn1  = lbuttn1e; 
   dial->pointbuttn2  = lbuttn2e; 
 }
 else
 {
   dial->splinelabel0 = llabel5p; 
   dial->splinelabel2 = llabel2p; 
   dial->splinelabel3 = llabel3p; 
   dial->pointbuttn1  = lbuttn1p; 
   dial->pointbuttn2  = lbuttn2p; 
 }
}

/* ====================  dlgCreateXSpline  =================== */

static void* dlgCreateXSpline ( void )
{
  if ( xspline.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( xspline.pointbuttn1, "call_xspline_ok" );
    Ihandle *butt2 = IupButton ( xspline.pointbuttn2, "call_xspline_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( xspline.splinelabel2 );
    labl2 = IupLabel  ( xspline.splinelabel3 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  xspline.splinelabel0 );

    /* sets handles */
    IupSetHandle("x0_spline2_handle", text1);
    IupSetHandle("y0_spline2_handle", text2);
    iupmaskSet(IupGetHandle("x0_spline2_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("y0_spline2_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    xspline.dialog = IupDialog (
                      IupFrame (
		       IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
		        IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
           	        IupFill(),
		        NULL
		       )
		      )
                     );

    /* sets callbacks */
    IupSetFunction ( "call_xspline_ok", (Icallback)callbackXSplineDialogOk );
    IupSetFunction ( "call_xspline_cn", (Icallback)callbackXSplineDialogCancel);
    IupSetFunction ( "call_xspline_close",(Icallback)callbackXSplineDialogCloseCB);

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( xspline.dialog, "GAP",    "3" );
    IupSetAttribute ( xspline.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( xspline.dialog, "MAXBOX", "NO");
    IupSetAttribute ( xspline.dialog, "RESIZE", "NO");
    IupSetAttribute ( xspline.dialog, "CLOSE_CB", "call_xspline_close");
#ifdef _UNIX_
    IupSetAttribute ( xspline.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( xspline.dialog, "TITLE",  "Spline" );

    IupMap ( xspline.dialog );
  }

  return xspline.dialog;
}

/* ===================  dlgValueToXSpline  =================== */

static void dlgValueToXSpline ( float x0, float y0 )
{
 char cx0[50];
 char cy0[50];

 if ( xspline.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);

 IupSetAttribute (IupGetHandle("x0_spline2_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_spline2_handle"), "VALUE", cy0);
}

/* =================  dlgValueFromXSpline  =================== */

static void dlgValueFromXSpline ( float *x0, float *y0 )
{
  if ( xspline.dialog == NULL ) return;

  *x0  = IupGetFloat (IupGetHandle("x0_spline2_handle"), "VALUE");
  *y0  = IupGetFloat (IupGetHandle("y0_spline2_handle"), "VALUE");
}

/* ==================  dlgValidateXSpline  =================== */

static int dlgValidateXSpline ( void )
{
 char *x0;
 char *y0;

 if ( xspline.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(xspline.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x0 = IupGetAttribute (IupGetHandle("x0_spline2_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_spline2_handle"), "VALUE");

 return 1;
}

/* =======================  dlgInitDerSpline  ======================= */

static void dlgInitDerSpline ( SplineDlg *dial )
{
  if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->splinelabel0 = llabel6e; 
   dial->splinelabel1 = llabel7e; 
   dial->splinelabel2 = llabel8e;
   dial->splinelabel3 = llabel9; 
   dial->splinelabel4 = llabel10;
   dial->splinelabel5 = llabel11; 
   dial->splinelabel6 = llabel12;
   dial->pointbuttn1  = lbuttn1e; 
   dial->pointbuttn2  = lbuttn2e; 
 }
 else
 {
   dial->splinelabel0 = llabel6p; 	
   dial->splinelabel1 = llabel7p; 
   dial->splinelabel2 = llabel8p; 
   dial->splinelabel3 = llabel9; 
   dial->splinelabel4 = llabel10;
   dial->splinelabel5 = llabel11; 
   dial->splinelabel6 = llabel12;
   dial->pointbuttn1  = lbuttn1p; 
   dial->pointbuttn2  = lbuttn2p; 
 } 
}

/* ====================  dlgCreateDerSpline  =================== */

static void* dlgCreateDerSpline ( void )
{
  if ( dspline.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( dspline.pointbuttn1, "call_dspline_ok" );
    Ihandle *butt2 = IupButton ( dspline.pointbuttn2, "call_dspline_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;

    Ihandle *t1 = NULL;
    Ihandle *t2 = NULL;
    Ihandle *t3 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( dspline.splinelabel1 );
    labl2 = IupLabel  ( dspline.splinelabel2 );
    labl3 = IupLabel  ( dspline.splinelabel3 );
    labl4 = IupLabel  ( dspline.splinelabel4 );
    labl5 = IupLabel  ( dspline.splinelabel5 );
    labl6 = IupLabel  ( dspline.splinelabel6 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);		  
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl6,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    
    t2 = IupFrame (
           IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )        
         );
    
    t3 = IupFrame (
           IupVbox ( IupFill(), h3, IupFill(), h4, IupFill(), NULL )        
         );
    
    t1 = IupFrame (
	   IupVbox ( IupFill(), t2, IupFill(), t3, IupFill(), NULL )
	 );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP",    "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  dspline.splinelabel0 );

    IupSetAttribute ( t2, "GAP",    "1" );
    IupSetAttribute ( t2, "MARGIN", "1x1" );
    IupSetAttribute ( t2, "SIZE",   "150" );
    IupSetAttribute ( t2, "TITLE", "First Point"  );
    
    IupSetAttribute ( t3, "GAP",    "1" );
    IupSetAttribute ( t3, "MARGIN", "1x1" );
    IupSetAttribute ( t3, "SIZE",   "150" );
    IupSetAttribute ( t3, "TITLE", "Last Point"  );

    /* sets handles */
    IupSetHandle("fderx_spline1_handle", text1);
    IupSetHandle("fdery_spline1_handle", text2);
    IupSetHandle("lderx_spline1_handle", text3);
    IupSetHandle("ldery_spline1_handle", text4);
    iupmaskSet(IupGetHandle("fderx_spline1_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("fdery_spline1_handle"), IUPMASK_FLOAT , 0, 1); 
    iupmaskSet(IupGetHandle("lderx_spline1_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("ldery_spline1_handle"), IUPMASK_FLOAT , 0, 1); 

    /* creates dialog */
    dspline.dialog = IupDialog (
                      IupFrame (
		       IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_dspline_ok", (Icallback)callbackDerSplineDialogOk );
    IupSetFunction ( "call_dspline_cn", (Icallback)callbackDerSplineDialogCancel );
    IupSetFunction ( "call_dspline_close",(Icallback)callbackDerSplineDialogCloseCB);

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( dspline.dialog, "GAP",    "3" );
    IupSetAttribute ( dspline.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( dspline.dialog, "MAXBOX", "NO");
    IupSetAttribute ( dspline.dialog, "RESIZE", "NO");
    IupSetAttribute ( dspline.dialog, "RESIZE", "NO");
    IupSetAttribute ( dspline.dialog, "CLOSE_CB", "call_dspline_close");
#ifdef _UNIX_
    IupSetAttribute ( dspline.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( dspline.dialog, "TITLE",  "Spline Tangents" );

    IupMap ( dspline.dialog );
  }

  return dspline.dialog;
}

/* ===================  dlgValueToDerSpline  =================== */

static void dlgValueToDerSpline ( float fderx, float fdery, float lderx, float ldery )
{
  char cfderx[50];
  char cfdery[50];
  char clderx[50];
  char cldery[50];

  if ( dspline.dialog == NULL ) return;

 sprintf(cfderx, format, fderx);
 sprintf(cfdery, format, fdery);
 sprintf(clderx, format, lderx);
 sprintf(cldery, format, ldery); 

 IupSetAttribute (IupGetHandle("fderx_spline1_handle"), "VALUE", cfderx);
 IupSetAttribute (IupGetHandle("fdery_spline1_handle"), "VALUE", cfdery);
 IupSetAttribute (IupGetHandle("lderx_spline1_handle"), "VALUE", clderx);
 IupSetAttribute (IupGetHandle("ldery_spline1_handle"), "VALUE", cldery);
}

/* =================  dlgValueFromDerSpline  =================== */

static void dlgValueFromDerSpline ( float *fderx, float *fdery, float *lderx, float *ldery )
{
  if ( dspline.dialog == NULL ) return;

  /* normal case */
  *fderx  = IupGetFloat (IupGetHandle("fderx_spline1_handle"), "VALUE");
  *fdery  = IupGetFloat (IupGetHandle("fdery_spline1_handle"), "VALUE"); 
  *lderx  = IupGetFloat (IupGetHandle("lderx_spline1_handle"), "VALUE");
  *ldery  = IupGetFloat (IupGetHandle("ldery_spline1_handle"), "VALUE"); 
}

/* ==================  dlgValidateDerSpline  =================== */

static int dlgValidateDerSpline ( void )
{
  char *fderx;
  char *fdery;
  char *lderx;
  char *ldery;

  if ( dspline.dialog == NULL ) return 0;

  if ( strcmp(IupGetAttribute(dspline.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

  /* normal case */
  fderx = IupGetAttribute (IupGetHandle("fderx_spline1_handle"), "VALUE");
  fdery = IupGetAttribute (IupGetHandle("fdery_spline1_handle"), "VALUE");
  lderx = IupGetAttribute (IupGetHandle("lderx_spline1_handle"), "VALUE");
  ldery = IupGetAttribute (IupGetHandle("ldery_spline1_handle"), "VALUE");

  return 1;	
}

/* =======================  callbackNSplineDialogOk  ====================== */

static int callbackNSplineDialogOk ( Ihandle *bt )
{
 IupSetAttribute(nspline.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =====================  callbackNSplineDialogCancel  ==================== */

static int callbackNSplineDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(nspline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =====================  callbackNSplineDialogCloseCB  ==================== */

static int callbackNSplineDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(nspline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackXSplineDialogOk  ====================== */

static int callbackXSplineDialogOk ( Ihandle *bt )
{
 IupSetAttribute(xspline.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackXSplineDialogCancel  ==================== */

static int callbackXSplineDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(xspline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =====================  callbackXSplineDialogCloseCB  ==================== */

static int callbackXSplineDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(xspline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackDerSplineDialogOk  ====================== */

static int callbackDerSplineDialogOk ( Ihandle *bt )
{
 IupSetAttribute(dspline.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackDerSplineDialogCancel  ==================== */

static int callbackDerSplineDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(dspline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =====================  callbackDerSplineDialogCloseCB  ==================== */

static int callbackDerSplineDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(dspline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureNSpline  ======================== */

int DlgCaptureNSpline ( int *np, float *tol )
{
  Ihandle *d;

  /* initializes dialog. */
  dlgInitNSpline( &nspline );

  /* create dialog to capture the number of points of spline. */
  d = dlgCreateNSpline();

  /* sets spline data to dialog. */
  dlgValueToNSpline ( *np, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks spline data. */
  if ( dlgValidateNSpline ( ) == 0 ) return 0;

  /* get spline data from dialog. */
  dlgValueFromNSpline ( np, tol );

  return 1;
}

/* ======================  DlgCaptureXSpline  ===================== */

int DlgCaptureXSpline ( float *x0, float *y0 )
{
  Ihandle *d;

  /* initializes dialog. */
  dlgInitXSpline( &xspline );

  /* create dialog to capture points of spline. */
  d = dlgCreateXSpline();

  /* sets spline data to dialog. */
  dlgValueToXSpline ( *x0, *y0 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks spline data. */
  if ( dlgValidateXSpline ( ) == 0 ) return 0;

  /* gets spline data from dialog. */
  dlgValueFromXSpline ( x0, y0 );

  return 1;
}

/* =========================  DlgCaptureDerSpline  ======================== */

int DlgCaptureDerSpline ( float *fderx, float *fdery, float *lderx, float *ldery )
{
  Ihandle *d;

  /* initializes dialog. */
  dlgInitDerSpline( &dspline );

  /* create dialog to capture the initial and final tangent vectors. */
  d = dlgCreateDerSpline();

  /* sets spline data to dialog. */
  dlgValueToDerSpline ( *fderx, *fdery, *lderx, *ldery );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks spline data. */
  if ( dlgValidateDerSpline ( ) == 0 ) return 0;

  /* get spline data from dialog. */
  dlgValueFromDerSpline ( fderx, fdery, lderx, ldery );

  return 1;
}

