
/*
** dlgwimfence.c - Functions that establish the pre-defined dialogs 
**                 of the library to create wimfence for images.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureWimFence( *x1, *y1, *x2 )
**
** x1  - coordinate (X direction) of first point of wimfence. (IN/OUT)
** y1  - coordinate (Y direction) of first point of wimfence. (IN/OUT)
** x2  - coordinate (X direction) of end point of wimfence.   (IN/OUT)
**
** This function creates a dialog to capture the parameters of a wimfence.
**
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    08-Aug-02    Marcos C. de Arruda.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgwimfence.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store point dialog */

typedef struct _wimfencedlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} WimFenceDlg;


static void  dlgInitWimFence      ( WimFenceDlg *dial );
static void* dlgCreateWimFence    ( void );
static void  dlgValueToWimFence   ( float x1, float y1, float x2 );                                    
static int   dlgValidateWimFence  ( void );
static void  dlgValueFromWimFence ( float *x1, float *y1, float *x2 );

static int  callbackWimFenceDialogOk     ( Ihandle *bt );
static int  callbackWimFenceDialogCancel ( Ihandle *bt );
static int  callbackWimFenceDialogCloseCB( Ihandle *bt );

static WimFenceDlg  wimfence;

static char *size_butt = {"55x12"};
static char *size_text = {"65"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with wimfence coordinates: " };
static char *llabel1e = { "Point 1 X:  " };
static char *llabel2e = { "Point 1 Y:  " };
static char *llabel3e = { "Point 2 X:  " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel0p = { "Enter with wimfence coordinates: " };
static char *llabel1p = { "Ponto 1 X:  " };
static char *llabel2p = { "Ponto 1 Y:  " };
static char *llabel3p = { "Ponto 2 X:  " };
static char *lbuttn1p = { " Confirma " };
static char *lbuttn2p = { " Cancela " };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitWimFence  ======================= */

static void dlgInitWimFence ( WimFenceDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e;    
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p;    
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateWimFence  ======================= */

static void* dlgCreateWimFence ( void )
{
  if ( wimfence.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( wimfence.pointbuttn1, "call_wimfence_ok" );
    Ihandle *butt2 = IupButton ( wimfence.pointbuttn2, "call_wimfence_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;    

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( wimfence.pointlabel1 );
    labl2 = IupLabel  ( wimfence.pointlabel2 );
    labl3 = IupLabel  ( wimfence.pointlabel3 );    
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);    

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
	    NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );    
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );    
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );    
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  wimfence.pointlabel0 );

    /* sets handles */
    IupSetHandle("x1_wimfence1_handle", text1);
    IupSetHandle("y1_wimfence1_handle", text2);
    IupSetHandle("x2_wimfence1_handle", text3);
    iupmaskSet(IupGetHandle("x1_wimfence1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_wimfence1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x2_wimfence1_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    wimfence.dialog = IupDialog (
                      IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_wimfence_ok", (Icallback)callbackWimFenceDialogOk );
    IupSetFunction ("call_wimfence_cn",(Icallback)callbackWimFenceDialogCancel);
    IupSetFunction ("call_wimfence_cl",(Icallback)callbackWimFenceDialogCloseCB);

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( wimfence.dialog, "GAP",    "3" );
    IupSetAttribute ( wimfence.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( wimfence.dialog, "MAXBOX", "NO");
    IupSetAttribute ( wimfence.dialog, "RESIZE", "NO");
    IupSetAttribute ( wimfence.dialog, "CLOSE_CB", "call_wimfence_cl");
#ifdef _UNIX_
    IupSetAttribute ( wimfence.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( wimfence.dialog, "TITLE",  "WimFence" );

    IupMap ( wimfence.dialog );
  }

  return wimfence.dialog;
}

/* ======================  dlgValueToWimFence  ======================= */

static void dlgValueToWimFence ( float x1, float y1, float x2 )
{
 char cx1[50];
 char cy1[50];
 char cx2[50]; 

 if ( wimfence.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(cx2, format, x2); 

 IupSetAttribute (IupGetHandle("x1_wimfence1_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_wimfence1_handle"), "VALUE", cy1);
 IupSetAttribute (IupGetHandle("x2_wimfence1_handle"), "VALUE", cx2); 
}

/* =====================  dlgValueFromWimFence  ====================== */

static void dlgValueFromWimFence ( float *x1, float *y1, float *x2 )                                 
{
  if ( wimfence.dialog == NULL ) return;

  /* normal case */
  *x1  = IupGetFloat (IupGetHandle("x1_wimfence1_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_wimfence1_handle"), "VALUE");
  *x2  = IupGetFloat (IupGetHandle("x2_wimfence1_handle"), "VALUE");  
}

/* =====================  dlgValidateWimFence  ======================= */

static int dlgValidateWimFence ( void )
{
 char *x1;
 char *y1;
 char *x2; 

 if ( wimfence.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(wimfence.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 x1 = IupGetAttribute (IupGetHandle("x1_wimfence1_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_wimfence1_handle"), "VALUE");
 x2 = IupGetAttribute (IupGetHandle("x2_wimfence1_handle"), "VALUE"); 

 return 1;
}


/* =======================  callbackWimFenceDialogOk  ====================== */

static int callbackWimFenceDialogOk ( Ihandle *bt )
{
 IupSetAttribute(wimfence.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =====================  callbackWimFenceDialogCancel  ==================== */

static int callbackWimFenceDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(wimfence.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =====================  callbackWimFenceDialogCloseCB  ==================== */

static int callbackWimFenceDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(wimfence.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureWimFence  ======================== */

int DlgCaptureWimFence ( float *x1, float *y1, float *x2 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitWimFence( &wimfence );

  /* create dialog to capture point data. */
  d = dlgCreateWimFence();

  /* sets point data to dialog. */
  dlgValueToWimFence ( *x1, *y1, *x2 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateWimFence ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromWimFence ( x1, y1, x2 );

  return 1;
}

