char *RCSnd="$Id: nd.c,v 1.1 1997/05/19 11:26:46 lfm Exp $";
/*
%M ND para viewport em coordenadas normalizadas (NDC) no Canvas Draw
%a Luiz Fernando Martha
%d 14 May 97
%p Sistemas Graficos
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols:
*/
#include <stdlib.h>

#include "cd.h"
#include "wd.h"
#include "nd.h"

/*
** ---------------------------------------------------------------
** Private variables and definitions:
*/
typedef struct _ndcwin {     /* NDC window descriptor: */
  struct _ndcwin  *prev;           /* pointer to previous NDC window          */
  double          ox, oy;          /* origin in NDC coords. of previous level */
  double          w, h;            /* sizes in NDC coords. of previous level  */
 } NdcWin;

typedef struct _cnvdsc {     /* Canvas descriptor in local data structure: */
  struct _cnvdsc  *next;           /* pointer to next canvas descriptor  */
  void            *cd_cv;          /* CD handle of canvas                */
  NdcWin          *win_stack;      /* pointer to NDC window stack        */
  double          s, tx, ty;       /* current NDC window transformation  */
 } CnvDsc;

typedef struct _nd {      /* Local data structure descriptor: */
  CnvDsc          *cnv_head;       /* pointer to linked list of canvas desc. */
  CnvDsc          *cnv_cur;        /* pointer to active canvas descriptor    */
 } Nd;

static Nd      nd;     /* Local data structure variable */

/*
** ---------------------------------------------------------------
** Private module functions:
*/
static CnvDsc *_ndGetDescr( void *cd_cv );
static void _ndNdcB2Pixel( double ndc_dx, double ndc_dy, int *dx, int *dy );
static void _ndPixel2NdcB( int dx, int dy, double *ndc_dx, double *ndc_dy );
static void _ndNdc2NdcB( double xndc, double yndc, double *xb, double *yb );
static void _ndNdcB2Ndc( double xb, double yb, double *xndc, double *yndc );

/* =========================  ndGetDescr  ======================= */
/*
%F Retorna um ponteiro para a descricao do canvas na estrutura
   de dados local.  Se nao existir um descritor associado ao
   canvas fornecido, retorna nulo.
*/
static CnvDsc *_ndGetDescr( void *cd_cv )
{
 CnvDsc   *cnv;

 for( cnv = nd.cnv_head; cnv != NULL; cnv = cnv->next )
 {
  if( cnv->cd_cv == cd_cv ) break;
 }
 return( cnv );
}

/* =========================  ndNdcB2Pixel  ======================= */
/*
%F Converte de coordenadas NDC no n'ivel b'asico no canvas para pixel.
*/
static void _ndNdcB2Pixel( double ndc_dx, double ndc_dy, int *dx, int *dy )
{
 int      wr, hr;

 cdGetCanvasSize( &wr, &hr, NULL, NULL );

 if( wr > hr )
 {
  *dx = (int)(ndc_dx * (double)wr);
  *dy = (int)(ndc_dy * (double)wr);
 }
 else
 {
  *dx = (int)(ndc_dx * (double)hr);
  *dy = (int)(ndc_dy * (double)hr);
 }
}

/* =========================  ndPixel2NdcB  ======================= */
/*
%F Converte de pixel para coordenadas NDC no n'ivel b'asico no canvas.
*/
static void _ndPixel2NdcB( int dx, int dy, double *ndc_dx, double *ndc_dy )
{
 int      wr, hr;

 cdGetCanvasSize( &wr, &hr, NULL, NULL );

 if( wr > hr )
 {
  *ndc_dx = (double)dx / (double)wr;
  *ndc_dy = (double)dy / (double)wr;
 }
 else
 {
  *ndc_dx = (double)dx / (double)hr;
  *ndc_dy = (double)dy / (double)hr;
 }
}

/* ==========================  ndNdc2NdcB  ========================= */
/*
% F Converte de coordenadas NDC corrente para coordenadas NDC
    do n'ivel b'asico no canvas.
*/
static void _ndNdc2NdcB( double xndc, double yndc, double *xb, double *yb )
{
 *xb = nd.cnv_cur->s * xndc + nd.cnv_cur->tx;
 *yb = nd.cnv_cur->s * yndc + nd.cnv_cur->ty;
}

/* ==========================  ndNdcB2Ndc  ========================= */
/*
% F Converte de coordenadas NDC do n'ivel b'asico no canvas 
    para coordenadas NDC corrente.
*/
static void _ndNdcB2Ndc( double xb, double yb, double *xndc, double *yndc )
{
 *xndc = (xb - nd.cnv_cur->tx) / nd.cnv_cur->s;
 *yndc = (yb - nd.cnv_cur->ty) / nd.cnv_cur->s;
}


/*
** ---------------------------------------------------------------
** Public functions:
*/

/* =========================  ndActivate  ======================= */
/*
%F Registra e/ou ativa um canvas junto ao modulo ND.
   Se for a primeira vez que o canvas for ativado, inicializa
   a estrutura de dados local associada ao canvas.  A partir
   deste momento o ponteiro para canvas 'e usado como um
   identificador para esta estrutura de dados local.
   Inicializa a transforma'c~ao NDC acumulada do canvas.
   Aloca descritor para primeira janela NDC do canvas e 
   p~oe na pilha de janelas NDC.
   Define valores iniciais dos par^ametros de transforma'c~ao da 
   primeira janela NDC em rela'c~ao `a janela NDC b'asica (que 
   sempre ocupa o canvas todo).  
   Ajusta o viewport NDC inicial da primeira janela para ocupar o 
   canvas todo.
*/
int ndActivate( cdCanvas *cd_cv )
{
 CnvDsc   *cnv;

/* Ativa canvas junto ao CD/WD.
 */
 if( wdActivate( cd_cv ) != CD_OK )
  return( CD_ERROR );

/* Verifica se o canvas fornecido j'a est'a registrado 
 * no m'odulo.
 */
 cnv = _ndGetDescr( cd_cv );

 if( cnv == NULL )
 {
  NdcWin   *win;
  int      wr, hr;

/* Aloca descritor para o canvas e coloca na lista de 
 * descritores de canvas.
 */
  cnv = (CnvDsc *)malloc( sizeof( CnvDsc ) );
  if( cnv == NULL )
   return( CD_ERROR );
  cnv->next = nd.cnv_head;
  nd.cnv_head = cnv;
  cnv->cd_cv = cd_cv;

/* Inicializa a transforma'c~ao NDC acumulada do canvas.
 */
  cnv->s = 1.0;
  cnv->tx = cnv->ty = 0.0;

/* Aloca descritor para primeira janela NDC do canvas, 
 * p~oe na pilha de janelas NDC.
 */
  win = (NdcWin *)malloc( sizeof( NdcWin ) );
  if( win == NULL )
   return( CD_ERROR );
  win->prev = NULL;
  cnv->win_stack = win;

/* Ajusta o viewport inicial para o canvas todo.
 */
  cdGetCanvasSize( &wr, &hr, NULL, NULL );
  wdViewport( 0, wr, 0, hr );

/* Define valores iniciais dos par^ametros de transforma'c~ao da 
 * primeira janela NDC em rela'c~ao `a janela NDC b'asica.
 */
  win->ox = 0.0;
  win->oy = 0.0;
  if( wr > hr )
  {
   win->w = 1.0;
   win->h = (double)hr / (double)wr;
  }
  else
  {
   win->w = (double)wr / (double)hr;
   win->h = 1.0;
  }
 }

/* Atualiza ponteiro para descriptor de canvas ativo.
 */
 nd.cnv_cur = cnv;

 return( CD_OK );
}

/* ======================  ndGetCanvasSize  ===================== */
/*
%F Retorna o tamanho do canvas em pixel e em NDC.
   As dimens~oes NDC retornadas correspondem ao n'ivel b'asico 
   de transforma'c~oes NDC do canvas corrente.
*/
void ndGetCanvasSize( int *width, int *height, 
                      double *ndc_width, double *ndc_height )
{
 int    wr, hr;
 double ndc_w, ndc_h;

 if( nd.cnv_cur == NULL )
  return;

 cdGetCanvasSize( &wr, &hr, NULL, NULL );

 if( wr > hr)
 {
  ndc_w = 1.0;
  ndc_h = (double)hr/(double)wr;
 }
 else
 {
  ndc_w = (double)wr/(double)hr;
  ndc_h = 1.0;
 }
 if (width)      *width      = wr;
 if (height)     *height     = hr;
 if (ndc_width)  *ndc_width  = ndc_w;
 if (ndc_height) *ndc_height = ndc_h;
}

/* ===========================  ndReset  ======================== */
/*
%F Reinicializa a estrutura de dados local associada ao canvas 
   corrente de acordo com a 'area NDC fornecida.
   Libera a pilha de janelas NDC corrente, exceto a primeira.
   Ajusta o viewport NDC inicial da primeira janela de tal forma 
   que a 'area NDC fornecida fica ajustada para ocupar todo o 
   espa'co NDC b'asico (que ocupa o canvas todo).  Inicializa a 
   transforma'c~ao NDC acumulada do canvas de acordo com isso.  
   Para ajustar o viewport da primeira janela NDC ao canvas todo, 
   os par^ametros de entrada desta fun'c~ao devem ser 
     ndReset( 0.0, w_ndc, 0.0, h_ndc );
   onde w_ndc e h_ndc s~ao a largura e a altura do espa'co NDC b'asico 
   do canvas obtidos atrave's de 
     ndGetCanvasSize( NULL, NULL, &w_ndc, &h_ndc );
*/
void ndReset( double xmin, double xmax, double ymin, double ymax )
{
 NdcWin   *win;
 double   w_ndc, h_ndc;

 if( nd.cnv_cur == NULL )
  return;

/* Libera a pilha de janelas NDC corrente, exceto a primeira.
 */
 while( nd.cnv_cur->win_stack->prev != NULL )
 {
  win = nd.cnv_cur->win_stack;
  nd.cnv_cur->win_stack = win->prev;
  free( win );
 }

/* Define valores dos par^ametros de transforma'c~ao da 
 * primeira janela NDC em rela'c~ao `a janela NDC b'asica de 
 * forma que a 'area NDC fornecida ocupe o canvas todo.
 */
 ndGetCanvasSize( NULL, NULL, &w_ndc, &h_ndc );
 win = nd.cnv_cur->win_stack;
 win->ox = - (w_ndc * xmin) / (xmax - xmin);
 win->oy = - (h_ndc * ymin) / (ymax - ymin);
 win->w = (w_ndc * w_ndc) / (xmax - xmin);
 win->h = (h_ndc * h_ndc) / (ymax - ymin);

/* Inicializa transforma'c~ao NDC acumulada do canvas de 
 * acordo com a defini'c~ao da primeira janela NDC.
 */
 if( win->w > win->h )
  nd.cnv_cur->s = win->w;
 else
  nd.cnv_cur->s = win->h;
 nd.cnv_cur->tx = win->ox;
 nd.cnv_cur->ty = win->oy;

/* Automaticamente ajusta o viewport em raster do canvas para 
 * a nova janela definida.
 */
 if( win->w > win->h )
  ndViewport( 0.0, 1.0, 0.0, win->h/win->w );
 else
  ndViewport( 0.0, win->w/win->h, 0.0, 1.0 );
}

/* =========================  ndPushNdc  ======================== */
/*
%F Define um novo n'ivel de transforma'c~ao NDC, i.e., acrescenta 
   uma janela NDC na pilha de janelas NDC do canvas corrente.
   Concatena a transforma'c~ao da nova janela com a transforma'c~ao
   NDC acumulada do canvas.
   Automaticamente ajusta o viewport raster do canvas para a nova 
   janela definida.
*/
int ndPushNdc( double xmin, double xmax, double ymin, double ymax )
{
 NdcWin   *win;
 double   ws, s, tx, ty;

 if( nd.cnv_cur == NULL )
  return( CD_ERROR );

 if( (xmax <= xmin) || (ymax <= ymin) )
  return( CD_ERROR );

/* Aloca descritor para nova janela NDC e p~oe na pilha de janelas NDC.
 */
 win = (NdcWin *)malloc( sizeof( NdcWin ) );
 if( win == NULL )
  return( CD_ERROR );
 win->prev = nd.cnv_cur->win_stack;
 nd.cnv_cur->win_stack = win;

/* Define par^ametros de transforma'c~ao da janela NDC anterior 
 * para a nova janela NDC.
 */
 win->ox = xmin;
 win->oy = ymin;
 win->w = xmax - xmin;
 win->h = ymax - ymin;
 if( win->w > win->h )
  ws = win->w;
 else
  ws = win->h;

/* Concatena transforma'c~ao da nova janela com a transforma'c~ao
 * NDC acumulada do canvas.
 */
 s  = nd.cnv_cur->s * ws;
 tx = nd.cnv_cur->s * win->ox + nd.cnv_cur->tx;
 ty = nd.cnv_cur->s * win->oy + nd.cnv_cur->ty;
 nd.cnv_cur->s = s;
 nd.cnv_cur->tx = tx;
 nd.cnv_cur->ty = ty;

/* Automaticamente ajusta o viewport em raster do canvas para 
 * a nova janela definida.
 */
 if( win->w > win->h )
  ndViewport( 0.0, 1.0, 0.0, win->h/win->w );
 else
  ndViewport( 0.0, win->w/win->h, 0.0, 1.0 );

 return( CD_OK );
}

/* =========================  ndPopNdc  ========================= */
/*
%F Retira o 'ultimo n'ivel de transforma'c~ao NDC, i.e., retira  
   a 'ultima janela da pilha de janelas NDC do canvas corrente.
   Evita retirar a primeira janela NDC que foi definida 
   automaticamente na cria'c~ao do descritor do canvas.
   Concatena a transforma'c~ao inversa da janela sendo retirada 
   com a transforma'c~ao NDC acumulada do canvas.
   Automaticamente ajusta o viewport raster do canvas para a 
   janela anterior `a janela sendo retirada.
*/
int ndPopNdc( void )
{
 NdcWin   *win, *prev;
 double   ws, s, tx, ty;

 if( nd.cnv_cur == NULL )
  return( CD_ERROR );

/* Evita retirar a primeira janela NDC que foi definida 
 * automaticamente na cria'c~ao do descritor do canvas.
 * Se n~ao tiver uma janela para retirar,
 * retorna sem nada executar.
 */
 if( nd.cnv_cur->win_stack->prev == NULL )
  return( CD_ERROR );

/* Remove a 'ultima janela definida da pilha de janelas NDC.
 */
 win = nd.cnv_cur->win_stack;
 nd.cnv_cur->win_stack = win->prev;

/* Define par^ametro de escala da transforma'c~ao da janela NDC 
 * anterior para a janela NDC sendo retirada.
 */
 if( win->w > win->h )
  ws = win->w;
 else
  ws = win->h;

/* Concatena transforma'c~ao inversa da janela sendo retirada 
 * com a transforma'c~ao NDC acumulada do canvas.
 */
 s  =   nd.cnv_cur->s / ws;
 tx = -(nd.cnv_cur->s * win->ox) / ws + nd.cnv_cur->tx;
 ty = -(nd.cnv_cur->s * win->oy) / ws + nd.cnv_cur->ty;
 nd.cnv_cur->s = s;
 nd.cnv_cur->tx = tx;
 nd.cnv_cur->ty = ty;

/* Automaticamente ajusta o viewport raster do canvas para a 
 * janela anterior `a janela sendo retirada.
 */
 prev = nd.cnv_cur->win_stack;
 if( prev->w > prev->h )
  ndViewport( 0.0, 1.0, 0.0, prev->h/prev->w );
 else
  ndViewport( 0.0, prev->w/prev->h, 0.0, 1.0 );

 free( win );

 return( CD_OK );
}

/* ========================  ndGetNdcArea  ===================== */
/*
%F Obtem a 'area do espa'co NDC corrente em rela'c~ao ao espa'co NDC 
   anterior.
*/
void ndGetNdcArea( double *xmin, double *xmax, double *ymin, double *ymax )
{
 if( nd.cnv_cur == NULL )
  return;

 *xmin = nd.cnv_cur->win_stack->ox;
 *ymin = nd.cnv_cur->win_stack->oy;
 *xmax = nd.cnv_cur->win_stack->ox + nd.cnv_cur->win_stack->w;
 *ymax = nd.cnv_cur->win_stack->oy + nd.cnv_cur->win_stack->h;
}

/* =========================  ndClipArea  ======================= */
/*
%F Define a 'area de clip em coordenadas NDC corrente.
*/
void ndClipArea( double xmin, double xmax, double  ymin, double ymax )
{
 int    xminr, xmaxr, yminr, ymaxr;

 if( nd.cnv_cur == NULL )
  return;

 ndNdc2Pixel( xmin, ymin, &xminr, &yminr );
 ndNdc2Pixel( xmax, ymax, &xmaxr, &ymaxr );
 cdClipArea( xminr, xmaxr, yminr, ymaxr );
}

/* =======================  ndGetClipArea  ===================== */
/*
%F Obtem a 'area de clipping corrente em NDC e retorna o estado de clipping.
*/
int ndGetClipArea( double *xmin, double *xmax, double *ymin, double *ymax )
{
 int    xminr, xmaxr, yminr, ymaxr;
 int    clip;

 if( nd.cnv_cur == NULL )
  return CD_ERROR;

 clip = cdGetClipArea( &xminr, &xmaxr, &yminr, &ymaxr );
 ndPixel2Ndc( xminr, yminr, xmin, ymin );
 ndPixel2Ndc( xmaxr, ymaxr, xmax, ymax );
 return clip;
}

/* =========================  ndViewport  ======================= */
/*
%F Define novo viewport em NDC corrente no canvas para converter 
   primitivas do tipo wdLine atraves do par "window-viewport".
*/
void ndViewport( double xmin, double xmax, double ymin, double ymax )
{
 int    xminr, xmaxr, yminr, ymaxr;

 if( nd.cnv_cur == NULL )
  return;

 ndNdc2Pixel( xmin, ymin, &xminr, &yminr );
 ndNdc2Pixel( xmax, ymax, &xmaxr, &ymaxr );
 wdViewport( xminr, xmaxr, yminr, ymaxr );
}

/* =======================  ndGetViewport  ===================== */
/*
%F Obt'em o viewport corrente em NDC.
*/
void ndGetViewport( double *xmin, double *xmax, double *ymin, double *ymax )
{
 int    xminr, xmaxr, yminr, ymaxr;

 if( nd.cnv_cur == NULL )
  return;

 wdGetViewport( &xminr, &xmaxr, &yminr, &ymaxr );
 ndPixel2Ndc( xminr, yminr, xmin, ymin );
 ndPixel2Ndc( xmaxr, ymaxr, xmax, ymax );
}

/* ==========================  ndWorld2Ndc  ======================= */
/*
%F Converte de coordenadas do mundo (reais) para coordenadas 
   em NDC corrente (reais) no canvas.
*/
void ndWorld2Ndc( double xw, double yw, double *xndc, double *yndc )
{
 int    xr, yr;

 if( nd.cnv_cur == NULL )
  return;

 wdWorld2Pixel( xw, yw, &xr, &yr );
 ndPixel2Ndc( xr, yr, xndc, yndc );
}

/* ==========================  ndNdc2World  ======================= */
/*
%F Converte de coordenadas em NDC corrente (reais) do canvas para 
   coordenadas do mundo (reais).
*/
void ndNdc2World( double xndc, double yndc, double *xw, double *yw )
{
 int    xr, yr;

 if( nd.cnv_cur == NULL )
  return;

 ndNdc2Pixel( xndc, yndc, &xr, &yr );
 wdPixel2World( xr, yr, xw, yw );
}

/* ==========================  ndNdc2Pixel  ======================= */
/*
%F Converte de NDC corrente para pixel.
*/
void ndNdc2Pixel( double xndc, double yndc, int *xr, int *yr )
{
 if( nd.cnv_cur == NULL )
  return;

 _ndNdc2NdcB( xndc, yndc, &xndc, &yndc );
 _ndNdcB2Pixel( xndc, yndc, xr, yr );
}

/* ==========================  ndPixel2Ndc  ======================= */
/*
%F Converte de pixel para NDC corrente.
*/
void ndPixel2Ndc( int xr, int yr, double *xndc, double *yndc )
{
 if( nd.cnv_cur == NULL )
  return;

 _ndPixel2NdcB( xr, yr, xndc, yndc );
 _ndNdcB2Ndc( *xndc, *yndc, xndc, yndc );
}

