char *RCStrfmirr="$Id: trfmirr.c,v 1.3 2001/01/19 13:29:11 lfm Exp $";
/*
** trfmirr.c - Functions to generate a matrix transformation for
**             mirroring transformations.
**
** Public functions:
** -------------------------------------------------------------------
**
** void TrfMirror( Ihandle *iup_cv, void *cd_cv,
**                 void (*end_action)( Ihandle *iup_cv, int status ), 
**                 void (*put)( Ihandle *iup_cv, void *m, double tol ),
**                 void (*prompt)( Ihandle *iup_cv, char *msg ),
**                 void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                               double tol, int shift, int control ),
**                 void (*echo)( Ihandle *iup_cv, double x, double y,
**                               int status, double mat[2][3] ) )
**
**    iup_cv     - IUP handle of canvas where object is selected     (IN)
**    cd_cv      - CD handle of canvas                               (IN)
**    end_action - function to be called after finishing operation   (IN)
**    put        - ptr. to function to put matrix after collection   (IN)
**    prompt     - ptr. to function to prompt messg. during collect. (IN)
**    snap       - ptr. to function to snap points during collection (IN)
**    echo       - ptr. to func. to echo cursor pos. and transf.mat. (IN)
**
** This function interactively collects from the user a straight
** line on a given canvas for constructing a mirroring transformation 
** matrix.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal transformation collection ==> status = TRF_NORMAL = 1
** (2) Quit transformation collection ==> status = TRF_QUIT = 0
**     This is returned if user quit transformation collection with
**     mouse right button.
** (3) Interupted transf. collection ==> status = TRF_INTERUPTED = -1
**     This is returned when the transformation collection is closed 
**     by something else (not by the mouse interaction).
** 
** -------------------------------------------------------------------
**
** int  TrfKbdMirror( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *m, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put matrix after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a 
** mirror line.
** If the collection is successfull the function returns a true (1)
** status.  Otherwise, it returns a false (0) status.
**
** --------------------------------------------------------------------
**
** Version:      1-000
** 
** Created:    23-Apr-97    Luiz Fernando Martha
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   08-Aug-2002    Marcos C. de Arruda
**      Replaced call to IupScanf by call to corresponding dialog
**      (Dlg) function.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** -------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "pck.h"
#include "t2d.h"
#include "trf.h"
#include "trfmirr.h"
#include "dlgmirr.h"

/*
** -------------------------------------------------------------------
** Local variables and definitions:
*/
/* Mirror data structure:
 */
typedef struct _trfmirrorp
{
 TrfMr  m;
 PckPnt p0, p1;            /* mirror line points            */
 int    fst;               /* if true, first point given    */
 void   *iup_cv;           /* IUP handle to target canvas   */
 void   *cd_cv;            /* cd handle to corresp.canvas   */
 void   (*end_action)( Ihandle *iup_cv, int status );
 void   (*put)( Ihandle *iup_cv, void *m, double tol );
 void   (*prompt)( Ihandle *iup_cv, char *msg );
 void   (*snap)( Ihandle *iup_cv, double *x, double *y, double tol,
                               int shift, int control );
 void   (*echo)( Ihandle *iup_cv, double x, double y, 
                               int status, double mat[2][3] );
} TrfMirrorP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;

static char *prompt_msg1p ={ "Entre primeiro ponto com botao esquerdo do mouse;"                             " Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com ponto final; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "Enter first point with mouse left button; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter end point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };

/*
** ------------------------------------------------------------------
** Local functions
*/

/* ==========================  TrfMirrorDel  ======================== */
 
static void TrfMirrorDel( TrfMirrorP *mirr )
{
 if( mirr == 0L ) return;
 
/*  Release memory used for collection.
 */
 free( mirr );
}

/* ========================  TrfMirrorBuildMat  ===================== */
 
static void TrfMirrorBuildMat( TrfMirrorP *mirr )
{
 mirr->m.p0x = mirr->p0.x;
 mirr->m.p0y = mirr->p0.y;
 mirr->m.p1x = mirr->p1.x;
 mirr->m.p1y = mirr->p1.y;
 T2dIdentity( );
 T2dMirrorVecAbout( mirr->m.p0x, mirr->m.p0y, mirr->m.p1x, mirr->m.p1y );
 T2dGetTransform( mirr->m.mat );
}

/* ========================  TrfMirrorSnap  ========================= */

static void TrfMirrorSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 TrfMirrorP  *mirr = (TrfMirrorP *)pk->data;

/* If shift key is pressed and first mirror line point has been given,
 * snap mirror line to the horizontal axis, to the vertical axis, or
 * the bi-sector lines.
 */
 if( (mirr->fst) && isshift(pk->mousecfg) )
 {
  TrfShiftSnap( mirr->p0.x, mirr->p0.y, x, y );
 }

/* Check to see whether client had registered a snap function and
 * call it passing the given mouse position and parameters.
 */
 if( mirr->snap != 0L)
  (*mirr->snap )( mirr->iup_cv, x, y, tol,
                             isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}

/* ========================  TrfMirrorEcho  ========================= */
 
static void TrfMirrorEcho( PckDesc *pk, double x, double y )
{
 int    status = 0;

/* Restore registered collection spec. from the data field.
 */
 TrfMirrorP  *mirr = (TrfMirrorP *)pk->data;
 
/* If first point was already selected, compute current
 * transformation matrix.
 */
 if( mirr->fst )
 {
  status = 1;
  mirr->p1.x = x;
  mirr->p1.y = y;
  TrfMirrorBuildMat( mirr );
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position and
 * the current transformation matrix.
 */
 if( mirr->echo != 0L )
  (*mirr->echo )( mirr->iup_cv, x, y, status, mirr->m.mat );
}


/*
** ------------------------------------------------------------------
** Public functions
*/

static void TrfMirror1stPt( PckDesc * );
static void TrfMirror2ndPt( PckDesc * );

/* ==========================  TrfMirror  ========================== */

void TrfMirror( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ), 
              void (*put)( Ihandle *iup_cv, void *m, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y,
                            int status, double mat[2][3] ) )
{
 TrfMirrorP  *mirr;
 PckDesc     pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
 }

/* Allocate a mirror collection descriptor
 */
 mirr = (TrfMirrorP *)malloc( sizeof(TrfMirrorP) );

/* Initialize collection
 */
 mirr->fst = 0;
 mirr->iup_cv = iup_cv;
 mirr->cd_cv = cd_cv;
 mirr->end_action = end_action;
 mirr->put = put;
 mirr->prompt = prompt;
 mirr->snap = snap;
 mirr->echo = echo;
 mirr->p0.x = 0.0;
 mirr->p0.y = 0.0;
 mirr->p1.x = 1.0;
 mirr->p1.y = 0.0;
 mirr->m.p0x = 0.0;
 mirr->m.p0y = 0.0;
 mirr->m.p1x = 1.0;
 mirr->m.p1y = 0.0;
/* Initialize as an identity matrix.
 */
 mirr->m.mat[0][0] = mirr->m.mat[1][1] = 1.0;
 mirr->m.mat[0][1] = mirr->m.mat[1][0] = 0.0;
 mirr->m.mat[0][2] = mirr->m.mat[1][2] = 0.0;

/* Setup pick descriptor.  Register the appropriate mirroring 
 * collector function as the callback for pick interaction.
 */
 if( mirr->prompt != 0L )
  (*mirr->prompt )( mirr->iup_cv, prompt_msg1 ) ;
 pk.iup_cv = mirr->iup_cv;
 pk.gra_cv = mirr->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = TrfMirror1stPt;
 pk.btup_callback = 0L;
 pk.echofunc = TrfMirrorEcho;
 pk.snapfunc = TrfMirrorSnap;
 pk.data = mirr;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* -------------------------  TrfMirror1stPt  ---------------------- */

static void TrfMirror1stPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfMirrorP  *mirr = (TrfMirrorP *)pk->data;
 void        *iup_cv = mirr->iup_cv;
 void        (*end_action)( Ihandle *iup_cv, int status ) = mirr->end_action;
 
/* Check to see if collected line is valid (greater then tolerance).
 * If not valid, call end action function and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfMirrorDel( mirr );
  if( end_action != 0L ) (*end_action)( iup_cv, status );
  return;
 }
 
/* Get button down point as first mirror line point.
 */
 mirr->p0.x = pk->p0.x;
 mirr->p0.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * first point using the keyboard.  In this case, prompt the user
 * for point coordinates.
 */
 if( pk->button == 2 )
 {
  float x0, y0;
  x0 = mirr->p0.x;
  y0 = mirr->p0.y;
  if( DlgCaptureFMirror( &x0, &y0 ) == 1 )
  {
   mirr->p0.x = pk->p0.x = x0;
   mirr->p0.y = pk->p0.y = y0;
  }
  else
  {
   TrfMirrorDel( mirr );
   if( end_action != 0L ) (*end_action)( iup_cv, TRF_QUIT );
   return;
  }
 }

/* Set flag to indicate that first mirror line point has been 
 * selected.
 */
 mirr->fst = 1;

/* Change cursor echo for rubber band line (first point
 * is initial point of rubber band line) and get pick from user
 * for end mirror line point.
 */
 if( mirr->prompt != NULL )
  (*mirr->prompt )( iup_cv, prompt_msg2 ) ;
 pk->cursorecho = CrossCursor | RubberBandCursor;
 pk->button = 0;       /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = TrfMirror2ndPt;
 PckSetup( pk );
}

/* -------------------------  TrfMirror2ndPt  ---------------------- */

static void TrfMirror2ndPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfMirrorP  *mirr = (TrfMirrorP *)pk->data;
 void      *iup_cv = mirr->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = mirr->end_action;
 
/* Get button down point as end mirror line point.
 */
 mirr->p1.x = pk->p0.x;
 mirr->p1.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * mirror line using the keyboard. In this case, prompt 
 * the user for line coordinates, build the mirroring data,
 * and return.
 */
 if( (pk->status == PCK_VALID) && (pk->button == 2) )
 {
  float x0, y0, x1, y1;
  x0 = mirr->p0.x;  y0 = mirr->p0.y;
  x1 = mirr->p1.x;  y1 = mirr->p1.y;
  if( DlgCaptureSMirror( &x0, &y0, &x1, &y1 ) == 1 )
  {
   mirr->p0.x = x0;  mirr->p0.y = y0;
   mirr->p1.x = x1;  mirr->p1.y = y1;
   if( LEN(mirr->p0,mirr->p1) >= pk->toler )
   {
    TrfMirrorBuildMat( mirr );
    if( mirr->put != 0L )
     (*mirr->put )( mirr->iup_cv, &mirr->m, pk->toler );
   }
   TrfMirrorDel( mirr );
   if( end_action ) (*end_action)( iup_cv, TRF_NORMAL );
  }
  else
  {
   TrfMirrorDel( mirr );
   if( end_action ) (*end_action)( iup_cv, TRF_QUIT );
  }
  return;
 }

/* Check to see if collected line is valid (greater then tolerance).
 * If not valid, call end action function and return.
 */
 if( (pk->status != PCK_VALID) || (LEN(mirr->p0,mirr->p1) < pk->toler) )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_VALID:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfMirrorDel( mirr );
  if( end_action != 0L ) (*end_action)( iup_cv, status );
  return;
 }
 
/* Call the client function to put the collected transformation matrix,
 * call end action function, and return.
 */
 TrfMirrorBuildMat( mirr );
 if( mirr->put != 0L ) 
  (*mirr->put )( mirr->iup_cv, &mirr->m, pk->toler );
 TrfMirrorDel( mirr );
 if( end_action != 0L ) (*end_action)( iup_cv, TRF_NORMAL );
}


/* ==========================  TrfKbdMirror  ======================= */

static PckPnt   kbdp0, kbdp1;
static double   kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados do espelhamento. " };
static char *kbdprompt_msge = { "Use the keyboard to enter mirror data. " };

int  TrfKbdMirror( Ihandle *iup_cv,
                 void (*put)( Ihandle *iup_cv, void *m, double tol ),
                 void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 TrfMr     mirr;
 float     x0, y0, x1, y1;
 float     toler;
 int       status = TRF_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 x0 = kbdp0.x;
 y0 = kbdp0.y;
 x1 = kbdp1.x;
 y1 = kbdp1.y;
 toler = kbdtol;
 if( DlgCaptureMirror( &x0, &y0, &x1, &y1, &toler ) == 1 )
 {
  mirr.p0x = (double)x0;
  mirr.p0y = (double)y0;
  mirr.p1x = (double)x1;
  mirr.p1y = (double)y1;
  if( (fabs(mirr.p1x - mirr.p0x) > 0.0) || (fabs(mirr.p1y - mirr.p0y) > 0.0) )
  {
   kbdp0.x = mirr.p0x;
   kbdp0.y = mirr.p0y;
   kbdp1.x = mirr.p1x;
   kbdp1.y = mirr.p1y;
   kbdtol = toler;
   T2dIdentity( );
   T2dMirrorVecAbout( mirr.p0x, mirr.p0y, mirr.p1x, mirr.p1y );
   T2dGetTransform( mirr.mat );
   if( put != 0L )
    (*put)( iup_cv, &mirr, kbdtol );
   status = TRF_NORMAL;
  }
 }

 return( status );
}


